package com.bocloud.cos.booter.controller;


import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ServiceCategory;
import com.bocloud.cos.intf.CloudServiceService;
import com.bocloud.cos.intf.ServiceCategoryService;
import com.bocloud.cos.model.BatchServiceFlowBean;
import com.bocloud.cos.model.ServiceFlowBean;
import com.bocloud.cos.params.CloudServiceActionParams;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author tyl
 */
@RestController
@RequestMapping("/v1/cloud/services")
@Api(value = "云服务管理", tags = "云服务管理")
public class CloudServiceController {

    private final CloudServiceService cloudServiceService;
    private final ServiceCategoryService serviceCategoryService;

    public CloudServiceController(CloudServiceService cloudServiceService,
                                  ServiceCategoryService serviceCategoryService) {
        this.cloudServiceService = cloudServiceService;
        this.serviceCategoryService = serviceCategoryService;
    }

    /**
     * 获取服务列表
     *
     * @param pager
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "获取服务列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<CloudService>> list(Pager pager) {
        return cloudServiceService.list(pager);
    }

    /**
     * 获取服务列表condition查询方式
     *
     * @param condition
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "获取服务列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/condition")
    public BocloudResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.condition(condition, context);
    }

    /**
     * 添加云服务
     *
     * @param cloudService
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "添加云服务", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public Result create(@RequestBody CloudService cloudService,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        cloudService.setCreatorId(context.getTarget());
        cloudService.setMenderId(context.getTarget());
        return cloudServiceService.create(cloudService);
    }

    /**
     * 修改服务
     *
     * @param id
     * @param cloudService
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "修改云服务信息", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@RequestBody CloudService cloudService,
                         @PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        cloudService.setId(id);
        return cloudServiceService.modify(cloudService, context.getTarget());
    }

    /**
     * 移除服务
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "移除服务", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.remove(id, context.getTarget());
    }

    /**
     * 查询详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询服务详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<CloudService> detail(@PathVariable(value = Common.ID) Long id) {
        return cloudServiceService.detail(id);
    }

    /**
     * 服务操作
     *
     * @param id           服务id
     * @param actionParams 操作参数
     * @param context      请求对象
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "服务操作", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping(value = "/{id}")
    public Result action(@PathVariable(value = Common.ID) Long id,
                         @RequestBody CloudServiceActionParams actionParams,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        switch (actionParams.getAction()) {
            case UNPUBLISHED:
            case PUBLISHED:
                return cloudServiceService.updateStatus(id, actionParams.getAction().name(), context.getTarget());
            case TEMPLATE:
                return cloudServiceService.updateTemplate(id, actionParams.getTemplateName(), context);
            default:
                return new Result(false, "暂不支持该类操作");
        }
    }


    /**
     * 服务批量操作
     *
     * @param actionParams 操作参数
     * @param context      请求对象
     * @return
     */
    @ApiOperation(tags = {"CMC"}, value = "服务操作", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping
    public Result batchAction(@RequestBody CloudServiceActionParams actionParams,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        switch (actionParams.getAction()) {
            case UNPUBLISHED:
                return cloudServiceService.batchUnpublished(actionParams.getIds(), context.getTarget());
            case PUBLISHED:
                return cloudServiceService.batchPublished(actionParams.getIds(), context.getTarget());
            default:
                return new Result(false, "暂不支持该类操作");
        }
    }

    /**
     * 根据code query
     *
     * @param code
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "根据code查询服务类型", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{code}/categories")
    public BocloudResult<List<ServiceCategory>> listByCode(@PathVariable(value = Common.CODE) String code) {
        return serviceCategoryService.listByCode(code);
    }

    /**
     * 获取租户授权服务的平台类型
     *
     * @param catalogId
     * @return
     */
    @ApiOperation(tags = {"CSC"}, value = "获取租户授权服务的平台类型", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/vendors")
    public BocloudResult<List<String>> vendors(@RequestParam Long catalogId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.vendors(catalogId, context.getTenant());
    }

    /**
     * 查询租户指定平台下的服务
     *
     * @param catalogId
     * @return
     */
    @ApiOperation(tags = {"CSC"}, value = "获取租户指定平台下的服务", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/vendors/{vendor}")
    public BocloudResult<List<CloudService>> vendorService(@PathVariable(value = "vendor") String vendor, @RequestParam Long catalogId, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.vendorService(vendor, catalogId, context.getTenant());
    }


    /**
     * 服务配置流程
     *
     * @param serviceId
     * @param serviceFlowBeans
     * @return
     */
    @ApiOperation(tags = {"CSC"}, value = "服务配置流程", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/{id}/flows")
    public Result configFlow(@PathVariable(value = "id") Long serviceId, @RequestBody List<ServiceFlowBean> serviceFlowBeans, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.configFlow(serviceId, serviceFlowBeans, context);
    }


    /**
     * 服务批量配置流程
     *
     * @param batchServiceFlowBean
     * @return
     */
    @ApiOperation(tags = {"CSC"}, value = "服务批量配置流程", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/flows")
    public Result batchConfigFlow(@RequestBody BatchServiceFlowBean batchServiceFlowBean, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return cloudServiceService.batchConfigFlow(batchServiceFlowBean, context);
    }

}
