package com.bocloud.cos.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.ServiceOrder;
import com.bocloud.cos.enums.ApplyCategory;
import com.bocloud.cos.internal.FmsFlowService;
import com.bocloud.cos.internal.ImsInternalService;
import com.bocloud.cos.intf.ResourceAlterationService;
import com.bocloud.cos.intf.ResourceApplicationService;
import com.bocloud.cos.intf.ResourceExtensionService;
import com.bocloud.cos.intf.ResourceUnsubscribeService;
import com.bocloud.cos.model.*;
import com.bocloud.cos.params.CloneNetworkCardParams;
import com.bocloud.cos.params.CloneParams;
import com.bocloud.cos.repository.CloudServiceRepository;
import com.bocloud.cos.repository.OrderItemRepository;
import com.bocloud.cos.repository.ServiceFlowConfigRepository;
import com.bocloud.cos.repository.ServiceOrderRepository;
import com.bocloud.rpc.interfaces.ims.ImsRpcService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 资源申请入口controller
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月8日
 */
@RestController
@RequestMapping("/v1/resource")
@Api(value = "服务申请管理", tags = "服务申请管理")
public class ResourceApplyController {

    private static final Logger logger = LoggerFactory.getLogger(ResourceApplyController.class);

    private final ResourceApplicationService resourceApplicationService;
    private final ResourceAlterationService resourceAlterationService;
    private final ResourceExtensionService resourceExtensionService;
    private final ResourceUnsubscribeService resourceUnsubscribeService;
    private final ServiceFlowConfigRepository serviceFlowConfigRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final OrderItemRepository orderItemRepository;
    private final ServiceOrderRepository serviceOrderRepository;
    private final ThreadPoolTaskExecutor taskExecutor;
    private final FmsFlowService fmsFlowService;
    private final ImsInternalService imsInternalService;

    @Autowired
    ImsRpcService imsRpcService;

    public ResourceApplyController(ResourceApplicationService resourceApplicationService, ResourceAlterationService resourceAlterationService,
                                   ResourceExtensionService resourceExtensionService, ResourceUnsubscribeService resourceUnsubscribeService,
                                   ServiceFlowConfigRepository serviceFlowConfigRepository, CloudServiceRepository cloudServiceRepository,
                                   OrderItemRepository orderItemRepository, ServiceOrderRepository serviceOrderRepository,
                                   ThreadPoolTaskExecutor taskExecutor, FmsFlowService fmsFlowService,
                                   ImsInternalService imsInternalService) {
        this.resourceApplicationService = resourceApplicationService;
        this.resourceAlterationService = resourceAlterationService;
        this.resourceExtensionService = resourceExtensionService;
        this.resourceUnsubscribeService = resourceUnsubscribeService;
        this.serviceFlowConfigRepository = serviceFlowConfigRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.orderItemRepository = orderItemRepository;
        this.serviceOrderRepository = serviceOrderRepository;
        this.taskExecutor = taskExecutor;
        this.fmsFlowService = fmsFlowService;
        this.imsInternalService = imsInternalService;
    }

    /**
     * 服务申请
     *
     * @param applicationParam
     * @param context
     * @return
     */
    @PostMapping("/application")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "服务申请", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result application(@RequestBody ApplicationParam applicationParam, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceApplicationService.process(applicationParam, context);
    }

    /**
     * 云主机克隆
     *
     * @param params
     * @param context
     * @return
     */
    @PostMapping("/clone")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "云主机克隆", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result clone(@RequestBody CloneParams params,
                        @Value(Common.REQ_CONTEXT) RequestContext context) {
        try {
            OrderItem item = orderItemRepository.queryBySn(params.getInstance());
            Assert.notNull(item, "订单未找到");
            CloudService service = cloudServiceRepository.queryByCode(item.getServiceCode());
            Assert.notNull(service, "该服务不存在或者未发布");
            // 校验并锁定ip
            List<String> ipList = new ArrayList<>();
            List<CloneNetworkCardParams> networkCards = params.getNetworkCards();
            for (CloneNetworkCardParams networkCard : networkCards) {
                if ("Manual".equals(networkCard.getIpPolicy())) {
                    String addresses = networkCard.getAddresses();
                    JSONArray objects = JSONArray.parseArray(addresses);
                    for (int i =0; i < objects.size(); i++) {
                        ipList.add(objects.getString(i));
                    }
                }
            }
            if (CollectionUtils.isNotEmpty(ipList)) {
                imsRpcService.preemptOrFreeIp(ipList, null);
            }
            Long flowId = serviceFlowConfigRepository.getServiceFlowId(service.getId(), ApplyCategory.ResourceComputerClone);
            if (null == flowId) {
                return resourceApplicationService.clone(params, context);
            }
            Map<String, Object> content = getParams(ApplyCategory.ResourceComputerClone.name(), JSONObject.toJSONString(params), "申请审核服务【" + service.getName() + "】使用", context);
            ServiceOrder serviceOrder = serviceOrderRepository.query(item.getOrderId());
            BocloudResult execution = fmsFlowService.startInstanceById(flowId, context, serviceOrder.getProjectId(), content);
            if (execution.isFailed()) {
                return new Result(false, "服务申请使用审核流程启动失败");
            }
            return new Result(true, "服务申请使用审核流程启动成功");
        }catch (Exception e){
            logger.error("克隆虚拟机失败", e);
            return new Result(false, "服务申请使用审核流程启动失败");
        }
    }

    /**
     * 服务变更
     *
     * @param params
     * @param context
     * @return
     */
    @PostMapping("/alteration")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "服务变更", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result alteration(@RequestBody AlterationParam params,
                             @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceAlterationService.process(params, context);
    }

    /**
     * 服务延期
     *
     * @param params
     * @param context
     * @return
     */
    @PostMapping("/extension")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "服务延期", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result extension(@RequestBody ExtensionParam params,
                            @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceExtensionService.process(params, context);
    }

    /**
     * 服务退订
     *
     * @param params
     * @param context
     * @return
     */
    @PostMapping("/cancelation")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "服务退订", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result cancelation(@RequestBody CancelationParam params,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceUnsubscribeService.process(params, context);
    }

    /**
     * 清空回收站
     *
     * @param list
     * @param context
     * @return
     */
    @PostMapping("/cancelation/all")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "清空回收站", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result cancelationAll(@RequestBody List<CancelationParam> list,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        Optional.ofNullable(list).orElse(new ArrayList<>()).stream().forEach(params -> {
            taskExecutor.execute(() -> {
                try {
                    Result result = this.cancelation(params, context);
                    if (result.isFailed()) {
                        logger.error(result.getMessage());
                    }
                } catch (RuntimeException e) {
                    logger.error("cancelation error, " + e.getMessage(), e);
                }
            });
        });
        return new BocloudResult(true, "回收站清空任务已全部下发！");
    }


    /**
     * 服务批量退订
     *
     * @param params
     * @param context
     * @return
     */
    @PostMapping("/batch/cancelation")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "服务批量退订", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result batchCancelation(@RequestBody CancelationParam params,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        for (String instance : params.getInstances()) {
            OrderItem item = orderItemRepository.queryBySn(instance);
            Assert.notNull(item, "订单未找到");
            CancelationParam cancelationParam = new CancelationParam();
            cancelationParam.setCategory(params.getCategory());
            cancelationParam.setInstance(instance);
            cancelationParam.setOldStatus(params.getOldStatus());
            cancelationParam.setService(params.getService());
            cancelationParam.setInstanceName(params.getInstanceName());
            cancelationParam.setContext(context);
            cancelationParam.setPreview(params.getPreview());
            resourceUnsubscribeService.process(cancelationParam, context);
        }
        return new Result(true, "服务实例退订申请已提交");
    }

    /**
     * k8s更新订单状态
     *
     * @param message
     * @return
     */
    @PatchMapping("/orders")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "k8s更新订单状态", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result updateOrder(@RequestBody OrderMessage message) {
        return resourceApplicationService.updateOrder(message);
    }

    /**
     * k8s服务变更
     *
     * @param params
     * @return
     */
    @PostMapping("/k8s/alteration")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "k8s服务变更", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result k8sAlteration(@RequestBody AlterationParam params) {
        return resourceAlterationService.k8sAlteration(params);
    }

    /**
     * 清空回收站
     *
     * @param context
     * @return
     */
    @DeleteMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "清空回收站", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result empty(@Value(Common.REQ_CONTEXT) RequestContext context) {
        return resourceUnsubscribeService.empty(context);
    }

    private Map<String, Object> getParams(String category, String params, String theme, RequestContext context) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("ApplyCategory", category);
        map.put("ApplyParams", params);
        map.put("ApplyTheme", theme);
        map.put("ApplyContext", context);
        return map;
    }

}
