package com.bocloud.cos.booter.controller;


import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ResultTools;
import com.bocloud.cos.entity.ResourceQuota;
import com.bocloud.cos.intf.ResourceQuotaService;
import com.bocloud.cos.params.QuotaActionParams;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 资源配额管理
 *
 * @author tyl
 */
@RestController
@RequestMapping("/v1")
@Api(value = "资源配额管理", tags = "资源配额管理")
public class ResourceQuotaController {

    private final ResourceQuotaService resourceQuotaService;

    public ResourceQuotaController(ResourceQuotaService resourceQuotaService) {
        this.resourceQuotaService = resourceQuotaService;
    }

    /**
     * 删除配额
     *
     * @param id 租户id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除配额", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/quotas")
    public Result delete(@RequestParam(value = Common.ID) Long id, @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return resourceQuotaService.delete(id, requestContext);
    }

    /**
     * 配额操作
     *
     * @param actionParams 操作配额参数
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "批量更新资源配额", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/quotas")
    public Result quotaAction(@RequestBody QuotaActionParams actionParams,
                              @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        if (null != actionParams.getQuotas()) {
            return resourceQuotaService.quotaAction(actionParams.getQuotas(), actionParams.getAction(), actionParams.getTenantId());
        } else {
            return ResultTools.formatErrResult();
        }
    }

    /**
     * 查询配额详情
     *
     * @param condition 查询字典
     * @param code      配额编码
     * @param target    目标id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询配额详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/quotas")
    public BocloudResult getQuota(@RequestParam(value = Common.CONDITION, required = false) String condition,
                                  @RequestParam(value = Common.CODE, required = false) String code,
                                  @RequestParam(value = "target", required = false) Long target) {
        return resourceQuotaService.getQuota(condition, code, target);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询基础配额列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/quotas/metas")
    public BocloudResult<List<ResourceQuota>> getMetaQuota(@Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return resourceQuotaService.getMetaQuota(requestContext);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询租户基础配额列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/tenants/{id}/quotas/metas")
    public BocloudResult<List<ResourceQuota>> getTenantMetaQuota(@PathVariable Long id, @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return resourceQuotaService.getTenantMetaQuota(id, requestContext);
    }

    /**
     * 修改租户配额
     *
     * @param id
     * @param resourceQuotas
     * @param requestContext
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "租户批量配额", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/tenants/{id}/quotas")
    public Result assign(@PathVariable(value = Common.ID) Long id,
                         @RequestBody List<ResourceQuota> resourceQuotas,
                         @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return resourceQuotaService.assign(id, resourceQuotas, requestContext);
    }

    /**
     * 查询租户配额详情
     *
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查询租户配额详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/tenants/{id}/quotas")
    public BocloudResult<List<ResourceQuota>> listQuotas(@PathVariable(value = Common.ID) Long id,
                                                         @RequestParam(value = Common.TYPE, required = false) String type,
                                                         @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return resourceQuotaService.quotas(id, type, requestContext);
    }

    /**
     * 检查是否存在服务配额使用
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC", "CSC"}, value = "检查是否存在配额使用", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/quotas/check")
    public Result checkServiceQuota(@RequestParam(value = Common.ID, required = false) Long id) {
        return resourceQuotaService.checkServiceQuota(id);
    }

}
