package com.bocloud.cos.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.ServiceOrder;
import com.bocloud.cos.intf.ServiceOrderService;
import com.bocloud.cos.model.LineChartResult;
import com.bocloud.cos.model.ServiceOrderInfo;
import com.bocloud.cos.params.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * 订单
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@RestController
@RequestMapping("/v1/orders")
@Api(value = "订单管理", tags = "订单管理")
public class ServiceOrderController {

    private final ServiceOrderService serviceOrderService;
    private final Logger logger = LoggerFactory.getLogger(ServiceOrderController.class);

    public ServiceOrderController(ServiceOrderService serviceOrderService) {
        this.serviceOrderService = serviceOrderService;
    }

    /**
     * 获取服务资源订购数量
     *
     * @param context 请求对象
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "获取服务资源订购数量", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/count")
    public BocloudResult<List<ServiceOrderInfo>> orderCount(
            @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return serviceOrderService.orderCount(context, request.getHeader(Common.TOKEN));
    }

    /**
     * 获取订单列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "获取订单列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping
    public BocloudResult<GridBean<ServiceOrder>> list(Pager pager,
                                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceOrderService.list(pager, context);
    }

    /**
     * 导出订单列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Deprecated
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "导出订单列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @RequestMapping(value = "/export", method = {RequestMethod.GET})
    public void export(Pager pager,
                       @Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request,
                       HttpServletResponse response) {
        List<Param> paramList = pager.getParams();
        try {
            XSSFWorkbook workbook = serviceOrderService.export(paramList, requestContext);
            // 下载表格到浏览器
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/octet-stream");
            String fileName = "订单统计数据" + System.currentTimeMillis() + ".xlsx";
            final String userAgent = request.getHeader("user-agent").toLowerCase();
            if (userAgent.contains("trident") || userAgent.contains("edge")) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
            OutputStream os = response.getOutputStream();
            workbook.write(os);
            os.flush();
            os.close();
        } catch (Exception e) {
            logger.error("export service order error,", e);
        }
    }

    /**
     * 添加订单
     *
     * @param serviceOrder
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "添加订单", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping
    public BocloudResult<ServiceOrder> create(@RequestBody ServiceOrder serviceOrder,
                                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceOrderService.create(serviceOrder, context);
    }

    /**
     * 修改订单
     *
     * @param id
     * @param serviceOrder
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "修改订单", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody ServiceOrder serviceOrder, @Value(Common.REQ_CONTEXT) RequestContext context) {
        serviceOrder.setId(id);
        return serviceOrderService.modify(serviceOrder, context);
    }

    /**
     * 更新订单
     *
     * @param id
     * @param serviceOrder
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "修改订单", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/{id}")
    public Result updateOrder(@PathVariable(value = Common.ID) Long id,
                              @RequestBody ServiceOrder serviceOrder, @Value(Common.REQ_CONTEXT) RequestContext context) {
        serviceOrder.setId(id);
        return serviceOrderService.updateOrder(serviceOrder, context);
    }

    /**
     * 删除订单
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "删除订单", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceOrderService.remove(id, context);
    }

    /**
     * 删除租户相关订单（删除租户时调用）
     *
     * @param tenantId
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "删除订单", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping
    public Result removeByTenantId(@RequestParam(value = "tenantId") Long tenantId,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceOrderService.removeTenantOrder(tenantId, context);
    }

    /**
     * 查看订单详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "查看订单详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/{id}")
    public BocloudResult<ServiceOrder> detail(@PathVariable(value = Common.ID) Long id) {
        return serviceOrderService.detail(id);
    }

    /**
     * 获取订单明细列表
     *
     * @param pager
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "获取订单明细列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/items")
    public BocloudResult<GridBean<OrderItem>> items(Pager pager,
                                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceOrderService.items(pager, context);
    }

    /**
     * 添加订单明细
     *
     * @param item
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "添加订单明细", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/items")
    public BocloudResult<OrderItem> createItem(@RequestBody OrderItem item,
                                               @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceOrderService.createItem(item, context);
    }

    /**
     * 添加变更订单明细
     *
     * @param alterationParams
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "添加变更订单明细", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PostMapping("/items/alteration")
    public Result createAlterationItem(@RequestBody AlterationParams alterationParams,
                                       @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceOrderService.createAlterationItem(alterationParams.getSn(), alterationParams.getSkuId(), context);
    }

    /**
     * 更新订单明细
     *
     * @param itemParams
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "更新订单明细", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/items")
    public Result updateItem(@RequestBody UpdateItemParams itemParams,
                             @Value(Common.REQ_CONTEXT) RequestContext context) {
        logger.info("接收到更新订单明细信息{}", JSONObject.toJSONString(itemParams));
        return serviceOrderService.updateItem(itemParams.getSn(), itemParams.getStatus(), context);
    }

    /**
     * 更新订单明细
     *
     * @param params
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "更新订单明细", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping("/items/elements")
    public Result updateElements(@RequestBody UpdateElementsParams params,
                                 @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceOrderService.updateElements(params.getSn(), params.getElements(), context);
    }

    /**
     * 修改订单明细
     *
     * @param id
     * @param item
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "更新订单明细", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PutMapping("/items/{id}")
    public Result modifyItem(@PathVariable(value = Common.ID) Long id,
                             @RequestBody OrderItem item, @Value(Common.REQ_CONTEXT) RequestContext context) {
        item.setId(id);
        return serviceOrderService.modifyItem(item, context);
    }

    /**
     * 删除订单明细
     *
     * @param id
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "删除订单明细", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @DeleteMapping("/items/{id}")
    public Result removeItem(@PathVariable(value = Common.ID) Long id,
                             @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceOrderService.removeItem(id, context);
    }

    /**
     * 查看订单明细详情
     *
     * @param id
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "查看订单明细详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/items/{id}")
    public BocloudResult<OrderItem> detailItem(@PathVariable(value = Common.ID) Long id) {
        return serviceOrderService.detailItem(id);
    }

    /**
     * 支付订单/取消订单支付
     *
     * @param paidParams
     * @param context
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "支付订单/取消订单支付", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @PatchMapping
    public Result action(@RequestBody OrderPaidParams paidParams,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        if ("pay".equals(paidParams.getAction())) {
            return serviceOrderService.pay(paidParams.getSn(), paidParams.getCouponId(), context);
        } else if ("cancel".equals(paidParams.getAction())) {
            return serviceOrderService.cancel(paidParams.getSn(), context);
        } else {
            return new Result(false, "暂不支持此操作");
        }
    }

    /**
     * 查询订单概览
     *
     * @param code
     * @return
     */
    @ApiOperation(tags = {"CMC",
            "CSC"}, value = "查询订单概览", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    @GetMapping("/stats")
    public BocloudResult<LineChartResult> stats(@RequestParam(value = Common.CODE) String code) {
        return serviceOrderService.stats(code);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "重新分配订单")
    @PutMapping("/redistribution/{id}")
    public BocloudResult<ServiceOrder> redistribute(@PathVariable(value = Common.ID) Long id,
                                                    @RequestBody RedistributionParams params, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceOrderService
                .redistribute(id, params.getTenantId(), params.getProjectId(), params.getUserId(), context);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "订单字典查询")
    @GetMapping("/condition")
    public BocloudResult listByCondition(@RequestParam(value = Common.CONDITION) String condition,
                                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceOrderService.listByCondition(condition, context);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "订单Top5查询")
    @GetMapping("/top5")
    public BocloudResult<LineChartResult> listByCondition(@RequestParam(value = "start", required = false) String start,
                                                          @RequestParam(value = "end", required = false) String end,
                                                          @RequestParam(value = "tenant", required = false) Boolean tenant,
                                                          @RequestParam(value = "zone", required = false) Boolean zone) {
        return serviceOrderService.top5(start, end, tenant, zone);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "根据订单sn查询")
    @GetMapping("/sn")
    public BocloudResult<ServiceOrder> listByCondition(@RequestParam(value = "condition") String condition,
                                                       @RequestParam(value = "sn", required = false) String sn) {
        return serviceOrderService.queryBySn(condition, sn);
    }

    @ApiOperation(tags = {"CMC", "CSC"}, value = "检查是否有正在进行中的订单")
    @GetMapping("/underway/check")
    public BocloudResult underwayCheck(@RequestParam(value = "sn") String sn) {
        String result = serviceOrderService.underwayCheck(sn);
        if (null != result) {
            return new BocloudResult<>(false, result);
        }
        return new BocloudResult<>(true, null);
    }
}
