package com.bocloud.cos.booter.controller;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.intf.ServiceSkuService;
import com.bocloud.cos.params.SkuActionParams;
import com.bocloud.cos.params.SkuParams;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author admin
 */
@RestController
@RequestMapping("/v1/services/skus")
@Api(value = "服务产品管理", tags = "服务产品管理")
public class ServiceSkuController {

    private final ServiceSkuService serviceSkuService;

    public ServiceSkuController(ServiceSkuService serviceSkuService) {
        this.serviceSkuService = serviceSkuService;
    }

    /**
     * 获取服务产品列表
     *
     * @param pager
     * @return
     */
    @GetMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "获取服务产品列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<GridBean<ServiceSku>> list(Pager pager) {
        return serviceSkuService.list(pager);
    }

    /**
     * 查看服务产品详情
     *
     * @param id
     * @return
     */
    @GetMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "查看服务产品详情", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<ServiceSku> detail(@PathVariable(value = Common.ID) Long id) {
        return serviceSkuService.detail(id);
    }

    /**
     * 添加服务产品
     *
     * @param sku
     * @param context
     * @return
     */
    @PostMapping
    @ApiOperation(tags = {"CMC", "CSC"}, value = "添加服务产品", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result create(@RequestBody SkuParams sku,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceSkuService.create(sku, context);
    }

    /**
     * 修改服务产品
     *
     * @param id
     * @param sku
     * @param context
     * @return
     */
    @PutMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "修改服务产品", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result modify(@PathVariable(value = Common.ID) Long id, @RequestBody SkuParams sku, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceSkuService.modify(id, sku, context);
    }

    /**
     * 删除服务产品
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "删除服务产品", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceSkuService.remove(id, context.getTarget());
    }

    /**
     * 批量上下架服务产品
     *
     * @param params
     * @param context
     * @return
     */
    @PatchMapping("/batch")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "批量上下架服务产品", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public Result batchUpdate(@RequestBody SkuActionParams params,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        switch (params.getAction()) {
            case enable:
            case disable:
                return serviceSkuService.updateStatus(params.getIds(), params.getAction().name(), context.getTarget());
            default:
                return new Result(false, "暂不支持该类操作");
        }
    }

    /**
     * 根据规格查询服务产品
     *
     * @param spec
     * @param categoryId
     * @return
     */
    @GetMapping("/getbyspec")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "根据规格查询服务产品", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<ServiceSku> getBySpec(@RequestParam(value = "spec") String spec,
                                               @RequestParam(value = "categoryId") Long categoryId) {
        return serviceSkuService.getBySpec(spec, categoryId);
    }

    /**
     * 通过任务模板获取服务产品
     *
     * @param templateId
     * @return
     */
    @GetMapping(value = "/templates/skus")
    @ApiOperation(tags = {"CMC", "CSC"}, value = "通过任务模板获取服务产品", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<List<ServiceSku>> list(@RequestParam(value = "templateId") Long templateId) {
        return serviceSkuService.listByTemplate(templateId);
    }

    /**
     * 获取服务产品列表
     *
     * @param pager
     * @return
     */
    @GetMapping("/condition")
    @ApiOperation(value = "获取服务产品列表", response = BocloudResult.class, produces = MediaType.APPLICATION_JSON_VALUE)
    public BocloudResult<List<ServiceSku>> listByCondition(Pager pager) {
        return serviceSkuService.list(pager.getParams());
    }

}
