package com.bocloud.cos.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.RequestContext;
import com.bocloud.cos.entity.ResourceQuota;
import com.bocloud.cos.entity.ServiceTenant;
import com.bocloud.cos.entity.TenantAccount;
import com.bocloud.cos.model.NotifyMessage;
import com.bocloud.cos.model.TenantGrantMessage;
import com.bocloud.cos.repository.CloudServiceRepository;
import com.bocloud.cos.repository.ResourceQuotaRepository;
import com.bocloud.cos.repository.ServiceTenantRepository;
import com.bocloud.cos.repository.TenantAccountRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @author tyl
 * @since 2021/3/30 19:45
 */
@Component
public class TenantGrantQuotaListener {
    private static final Logger logger = LoggerFactory.getLogger(TenantGrantQuotaListener.class);
    private final CloudServiceRepository cloudServiceRepository;
    private final ResourceQuotaRepository resourceQuotaRepository;
    private final TenantAccountRepository tenantAccountRepository;
    private static final String TENANT_GRANT_QUEUE = "tenant.grant.queue";

    private static final String NOTICE_QUEUE = "event.notice.queue";
    private final AmqpTemplate amqpTemplate;
    private final ServiceTenantRepository serviceTenantRepository;

    public TenantGrantQuotaListener(CloudServiceRepository cloudServiceRepository, ResourceQuotaRepository resourceQuotaRepository, TenantAccountRepository tenantAccountRepository, AmqpTemplate amqpTemplate,ServiceTenantRepository serviceTenantRepository) {
        this.cloudServiceRepository = cloudServiceRepository;
        this.resourceQuotaRepository = resourceQuotaRepository;
        this.tenantAccountRepository = tenantAccountRepository;
        this.amqpTemplate = amqpTemplate;
        this.serviceTenantRepository = serviceTenantRepository;
    }

    @RabbitListener(admin = "rabbitAdmin", queues = TENANT_GRANT_QUEUE, autoStartup = "true")
    public void onMessage(TenantGrantMessage message) {
        try {
            logger.info("get tenant grant message {}", JSONObject.toJSONString(message));
            Long tenantId = message.getTenantId();
            //授权服务处理
            List<ServiceTenant> serviceTenants = new ArrayList<>();
            //获取授权的服务id
            List<Long> serviceIds = Optional.ofNullable(message.getServiceIds()).orElse(new ArrayList<>());
            for (Long serviceId : serviceIds) {
                ServiceTenant serviceTenant = new ServiceTenant(serviceId, tenantId);
                serviceTenants.add(serviceTenant);
            }
            if (!serviceTenants.isEmpty()) {
                boolean flag = serviceTenantRepository.batchSave(serviceTenants, serviceTenants.size());
                if (!flag) {
                    sendMessage("授权服务失败", "授权服务失败", message.getRequestContext());
                }
            }
            //授权配额处理
            List<ResourceQuota> addQuota = Optional.ofNullable(message.getAddQuota()).orElse(new ArrayList<>());
            if (!addQuota.isEmpty()) {
                boolean flag = resourceQuotaRepository.batchSave(addQuota, addQuota.size());
                if (!flag) {
                    sendMessage("授权配额失败", "授权配额失败", message.getRequestContext());
                }
            }
            //配额更新处理
            List<ResourceQuota> updateQuota = Optional.ofNullable(message.getUpdateQuota()).orElse(new ArrayList<>());
            if (!updateQuota.isEmpty()) {
                boolean flag = resourceQuotaRepository.batchUpdate(updateQuota, updateQuota.size());
                if (!flag) {
                    sendMessage("配额更新失败", "配额更新失败", message.getRequestContext());
                }
            }
            //授信账号处理
            TenantAccount tenantAccount = message.getTenantAccount();
            if (tenantAccount != null) {
                tenantAccountRepository.save(tenantAccount);
            }
        } catch (Exception e) {
            logger.error("service grant message consume error,", e);
            sendMessage("租户授权失败", "租户授权失败：" + e.getMessage(), message.getRequestContext());
        }
    }

    private void sendMessage(String subject, String content, RequestContext requestContext) {
        // 发送站内信
        NotifyMessage notifyMessage = new NotifyMessage();
        notifyMessage.setSubject(subject);
        notifyMessage.setContent(content);
        notifyMessage.setContext(requestContext);
        amqpTemplate.convertAndSend(NOTICE_QUEUE, notifyMessage);
    }
}
