package com.bocloud.cos.booter.scheduler;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.model.Result;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.cos.intf.TenantAccountService;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.internal.ImsInternalService;
import com.bocloud.cos.internal.SmsUserService;
import com.bocloud.cos.utils.IdGenerator;
import com.bocloud.cos.utils.OrderItemUtils;
import com.bocloud.cos.internel.entity.User;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 计费调度器
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月30日
 */
@Component
public class ChargeScheduler {

    private static final Logger logger = LoggerFactory.getLogger(ChargeScheduler.class);
    private final CurrentService currentService;
    private final ServiceOrderRepository serviceOrderRepository;
    private final ServiceBillRepository billRepository;
    private final BillItemRepository itemRepository;
    private final OrderItemRepository orderItemRepository;
    private final TenantAccountService accountService;
    private final ImsInternalService imsInternalService;
    private final SmsUserService smsUserService;
    private final ChargeItemRepository chargeItemRepository;
    private final ThreadPoolTaskExecutor taskExecutor;
    private final LockFactory lockFactory;

    public ChargeScheduler(BillItemRepository itemRepository, CurrentService currentService,
                           ServiceOrderRepository serviceOrderRepository, ServiceBillRepository billRepository,
                           OrderItemRepository orderItemRepository, TenantAccountService accountService,
                           ImsInternalService imsInternalService, SmsUserService smsUserService, ChargeItemRepository chargeItemRepository,
                           ThreadPoolTaskExecutor taskExecutor, LockFactory lockFactory) {
        this.itemRepository = itemRepository;
        this.currentService = currentService;
        this.serviceOrderRepository = serviceOrderRepository;
        this.billRepository = billRepository;
        this.orderItemRepository = orderItemRepository;
        this.accountService = accountService;
        this.imsInternalService = imsInternalService;
        this.smsUserService = smsUserService;
        this.chargeItemRepository = chargeItemRepository;
        this.taskExecutor = taskExecutor;
        this.lockFactory = lockFactory;
    }


//    @PostConstruct
//    public void init() throws Exception{
//        executeCharge();
//    }

    /**
     * 每天跑一次
     *
     * @throws Exception
     */
    @Scheduled(cron = "0 0 0 * * ?")
    protected void executeCharge() throws Exception {
        // sonar问题: format 内部线程不稳定，采用局部变量
        SimpleDateFormat FORMAT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        if (!currentService.isLeader()) {
            return;
        }
        logger.info("【{}】start to batch charge...", FORMAT.format(new Date()));
        List<Future<Result>> futures = new ArrayList<>();
        Map<Long, ServiceOrder> orderMap =
                serviceOrderRepository.orders(ChargeCommon.ChargeMode.PostCharge.name()).stream()
                        .collect(Collectors.toMap(ServiceOrder::getId, Function.identity()));
        List<ChargeItem> chargeItems = chargeItemRepository.listAllLoopCost();
        logger.info("本次跑批订单数：{}", chargeItems.size());
        for (ChargeItem chargeItem : chargeItems) {
            OrderItem orderItem = orderItemRepository.query(chargeItem.getItemId());
            if (orderItem == null) {
                continue;
            }
            ServiceOrder order = orderMap.get(orderItem.getOrderId());
            if (order == null) {
                continue;
            }
            String path = ChargeScheduler.class.getSimpleName() + "_batch_charge";
            try (BocloudACLock lock = lockFactory.getACLock(path)) {
                Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "请求超时");
                // 查找当月账单
                String month = new SimpleDateFormat("yyyy-MM").format(new Date());
                ServiceBill bill = billRepository.getMonthBill(order.getId(), month);
                if (null == bill) {
                    bill =
                            new ServiceBill(IdGenerator.getInstance().nextId(), order.getId(), ChargeCommon.BillStatus.paid,
                                    null, order.getTenantId(), month, new Date());
                    bill.setUserId(order.getUserId());
                    bill.setProjectId(order.getProjectId());
                    bill.setBusinessId(order.getBusinessId());
                    bill.setMoney(BigDecimal.ZERO);
                    billRepository.save(bill);
                }
                final ServiceBill finalBill = bill;
                Future<Result> future =
                        taskExecutor.submit(() -> this.executeOrder(order, chargeItem, orderItem, finalBill));
                futures.add(future);

            } catch (Exception e) {
                logger.error("Get error message:", e);
            }
        }
        for (Future<Result> future : futures) {
            Result result = future.get();
            if (result.isFailed()) {
                logger.error(result.getMessage());
            }
        }
        logger.info("【{}】batch charge process over!", FORMAT.format(new Date()));
    }

    private Result executeOrder(ServiceOrder order, ChargeItem chargeItem, OrderItem item, ServiceBill bill) {
        try {
            // 获取计费开始时间
            Date billStart = getBillStartDate();
            // 获取计费结束时间
            Date billEnd = getBillEndDate();
            Date chargeBegin = chargeItem.getGmtCreate();
            if (null != chargeBegin && chargeBegin.after(billStart)) {
                billStart = chargeBegin;
            }
            BillItem billItem = buildBillItem(bill.getId(), order, item, chargeItem,
                    ((billEnd.getTime() - billStart.getTime()) / 60000) + 1, billStart, billEnd);
            itemRepository.save(billItem);
            Double money = itemRepository.summary(bill.getId());
            BigDecimal finalMoney = BigDecimal.valueOf(0);
            if (null != money) {
                finalMoney = BigDecimal.valueOf(money);
            }
            bill.setMoney(finalMoney);
            bill.setProjectId(order.getProjectId());
            logger.info("bill:"+ JSON.toJSONString(bill));
            billRepository.update(bill);
            RequestContext requestContext =
                    new RequestContext(Catalog.Tenant, order.getTenantId(), order.getTenantId(), order.getProjectId(),
                            null);
            Long tenantId = order.getTenantId();
            if (null != order.getUserId() && 0L != order.getUserId()) {
                BocloudResult<User> userResult = smsUserService.detail(order.getUserId());
                if (userResult.isSuccess() && null != userResult.getData()) {
                    User user = userResult.getData();
                    tenantId = user.getTenantId();
                    requestContext.setTarget(user.getId());
                    requestContext.setTenant(user.getTenantId());
                    requestContext.setCatalog(Catalog.User);
                }
            }
            if (null != tenantId && 0L != tenantId) {
                // 扣款
                Result bocloudResult = accountService
                        .accountAction(tenantId, bill.getId(), TenantAccount.Action.Consume, bill.getMoney(),
                                requestContext);
                if (bocloudResult.isFailed()) {
                    // 如果是云主机，则放入回收站
                    if (StringUtils.hasText(item.getServiceCode()) && ("server".equalsIgnoreCase(
                            item.getServiceCode().substring(item.getServiceCode().lastIndexOf(".") + 1)))) {
                        BocloudResult recycleResult = imsInternalService.recycle(item.getSn());
                        logger.info("云主机放入回收站结果：{}", JSONObject.toJSONString(recycleResult));
                    }
                }
            }
            return new Result(true, "订单跑批成功，订单号：" + order.getSn());
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new Result(false, "订单跑批失败：" + e.getMessage() + "订单号:" + order.getSn());
        }
    }

    private BillItem buildBillItem(Long billId, ServiceOrder order, OrderItem item, ChargeItem chargeItem, Long minutes,
                                   Date billStart, Date billEnd) {
        BigDecimal billMoney =
                chargeItem.getPrice().divide(new BigDecimal(60), 5, RoundingMode.HALF_UP).multiply(new BigDecimal(minutes));
        BillItem billItem = new BillItem();
        billItem.setBillId(billId);
        billItem.setItemId(item.getId());
        billItem.setSkuId(item.getSkuId());
        billItem.setChargeMode(order.getChargeMode());
        billItem.setPrice(billMoney);
        billItem.setBillStart(billStart);
        billItem.setBillEnd(billEnd);
        billItem.setMoney(billMoney);
        billItem.setGmtCreate(new Date());
        billItem.setCatalog(BillItem.Catalog.standard);
        OrderItemUtils.elementNames(item);
        billItem.setResource(JSONObject.toJSONString(item.getElementList()));
        billItem.setUserId(order.getUserId());
        billItem.setTenantId(order.getTenantId());
        billItem.setProjectId(order.getProjectId());
        billItem.setBusinessId(order.getBusinessId());
        billItem.setServiceCode(item.getServiceCode());
        billItem.setInstance(item.getSn());
        return billItem;
    }

    private Date getBillStartDate() {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DATE, -1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);
        return calendar.getTime();
    }

    private Date getBillEndDate() {
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DATE, -1);
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        calendar.set(Calendar.MILLISECOND, 999);
        return calendar.getTime();
    }

}
