package com.bocloud.cos.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.BillItem;
import com.bocloud.cos.enums.Cycle;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 账单明细数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository
public class BillItemRepository extends BasicGenericDao<BillItem, Long> {

    public BillItemRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<BillItem> list(int page, int rows, Map<String, String> sorter, List<Param> params, Long tenantId) {
        String sql = " select a.* from bill_item a left join service_bill b on b.id = a.bill_id where 1 = 1";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (null != tenantId && 0L != tenantId) {
            sql += " b.tenant_id = :tenantId";
            paramMap.put(Common.TENANTID, tenantId);
        }
        return this.list(BillItem.class, sql, paramMap);
    }

    public int count(List<Param> params, Long tenantId) {
        String sql = "select count(1) from bill_item a left join service_bill b on b.id = a.bill_id where 1 = 1";
        sql = QueryBuilder.build(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (null != tenantId && 0L != tenantId) {
            sql += " b.tenant_id = :tenantId";
            paramMap.put(Common.TENANTID, tenantId);
        }
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id) {
        String sql = "delete from bill_item where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.execute(sql, params) > 0;
    }

    public List<BillItem> listByBillId(Long billId) {
        String sql = "select * from bill_item a where a.bill_id = :billId";
        Map<String, Object> params = MapTools.simpleMap("billId", billId);
        return this.list(BillItem.class, sql, params);
    }

    public Boolean removeByBillId(Long billId) {
        String sql = "delete from bill_item where bill_id = :billId";
        Map<String, Object> params = MapTools.simpleMap("billId", billId);
        return this.execute(sql, params) > 0;
    }

    public BillItem query(Long id) {
        String sql = "select * from bill_item a where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<BillItem> list = this.list(BillItem.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    public List<BillItem> getByItemId(Long itemId) {
        String sql = "select * from bill_item a where a.item_id = :itemId";
        Map<String, Object> params = MapTools.simpleMap("itemId", itemId);
        return this.list(BillItem.class, sql, params);
    }

    public List<Map<String, Object>> top5(String start, String end, Boolean tenant, Boolean zone) {
        StringBuilder builder = new StringBuilder("select sum(a.money) total,c.");
        if (null != tenant && tenant) {
            builder.append("name tenant_name");
        } else if (null != zone && zone) {
            builder.append("city");
        }
        builder.append(
                " from bill_item a left join service_bill b on b.id = a.bill_id left join tenant c on c.id = b.tenant_id where 1 = 1");
        Map<String, Object> paramMap = new HashMap<>(8);
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and b.gmt_create >= :start");
            paramMap.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and b.gmt_create <= :end");
            paramMap.put("end", end);
        }
        builder.append(" group by b.tenant_id order by total desc limit 5");
        return this.list(builder.toString(), paramMap);
    }

    public Double summary(Long billId) {
        String sql = "select sum(money) summary from bill_item where bill_id = :billId";
        Map<String, Object> param = MapTools.simpleMap("billId", billId);
        List<Map<String, Object>> list = this.list(sql, param);
        Map<String, Object> resultMap = list.get(0);
        Object result = resultMap.get("summary");
        if (null == result) {
            return 0.0;
        }
        BigDecimal decimal = (BigDecimal) result;
        return decimal.doubleValue();
    }

    public List<BillItem> list(String start, String end, Long billId) {
        StringBuilder builder = new StringBuilder(
                "select a.*,d.name service_name from bill_item a left join service_bill b on b.id = a.bill_id left join service_sku c on c.id = a.sku_id left join cloud_service d on d.id = c.service_id where 1 = 1");
        Map<String, Object> paramMap = new HashMap<>();
        if (null != billId && billId != 0L) {
            builder.append("  and bill_id = :billId");
            paramMap.put("billId", billId);
        }
        if (StringUtils.isNotEmpty(start)) {
            builder.append(" and b.gmt_create >= :start");
            paramMap.put("start", start);
        }
        if (StringUtils.isNotEmpty(end)) {
            builder.append(" and b.gmt_create <= :end");
            paramMap.put("end", end);
        }
        return this.list(BillItem.class, builder.toString(), paramMap);
    }

    public List<Map<String, Object>> productSumary(String start, String end) throws BoCloudDBException {
        String sql = "select sum(a.money) money, cs.name service_name from bill_item a "
                + " left join service_bill sb on sb.id = a.bill_id "
                + " left join cloud_service cs on cs.code = a.service_code "
                + " where sb.status = 'paid' ";
        Map<String, Object> paramMap = new HashMap<>();
        if (null != start) {
            sql += " and a.gmt_create >= :start";
            paramMap.put("start", start);
        }
        if (null != end) {
            sql += " and a.gmt_create <= :end";
            paramMap.put("end", end);
        }
        sql += " GROUP BY cs.name";
        return this.list(sql, paramMap);
    }

    public List<Map<String, Object>> summary(String start, String end, Cycle cycle, Long tenantId, Long projectId) {

        String format = "'%Y-%m-%d'";
        if (Cycle.MONTH.equals(cycle)) {
            format = "'%Y-%m'";
        }

        StringBuilder builder = new StringBuilder(
                "select sum(a.money) money,DATE_FORMAT(a.gmt_create," + format + ") gmt_create,d.name service_name from bill_item a left join service_bill b on b.id = a.bill_id left join service_sku c on c.id = a.sku_id left join cloud_service d on d.id = c.service_id where 1 = 1 and b.status = 'paid' ");
        Map<String, Object> paramMap = new HashMap<>();
        if (null != tenantId && 0L != tenantId) {
            builder.append(" and a.tenant_id = :tenantId");
            paramMap.put("tenantId", tenantId);
        }
        if (null != start) {
            builder.append(" and a.gmt_create >= :start");
            paramMap.put("start", start);
        }
        if (null != end) {
            builder.append(" and a.gmt_create <= :end");
            paramMap.put("end", end);
        }
        if (null != projectId && 0L != projectId) {
            builder.append(" and b.project_id = :projectId");
            paramMap.put("projectId", projectId);
        }
        builder.append(" group by d.name, DATE_FORMAT(a.gmt_create," + format + ")");
        return this.list(builder.toString(), paramMap);
    }

}
