package com.bocloud.cos.repository;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.SimpleBean;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.*;

/**
 * 云服务数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Slf4j
@Repository
public class CloudServiceRepository extends JdbcGenericDao<CloudService, Long> {

    @Autowired
    public CloudServiceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 分页查询服务列表
     *
     * @param page     页码
     * @param rows     行数
     * @param sorter   排序参数
     * @param params   查询参数
     * @param tenantId 租户id
     * @return
     * @throws Exception
     */
    public List<CloudService> list(int page, int rows, Map<String, String> sorter, List<Param> params, Long tenantId) {
        String sql;
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (null == tenantId) {
            sql = "select a.*, i.icon icon, c.name catalog_name,c.has_category has_category from cloud_service a "
                    + " left join icon i on a.icon_id = i.id left join service_catalog c on a.catalog_id = c.id "
                    + " where a.is_deleted = 0 ";
        } else {
            sql = " select a.*, i.icon icon, c.name catalog_name,c.has_category has_category from cloud_service a "
                    + " left join icon i on a.icon_id = i.id left join service_catalog c on a.catalog_id = c.id "
                    + " left join service_tenant t on a.id = t.service_id  "
                    + " where a.is_deleted = 0 and  t.tenant_id =  " + tenantId;
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(CloudService.class, sql, paramMap);
    }

    /**
     * 根据服务目录id查询服务列表
     *
     * @param catalogId 服务目录id
     * @return
     * @throws Exception
     */
    public List<CloudService> listPublishedServicesByCatalog(Long catalogId) throws BoCloudDBException {
        String sql = " select a.*, i.icon icon, c.name catalog_name,c.has_category has_category from cloud_service a "
                + " left join icon i on a.icon_id = i.id left join service_catalog c on a.catalog_id = c.id " +
                " where a.is_deleted = 0 and a.catalog_id = :catalogId and a.status = :status ";
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("catalogId", catalogId);
        paramMap.put("status", CloudService.Status.PUBLISHED.name());
        return this.list(CloudService.class, sql, paramMap);
    }

    public List<CloudService> listByStatus(String status) throws BoCloudDBException {
        String sql = "select a.*,sc.name catalog_name,sc.has_category has_category from cloud_service a left join service_catalog sc on  a.catalog_id=sc.id where a.is_deleted = 0";
        Map<String, Object> paramMap = new HashMap<>();
        if (StringUtils.isNotEmpty(status)) {
            sql += " and a.status = :status";
            paramMap.put("status", status);
        }
        return this.list(CloudService.class, sql, paramMap);
    }

    /**
     * 根据服务目录id查询租户授权的服务
     *
     * @param catalogId 服务目录id
     * @param tenantId  租户id
     * @return
     * @throws Exception
     */
    public List<CloudService> listByCategory(Long catalogId, Long tenantId) throws BoCloudDBException {
        String sql = " SELECT c.*,s.tenant_id, i.icon icon FROM cloud_service c "
                + " left join service_tenant s on c.id = s.service_id " + " left join icon i on c.icon_id = i.id "
                + " where c.catalog_id = :catalogId  and s.tenant_id = :tenantId and c.is_deleted = 0  ";
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("catalogId", catalogId);
        paramMap.put("tenantId", tenantId);
        return this.list(CloudService.class, sql, paramMap);
    }

    /**
     * 查询租户授权的服务
     *
     * @param tenantId 租户id
     * @return
     * @throws Exception
     */
    public List<CloudService> listTenantService(Long tenantId) throws BoCloudDBException {
        String sql = "Select cs.*,sc.name catalog_name, i.icon icon from cloud_service cs left join service_tenant st on cs.id = st.service_id "
                + "left join service_catalog sc on cs.catalog_id = sc.id left join icon i on cs.icon_id = i.id "
                + "where cs.is_deleted = 0 and  st.tenant_id = :tenantId ";
        Map<String, Object> paramsMap = MapTools.simpleMap("tenantId", tenantId);
        return this.list(CloudService.class, sql, paramsMap);
    }

    /**
     * 查询服务目录
     *
     * @param params 查询参数
     * @param sorter 排序参数
     * @return
     * @throws Exception
     */
    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter) throws BoCloudDBException {
        String sql = "select a.* from cloud_service a where a.is_deleted = 0";
        sql = QueryBuilder.build(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        List<CloudService> cloudServices = this.list(CloudService.class, sql, paramMap);
        List<SimpleBean> beans = new ArrayList<>();
        for (CloudService cloudService : cloudServices) {
            SimpleBean bean = new SimpleBean(cloudService.getId(), cloudService.getName());
            beans.add(bean);
        }
        return beans;
    }

    /**
     * 查询服务数量
     *
     * @param params   查询参数
     * @param tenantId 租户id
     * @return
     * @throws Exception
     */
    public int count(List<Param> params, Long tenantId) throws BoCloudDBException {
        String sql;
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        if (null == tenantId) {
            sql = "select count(1) from cloud_service a where a.is_deleted = 0";
        } else {
            sql = "select count(1) from cloud_service a,service_tenant b where a.is_deleted = 0 and a.id = b.service_id and b.tenant_id = :tenantId";
            paramMap.put("tenantId", tenantId);
        }
        sql = QueryBuilder.build(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 删除服务
     *
     * @param id       服务id
     * @param menderId 修改者id
     * @return
     * @throws Exception
     */
    public Boolean remove(Long id, Long menderId) throws BoCloudDBException {
        String sql = "update cloud_service set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 根据查询服务详情
     *
     * @param id 服务id
     * @return
     * @throws Exception
     */
    public CloudService query(Long id) throws BoCloudDBException {
        String sql = "select a.*,i.icon icon, c.name catalog_name,c.has_category has_category from cloud_service a "
                + " left join icon i on a.icon_id = i.id " + " left join service_catalog c on a.catalog_id = c.id "
                + " where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据服务编码查询服务详情
     *
     * @param code 服务编码
     * @return
     * @throws Exception
     */
    public CloudService queryByCode(String code) throws BoCloudDBException {
        String sql = "select a.*,i.icon icon, c.name catalog_name,c.has_category has_category, p.path path from cloud_service a  left join icon i on a.icon_id = i.id "
                + " left join service_catalog c on a.catalog_id = c.id left join permission p on locate(a.code, p.service_codes)  where a.code = :code";
        Map<String, Object> params = MapTools.simpleMap(Common.CODE, code);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据菜单id查询服务数量（menu字段已废弃）
     *
     * @param menuId 菜单id
     * @return
     * @throws Exception
     */
    public int count(Long menuId) throws BoCloudDBException {
        String sql = "select count(1) from cloud_service a where a.is_deleted = 0 and a.menu_id =:menuId";
        Map<String, Object> params = MapTools.simpleMap("menuId", menuId);
        return this.countQuery(sql, params).intValue();
    }

    /**
     * 查询图标是否存在
     *
     * @param iconId 图标id
     * @return
     * @throws Exception
     */
    public boolean isExists(Long iconId) throws BoCloudDBException {
        String sql = "select * from cloud_service a where a.is_deleted = 0 and a.icon_id = :iconId";
        Map<String, Object> params = MapTools.simpleMap("iconId", iconId);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return !list.isEmpty();
    }

    /**
     * 判断服务名称是否存在
     *
     * @param name 服务名称
     * @return
     * @throws Exception
     */
    public boolean existsName(String name) throws BoCloudDBException {
        String sql = "select * from cloud_service a where a.is_deleted = 0 and a.name = :name";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return !list.isEmpty();
    }

    /**
     * 判断服务编码是否存在
     *
     * @param code 服务编码
     * @return
     * @throws Exception
     */
    public boolean existsCode(String code) throws BoCloudDBException {
        String sql = "select * from cloud_service a where a.is_deleted = 0 and a.code = :code";
        Map<String, Object> params = MapTools.simpleMap("code", code);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return !list.isEmpty();
    }

    /**
     * 判断服务目录下产品编码是否存在
     *
     * @param sn        产品编码
     * @param catalogId 服务目录id
     * @return
     * @throws Exception
     */
    public boolean existsSn(String sn, Long catalogId) throws BoCloudDBException {
        String sql = "select * from cloud_service a where a.is_deleted = 0 and a.sn = :sn and a.catalog_id = :catalogId";
        Map<String, Object> params = MapTools.simpleMap("sn", sn);
        params.put("catalogId", catalogId);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return !list.isEmpty();
    }

    /**
     * 查询租户授权的服务列表
     *
     * @param tenantId 租户id
     * @param params   查询参数
     * @return
     * @throws Exception
     */
    public List<CloudService> list(Long tenantId, List<Param> params) throws BoCloudDBException {
        String sql = " SELECT c.*,sc.name catalog_name,sc.has_category has_category FROM	cloud_service c left join service_tenant s on c.id = s.service_id" +
                " left join service_catalog sc on sc.id = c.catalog_id  where c.is_deleted = 0 AND s.tenant_id = :tenantId ";
        sql = QueryBuilder.buildRaw(sql, params, null, null);
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("tenantId", tenantId);
        return this.list(CloudService.class, sql, paramMap);
    }


    /**
     * 解除租户服务授权
     *
     * @param tenantId  租户id
     * @param serviceId 服务id
     * @return
     * @throws Exception
     */
    public boolean deleteTenant(Long tenantId, Long serviceId) throws BoCloudDBException {
        String sql = "delete from service_tenant where 1=1 ";
        Map<String, Object> params = new HashMap<>();
        if (null != tenantId) {
            sql += "and tenant_id = :tenantId ";
            params.put("tenantId", tenantId);
        }
        if (null != serviceId) {
            sql += "and service_id = :serviceId ";
            params.put("serviceId", serviceId);
        }
        // 避免传入两个null而删除所有数据
        if (null == tenantId && null == serviceId) {
            sql += " and 0 = 1 ";
        }
        return super.execute(sql,params) > 0;
    }

    /**
     * 解除租户所有的服务授权
     *
     * @param tenantId 租户id
     * @return
     * @throws Exception
     */
    public boolean deleteByTenant(Long tenantId) throws BoCloudDBException {
        String sql = "delete from service_tenant where tenant_id = :tenantId ";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return super.execute(sql,params) > 0;
    }

    public List<CloudService> listByCode(List<String> codes) throws BoCloudDBException {
        String sql = "select * from cloud_service where code in (:codes)";
        Map<String, Object> params = MapTools.simpleMap("codes", codes);
        return this.list(CloudService.class, sql, params);
    }

    public Integer getMaxId() throws BoCloudDBException {
        String sql = "select max(a.id) from cloud_service a";
        Long count = this.countQuery(sql, null);
        return count == null ? 0 : count.intValue();
    }

    public List<CloudService> queryByCatalog(List<String> codeArray) throws BoCloudDBException {
        String sql = " select a.* from cloud_service a where a.catalog_id in ( select id from service_catalog c where c.code in ( :codeArray ) ) ";
        Map<String, Object> params = MapTools.simpleMap("codeArray", codeArray);
        List<CloudService> list = this.list(CloudService.class, sql, params);
        return list.isEmpty() ? new ArrayList<CloudService>() : list;
    }

    public List<CloudService> listByIds(List<Long> ids) throws BoCloudDBException {
        String sql = "select a.*,b.name catalog_name from cloud_service a left join service_catalog b on a.catalog_id = b.id where a.is_deleted = 0 and a.id in (:ids) ";
        Map<String, Object> paramsMap = MapTools.simpleMap("ids", ids);
        return this.list(CloudService.class, sql, paramsMap);
    }
}
