package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.database.core.intf.impl.BasicGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 订单明细数据访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Repository
public class OrderItemRepository extends BasicGenericDao<OrderItem, Long> {

    public OrderItemRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    public List<OrderItem> list(int page, int rows, Map<String, String> sorter, List<Param> params) {
        String sql = "select a.* from order_item a where 1 = 1 ";
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.list(OrderItem.class, sql, paramMap);
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from order_item a where 1 = 1  ";
        sql = QueryBuilder.build(sql, params, null, "a");
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public Boolean remove(Long id) {
        String sql = "update order_item set is_deleted = 1 where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        return this.execute(sql, params) > 0;
    }

    public List<OrderItem> listByOrderId(Long orderId) {
        String sql =  "select a.*,b.name service_name from order_item a left join cloud_service b on b.code = a.service_code where a.order_id = :orderId";
        Map<String, Object> params = MapTools.simpleMap("orderId", orderId);
        List<OrderItem> list = this.list(OrderItem.class, sql, params);
        if (CollectionUtils.isNotEmpty(list)) {
            list = list.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(OrderItem::getId))), ArrayList::new));
        }
        return list;
    }

    public List<OrderItem> listByOrderId(Long orderId, String serviceCode) {
        String sql = "select a.*,b.name service_name from order_item a left join cloud_service b on b.code = a.service_code where a.order_id = :orderId and a.service_code =:serviceCode";
        Map<String, Object> params = MapTools.simpleMap("orderId", orderId);
        params.put("serviceCode", serviceCode);
        List<OrderItem> list = this.list(OrderItem.class, sql, params);
        if (CollectionUtils.isNotEmpty(list)) {
            list = list.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(OrderItem::getId))), ArrayList::new));
        }
        return list;
    }

    public List<OrderItem> listByOrderId(Long orderId, List<String> statusList) {
        String sql = "select a.* from order_item a where a.order_id = :orderId and a.status in (:statusList)";
        Map<String, Object> params = MapTools.simpleMap("orderId", orderId);
        params.put("statusList", statusList);
        return this.list(OrderItem.class, sql, params);

    }

    public Boolean removeByOrderId(Long orderId) {
        String sql = "update order_item set is_deleted = 1 where order_id = :orderId";
        Map<String, Object> params = MapTools.simpleMap("orderId", orderId);
        return this.execute(sql, params) > 0;
    }

    public Boolean removeByOrderIds(List<Long> orderIds) {
        String sql = "update order_item set is_deleted = 1 where order_id in (:orderIds)";
        Map<String, Object> params = MapTools.simpleMap("orderIds", orderIds);
        return this.execute(sql, params) > 0;
    }

    public OrderItem query(Long id) {
        String sql = "select * from order_item a where a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<OrderItem> list = this.list(OrderItem.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 获取最新有效的订单项
     *
     * @param sn
     * @return
     */
    public OrderItem queryBySn(String sn) {
        String sql = "select a.* from order_item a where a.is_deleted = 0 and a.sn = :sn and a.status != 'Canceld' ORDER BY a.id desc ";
        Map<String, Object> paramMap = MapTools.simpleMap("sn", sn);
        List<OrderItem> list = this.list(OrderItem.class, sql, paramMap);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    /**
     * 查询最后有效项
     *
     * @param sn
     * @return
     */
    public OrderItem queryLastSuccessBySn(String sn) {
        String sql = "select a.* from order_item a where a.is_deleted = 0 and a.sn = :sn  and a.status in (:status) order by a.id desc";
        List<String> statusList = new ArrayList<>();
        statusList.add(OrderItemStatus.OpenedSuccess.getCode());
        statusList.add(OrderItemStatus.ChangeSuccess.getCode());
        statusList.add(OrderItemStatus.ExtensionSuccess.getCode());
        statusList.add(OrderItemStatus.UnsubscribeSuccess.getCode());
        Map<String, Object> paramMap = MapTools.simpleMap("sn", sn);
        paramMap.put("status", statusList);
        List<OrderItem> list = this.list(OrderItem.class, sql, paramMap);
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }

    /**
     * 查询正在进行中的订单项
     *
     * @param sn
     * @return
     */
    public List<OrderItem> queryUnderwayOrders(String sn) {
        StringBuilder sql = new StringBuilder("select * from order_item")
                .append(" where sn in (select sn from order_item where order_id = (select order_id from order_item where sn = :sn order by id limit 1) group by sn)")
                .append(" and is_deleted = 0")
                .append(" and status in (:status)");
        List<String> statusList = new ArrayList<>();
        statusList.add(OrderItemStatus.Opening.getCode());
        statusList.add(OrderItemStatus.Changing.getCode());
        statusList.add(OrderItemStatus.Extensioning.getCode());
        statusList.add(OrderItemStatus.Unsubscribing.getCode());
        statusList.add(OrderItemStatus.Audit.getCode());
        Map<String, Object> paramMap = MapTools.simpleMap("sn", sn);
        paramMap.put("status", statusList);
        return this.list(OrderItem.class, sql.toString(), paramMap);
    }

    public OrderItem queryBySn(String sn, String status) {
        String sql =
                "select a.* from order_item a where a.is_deleted = 0 and a.sn = :sn and  (a.status = :status or a.status = 'Audit') ";
        Map<String, Object> paramMap = MapTools.simpleMap("sn", sn);
        paramMap.put("status", status);
        List<OrderItem> list = this.list(OrderItem.class, sql, paramMap);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 获取软件安装最新磁盘订单项
     *
     * @param sn
     * @return
     */
    public OrderItem queryLastDiskSn(String sn) {
        String sql =
                "select a.* from order_item a where a.service_code = 'vmware.storage.disk' and a.order_id in (select b.order_id from order_item b where b.sn = :sn ) ";
        Map<String, Object> paramMap = MapTools.simpleMap("sn", sn);
        List<OrderItem> list = this.list(OrderItem.class, sql, paramMap);
        if (list.isEmpty()) {
            return null;
        }
        list.sort(Comparator.comparing(OrderItem::getId).reversed());
        return list.get(0);
    }

    /**
     * 获取软件安装磁盘订单项
     *
     * @param sn
     * @return
     */
    public List<OrderItem> queryDiskSn(String sn) {
        String sql =
                "select a.* from order_item a where a.service_code = 'vmware.storage.disk' and a.order_id in (select b.order_id from order_item b where b.sn = :sn ) ";
        Map<String, Object> paramMap = MapTools.simpleMap("sn", sn);
        return this.list(OrderItem.class, sql, paramMap);
    }

    public List<Map<String, Object>> tenantTop5() {
        String sql =
                "select service_code,count(id) total from order_item where status = 'OpenedSuccess' and is_deleted = 0 group by service_code order by total desc limit 5";
        return this.list(sql);
    }

    public List<Map<String, Object>> tenantOrderTop5() {
        String sql =
                "select count(a.id) total,b.name tenant_name from service_order a left join tenant b on b.id = a.tenant_id group by a.tenant_id order by total desc limit 5";
        return this.list(sql);
    }

    public boolean existBySn(String sn) {
        String sql = "select a.id from order_item a where a.is_deleted = 0 and a.sn = :sn ";
        Map<String, Object> paramMap = MapTools.simpleMap("sn", sn);
        List<OrderItem> list = this.list(OrderItem.class, sql, paramMap);
        if (list.isEmpty()) {
            return false;
        }
        return true;
    }

    public List<OrderItem> listAuditItems(Long orderId) {
        String sql =
                "select * from order_item a where a.order_id = :orderId and a.status = '" + OrderItemStatus.Audit.name()
                        + "'";
        Map<String, Object> params = MapTools.simpleMap("orderId", orderId);
        return this.list(OrderItem.class, sql, params);
    }

    public List<OrderItem> resourceCount(List<Long> tenantIds, List<String> codes, Long vendorId) {
        String sql =
                "SELECT a.*,b.tenant_id,b.vendor_id FROM order_item a LEFT JOIN service_order b ON a.order_id = b.id WHERE a.is_deleted = 0 AND a.status IN('OpenedSuccess','ChangeSuccess','Changing','Unsubscribing','ExtensionSuccess','ExtensionFailure','Extensioning') and a.effective_time is NOT NULL ";
        Map<String, Object> paramMap = Maps.newHashMap();
        if (null != codes) {
            sql += " AND a.service_code in(:codes) ";
            paramMap.put("codes", codes);
        }
        if (!tenantIds.isEmpty()) {
            sql += " and b.tenant_id in(:tenantIds) ";
            paramMap.put("tenantIds", tenantIds);
        }
        if (null != vendorId) {
            sql += " and b.vendor_id = :vendorId ";
            paramMap.put("vendorId", vendorId);
        }
        return this.list(OrderItem.class, sql, paramMap);
    }

}
