package com.bocloud.cos.repository;

import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.ShoppingCart;
import com.bocloud.database.core.intf.impl.JdbcGenericDao;
import com.bocloud.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 购物车DAO
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2019/2/18
 */
@Repository
public class ShoppingCartRepository extends JdbcGenericDao<ShoppingCart, Long> {

    public ShoppingCartRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate) {
        super(jdbcTemplate, npJdbcTemplate);
    }

    /**
     * 查询用户购物车列表
     *
     * @param page
     * @param rows
     * @param params
     * @param sorter
     * @param userId
     * @param projectIds
     * @return
     * @throws Exception
     */

    public List<ShoppingCart> list(int page, int rows, List<Param> params, Map<String, String> sorter, Long userId, Long ownerId) {
        String sql = "select a.*,b.name service_name from shopping_cart a left join cloud_service b on b.id = a.service_id where a.is_deleted = 0 and a.creator_id = :userId";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("userId", userId);
        if (null != ownerId) {
            sql += " and  a.owner_id = :ownerId ";
            paramMap.put("ownerId", ownerId);
        }
        sql = QueryBuilder.buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return this.list(ShoppingCart.class, sql, paramMap);
    }

    /**
     * 获取用户购物车数量
     *
     * @param params
     * @param userId
     * @param projectIds
     * @return
     * @throws Exception
     */

    public int count(List<Param> params, Long userId, Long ownerId) {
        String sql = "select count(a.id) from shopping_cart a left join cloud_service b on b.id = a.service_id where a.is_deleted = 0 and a.creator_id = :userId ";
        Map<String, Object> paramMap = QueryBuilder.getParam(params);
        paramMap.put("userId", userId);
        if (null != ownerId) {
            sql += " and  a.owner_id = :ownerId ";
            paramMap.put("ownerId", ownerId);
        }
        sql = QueryBuilder.buildRaw(sql, params, null, "a");
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 删除资源订单
     *
     * @param id
     * @param userId
     * @return
     * @throws Exception
     */

    public boolean remove(Long id, Long userId) {
        String sql = "update shopping_cart set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 清空购物车
     *
     * @param userId
     * @return
     * @throws Exception
     */

    public boolean empty(Long userId, Long ownerId) {
        String sql = "update shopping_cart set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where creator_id = :userId and is_deleted = 0 ";
        Map<String, Object> params = MapTools.simpleMap("userId", userId);
        if (null != ownerId) {
            sql += " and owner_id = :ownerId";
            params.put("ownerId", ownerId);
        }
        params.put("userId", userId);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 查询购物车清单详情
     *
     * @param id
     * @return
     * @throws Exception
     */

    public ShoppingCart query(Long id) {
        String sql = "select * from shopping_cart where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<ShoppingCart> list = this.list(ShoppingCart.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 查询指定人员项目下的购物车订单
     *
     * @param userId
     * @param projectId
     * @return
     */

    public List<ShoppingCart> list(Long userId, Long projectId) {
        String sql = "select * from shopping_cart where creator_id = :userId and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("userId", userId);
        if (null != projectId && 0L != projectId) {
            sql += " and project_id = :projectId";
            params.put("projectId", projectId);
        }
        return this.list(ShoppingCart.class, sql, params);
    }

    /**
     * 判断购物车订单是否已存在
     *
     * @param name
     * @param resourceCategory
     * @return
     * @throws Exception
     */

    public boolean exists(String name, String resourceCategory) {
        String sql = "select * from shopping_cart where is_deleted = 0 and name = :name and resource_category = :resourceCategory";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("resourceCategory", resourceCategory);
        List<ShoppingCart> list = this.list(ShoppingCart.class, sql, params);
        return !list.isEmpty();
    }
}
