package com.bocloud.cos.internal;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudMethod;
import com.bocloud.common.enums.BoCloudService;
import com.bocloud.common.enums.PostDataFormat;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.utils.StringUtil;
import com.bocloud.cos.internel.entity.Favorite;
import com.bocloud.cos.internel.entity.Tenant;
import com.bocloud.service.core.CurrentService;
import com.bocloud.service.core.Service;
import com.bocloud.service.http.core.ServiceFactory;
import com.bocloud.service.http.model.RemoteService;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * soc服务租户内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月4日
 */
@Component
public class SmsTenantService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public SmsTenantService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private static final Service SERVICE = Service.create(BoCloudService.SMS);
    private final Map<String, Object> headers = new HashMap<>(4);

    /**
     * 获取租户详情
     *
     * @param id 租户id
     * @return
     */
    public Tenant detail(Long id) {
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/tenants/" + id, BoCloudMethod.GET, headers, null);
        BocloudResult result = remote.invoke();
        if (result.isSuccess() && result.getData() != null) {
            return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), Tenant.class);
        } else {
            return null;
        }
    }

    /**
     * 分页获取用户列表
     *
     * @param pager 分页对象
     * @return
     */
    public List<Tenant> list(Pager pager) {
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS), "/v1/tenants",
                BoCloudMethod.GET, headers, pager.toMap());
        BocloudResult<?> result = remoteService.invoke();
        return ((JSONObject) result.getData()).getJSONArray(Common.ROWS).toJavaList(Tenant.class);
    }

    /**
     * 根据id集合获取对应用户集合
     *
     * @param ids 用户id集合
     * @return
     */
    public List<Tenant> listByIds(List<Long> ids, String status) {
        List<Param> params = new ArrayList<>();
        if (ids.isEmpty()) {
            ids.add(-1L);
        }
        params.add(new Param(MapTools.simpleMap("id", JSONObject.toJSONString(ids).substring(1, JSONObject.toJSONString(ids).length() - 1)), Sign.IN));
        if (StringUtil.isNotEmpty(status)) {
            params.add(new Param(MapTools.simpleMap("status", status), Sign.EQ));
        }
        return list(new Pager(1, Integer.MAX_VALUE, params, null));
    }


    /**
     * 获取所有一级租户(分页)
     *
     * @param page
     * @param rows
     * @param params
     * @param sorter
     * @return
     */
    public BocloudResult listParentTenant(int page, int rows, List<Param> params, Map<String, String> sorter) {
        Pager pager = new Pager(page, rows, params, sorter);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants",
                BoCloudMethod.GET, headers, pager.toMap());
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult(false, result.getMessage());
        }
        Assert.notNull(result.getData(), "获取租户信息失败");
        return result;
    }

    public BocloudResult<List<Tenant>> listChildren(Long parentId) {
        Map<String, Object> params = Maps.newHashMap();
        params.put("parentId", parentId);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/" + parentId + "/children",
                BoCloudMethod.GET, headers, params);
        BocloudResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new BocloudResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new BocloudResult<>(true, JSONArray.parseArray(data, Tenant.class), "查询成功");
    }

    /**
     * 添加服务收藏
     *
     * @param favorites
     * @return
     */
    public boolean addFavorites(List<Favorite> favorites) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/favorites", BoCloudMethod.POST, headers, favorites);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke().isSuccess();
    }

    /**
     * 取消收藏
     *
     * @param tenantId
     * @param itemId
     * @return
     */
    public boolean cancelFavorite(Long tenantId, Long itemId) {
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        params.put("serviceId", itemId);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/favorites", BoCloudMethod.DELETE, headers, params);
        remoteService.setDataFormat(PostDataFormat.RAW);
        BocloudResult<?> invoke = remoteService.invoke();
        return invoke.isSuccess();
    }

    public boolean changeServiceRole(Long tenantId, List<Long> services, String action) {
        JSONObject params = new JSONObject();
        params.put("services", JSONObject.toJSONString(services));
        params.put("action", action);
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/" + tenantId + "/roles",
                BoCloudMethod.PUT, headers, MapTools.simpleMap(Common.PARAMS, params.toJSONString()));
        return remoteService.invoke().isSuccess();
    }

    public Result update(Tenant tenant) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/" + tenant.getId(),
                BoCloudMethod.PUT, headers, tenant);
        remoteService.setDataFormat(PostDataFormat.RAW);
        return remoteService.invoke();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        this.headers.put("source", currentService.getService().getName());
    }

}
