package com.bocloud.cos.operate;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.ServiceCategory;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.intf.OrderSnService;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.EmptionModel;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.utils.IdGenerator;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @author tyl
 * @since 2021/5/20 18:32
 */
@Slf4j
public abstract class AbstractResourceOperate implements ResourceOperate {

    protected final CloudServiceRepository serviceRepository;
    protected final ServiceSkuRepository skuRepository;
    protected final OrderItemRepository itemRepository;
    protected final ServiceOrderRepository orderRepository;
    protected final ServiceCategoryRepository categoryRepository;
    protected final LockFactory lockFactory;
    protected final OrderSnService orderSnService;

    protected AbstractResourceOperate(CloudServiceRepository serviceRepository, ServiceSkuRepository skuRepository,
                                      OrderItemRepository itemRepository, ServiceOrderRepository orderRepository,
                                      ServiceCategoryRepository categoryRepository, LockFactory lockFactory,
                                      OrderSnService orderSnService) {
        this.serviceRepository = serviceRepository;
        this.skuRepository = skuRepository;
        this.itemRepository = itemRepository;
        this.orderRepository = orderRepository;
        this.categoryRepository = categoryRepository;
        this.lockFactory = lockFactory;
        this.orderSnService = orderSnService;
    }

    @Override
    public List<OrderItem> buildApplyItem(String service, EmptionModel emptionModel, List<ElementModel> elements) {
        List<OrderItem> items = new ArrayList<>();
        DurationModel duration = emptionModel.getDuration();
        Assert.notEmpty(elements, "订购内容不能为空");
        CloudService orderService = serviceRepository.queryByCode(service);
        // 订购数量
        Integer count = emptionModel.getCount();
        //计算资源消耗
        Assert.isTrue(null != count && count > 0, "订购数量不能小于0");
        //处理订单项内容和配额
        for (int i = 0; i < count; i++) {
            synchronized (this) {
                // 集群版资源申请时，附带的资源（磁盘）也要按照集群版得数量进行计费
                Integer elementCount = 1;
                for (ElementModel element : elements) {
                    String serviceCode = element.getServiceCode();
                    CloudService elementService = serviceRepository.queryByCode(serviceCode);
                    ServiceSku sku = skuRepository.query(element.getSkuId());
                    BigDecimal price = null;
                    // 不开启计费则子订单计费为0
                    if (!orderService.getBillable()) {
                        price = BigDecimal.ZERO;
                    } else {
                        price = OrderPriceUtils.getUnitPrice(elementService, sku, element.getSpecs(), duration);
                    }
                    element.setPrice(price);
                    String elementServiceCode = element.getServiceCode();
                    // 构建订单内容 设置服务类型名称
                    if (null != element.getCategoryId() && 0 != element.getCategoryId()) {
                        ServiceCategory serviceCategory = categoryRepository.query(element.getCategoryId());
                        element.setCategory(serviceCategory.getName());
                        // paas资源时会存在一个实例多节点的情况，例如redis集群等
                        JSONObject object = JSONObject.parseObject(serviceCategory.getProps());
                        if (null != object && null != object.getInteger("nodes")) {
                            elementCount = object.getInteger("nodes");
                        }
                        element.setCount(elementCount);
                    } else {
                        element.setCategory(elementService.getName());
                    }

                    OrderItem orderItem = new OrderItem();
                    orderItem.setSkuId(element.getSkuId());
                    String itemSn = this.getItemSn(element.getSkuId());
                    Assert.notNull(itemSn, "生成订购单号失败");
                    orderItem.setSn(itemSn);
                    orderItem.setCatalog(this.getOrderItemCatalog());
                    orderItem.setServiceCode(elementServiceCode);
                    orderItem.setStatus(OrderItemStatus.Opening);
                    orderItem.setDuration(JSONObject.toJSONString(duration));
                    orderItem.setDeleted(false);
                    orderItem.setResources("[]");
                    orderItem.setElements(JSONObject.toJSONString(Lists.newArrayList(element)));
                    orderItem.setBasicPrice(price.multiply(BigDecimal.valueOf(elementCount)));
                    items.add(orderItem);
                }
            }
        }
        return items;
    }

    /**
     * 生成订单号，如果有skuId则按skuId生成，如果没有随机生成
     *
     * @param skuId
     * @return
     */
    public String getItemSn(Long skuId) {
        // 生成订单号，增加分布式锁
        String itemPath = IdGenerator.class.getSimpleName() + "_order_item_sn";
        try (BocloudACLock lock = lockFactory.getACLock(itemPath)) {
            if (null == skuId || 0L == skuId) {
                Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "请求超时");
                return IdGenerator.getInstance().nextId();
            } else {
                BocloudResult<String> result = orderSnService.generate(skuId);
                Assert.isTrue(result.isSuccess(), result.getMessage());
                return result.getData();
            }
        } catch (Exception e) {
            log.error("generate order sn error：{}", e.getMessage(), e);
            throw new RuntimeException("生成订单号生成失败：" + e.getMessage(), e);
        }
    }
}
