package com.bocloud.cos.operate;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.entity.SkuSpec;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.ServicePrice;
import com.bocloud.cos.model.SpecModel;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author tyl
 * @since 2021/5/19 18:19
 */
public class OrderPriceUtils {
    public static BigDecimal getPrice(DurationModel duration, ServiceSku sku) {
        switch (duration.getMode()) {
            case Hour:
                return Optional.ofNullable(sku.getHourPrice()).orElse(BigDecimal.ZERO);
            case Month:
                return Optional.ofNullable(sku.getMonthPrice()).orElse(BigDecimal.ZERO);
            default:
                return Optional.ofNullable(sku.getYearPrice()).orElse(BigDecimal.ZERO);
        }
    }

    public static Map<String, BigDecimal> getPrice(DurationModel duration, List<ServicePrice> servicePrice) {
        Map<String, BigDecimal> result = new HashMap<>(3);
        for (ServicePrice price : servicePrice) {
            String specName = price.getSpecName();
            BigDecimal priceValue;
            switch (duration.getMode()) {
                case Hour:
                    priceValue = price.getHourPrice();
                    break;
                case Month:
                    priceValue = price.getMonthPrice();
                    break;
                default:
                    priceValue = price.getYearPrice();
            }
            result.put(specName, priceValue);
        }
        return result;
    }

    public static BigDecimal getUnitPrice(CloudService cloudService, ServiceSku sku, List<SpecModel> specs,
                                          DurationModel duration) {
        //不开启计费或者公有云的时候 单价为0
        if (!cloudService.getBillable() || cloudService.getIsPublic()) {
            return BigDecimal.ZERO;
        }
        if (sku == null && specs != null) {
            Integer cpu =
                    specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0)).reduce(Integer::sum).orElse(0);
            BigDecimal memory =
                    specs.stream().map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                            .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
            Integer disk =
                    specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0)).reduce(Integer::sum)
                            .orElse(0);
            List<ServicePrice> elementPrice =
                    Optional.ofNullable(JSONArray.parseArray(cloudService.getBasicPrice(), ServicePrice.class))
                            .orElse(new ArrayList<>());
            Map<String, BigDecimal> priceMap = getPrice(duration, elementPrice);
            BigDecimal cpuPrice = Optional.ofNullable(priceMap.get("cpu")).orElse(BigDecimal.ZERO);
            BigDecimal memPrice = Optional.ofNullable(priceMap.get("memory")).orElse(BigDecimal.ZERO);
            BigDecimal diskPrice = Optional.ofNullable(priceMap.get("disk")).orElse(BigDecimal.ZERO);
            return cpuPrice.multiply(BigDecimal.valueOf(cpu)).add(memPrice.multiply(memory))
                    .add(diskPrice.multiply(BigDecimal.valueOf(disk)));
        }
        switch (cloudService.getBillPolicy()) {
            //规格计费
            case spec:
                if (sku == null) {
                    return BigDecimal.ZERO;
                }
                return OrderPriceUtils.getPrice(duration, sku);
            //灵活计费
            case agility:
                List<ServicePrice> elementPrice =
                        Optional.ofNullable(JSONArray.parseArray(cloudService.getBasicPrice(), ServicePrice.class))
                                .orElse(new ArrayList<>());
                Map<String, BigDecimal> priceMap = getPrice(duration, elementPrice);
                assert sku != null;
                List<SkuSpec> skuSpecs = JSONArray.parseArray(sku.getSpec(), SkuSpec.class);
                BigDecimal skuTotal = BigDecimal.ZERO;
                for (SkuSpec spec : skuSpecs) {
                    BigDecimal specPrice =
                            Optional.ofNullable(priceMap.get(spec.getSpecName())).orElse(BigDecimal.ZERO);
                    BigDecimal totalPrice = specPrice.multiply(BigDecimal.valueOf(Long.parseLong(spec.getSpecValue())));
                    skuTotal = skuTotal.add(totalPrice);
                }
                return skuTotal;
            default:
                return BigDecimal.ZERO;
        }
    }
}
