package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.BillAnalysisCondition;
import com.bocloud.cos.entity.BillItem;
import com.bocloud.cos.entity.ServiceBill;
import com.bocloud.cos.entity.ServiceOrder;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.cos.internal.ImsInternalService;
import com.bocloud.cos.internal.SmsTenantService;
import com.bocloud.cos.intf.BillAnalysisService;
import com.bocloud.cos.model.ChargeResultTrend;
import com.bocloud.cos.model.ChargeTrendData;
import com.bocloud.cos.repository.BillItemRepository;
import com.bocloud.cos.repository.BillVmLabelRepository;
import com.bocloud.cos.repository.ServiceBillRepository;
import com.bocloud.cos.repository.ServiceOrderRepository;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.cos.utils.DateUtils;
import com.bocloud.cos.utils.ExcelChartUtil;
import com.bocloud.cos.utils.MathUtils;
import com.bocloud.cos.internel.entity.BillVmLabel;
import com.google.common.collect.Maps;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.groupingBy;

/**
 * 账单收入分析业务类
 *
 * @author tyl
 * @time 2020/6/29 周一 17:02
 */
@Service
public class BillAnalysisServiceImpl implements BillAnalysisService {

    private final ServiceBillRepository serviceBillRepository;
    private final BillItemRepository billItemRepository;
    private final ServiceOrderRepository serviceOrderRepository;
    private final ImsInternalService imsInternalService;
    private final SmsTenantService tenantService;
    private final Logger logger = LoggerFactory.getLogger(BillAnalysisService.class);
    private final BillVmLabelRepository billVmLabelRepository;

    public BillAnalysisServiceImpl(ServiceBillRepository serviceBillRepository, BillItemRepository billItemRepository,
                                   ServiceOrderRepository serviceOrderRepository, ImsInternalService imsInternalService,
                                   SmsTenantService tenantService, BillVmLabelRepository billVmLabelRepository) {
        this.serviceBillRepository = serviceBillRepository;
        this.billItemRepository = billItemRepository;
        this.serviceOrderRepository = serviceOrderRepository;
        this.imsInternalService = imsInternalService;
        this.tenantService = tenantService;
        this.billVmLabelRepository = billVmLabelRepository;
    }

    @Override
    public BocloudResult incomeOverview() {
        try {
            JSONObject result = new JSONObject();
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime todayBegin = LocalDateTime.of(now.getYear(), now.getMonth(), now.getDayOfMonth(), 0, 0, 0, 0);
            LocalDateTime end = todayBegin.plus(1, ChronoUnit.DAYS);
            //获取总收入
            List<ServiceBill> totalBills =
                    serviceBillRepository.listByTenant(null, DateUtil.localDateTimeFormat(now), null, null);
            BigDecimal totalIncome = totalBills.stream()
                    .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            JSONObject totalObject = new JSONObject();
            totalObject.put("name", "总收入(￥)");
            totalObject.put("value", totalIncome.doubleValue());
            result.put("totalNum", totalObject);
            //获取日收入环比 环比增长率=（本期数-上期数）/上期数×100%
            LocalDateTime yesterday = todayBegin.minus(1, ChronoUnit.DAYS);
            List<ServiceBill> todayBills = serviceBillRepository
                    .listByTenant(DateUtil.localDateTimeFormat(todayBegin), DateUtil.localDateTimeFormat(now), null, null);
            BigDecimal todayIncome = todayBills.stream()
                    .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            //获取日订单
            JSONObject dayNum = new JSONObject();
            dayNum.put("name", "日收入(￥)");
            dayNum.put("value", todayIncome.doubleValue());
            dayNum.put("time", DateUtil.localDateFormat(now));
            result.put("dayNum", dayNum);
            List<ServiceBill> lastDayBills = serviceBillRepository
                    .listByTenant(DateUtil.localDateTimeFormat(yesterday), DateUtil.localDateTimeFormat(todayBegin), null,
                            null);
            BigDecimal lastDayIncome = lastDayBills.stream()
                    .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            double dayRate = lastDayIncome.compareTo(BigDecimal.ZERO) == 0 ? 0.00 :
                    (todayIncome.subtract(lastDayIncome)).divide(lastDayIncome, RoundingMode.HALF_DOWN).doubleValue();
            JSONObject dayIncomeRate = new JSONObject();
            dayIncomeRate.put("name", "日收入环比");
            dayIncomeRate.put("value", MathUtils.round(dayRate, 2));
            dayIncomeRate.put("time", DateUtil.localDateFormat(now));
            result.put("dayRate", dayIncomeRate);
            LocalDateTime weekBegin = end.minus(1, ChronoUnit.WEEKS);
            LocalDateTime lastWeekBegin = weekBegin.minus(1, ChronoUnit.WEEKS);
            List<ServiceBill> weekBills = serviceBillRepository
                    .listByTenant(DateUtil.localDateTimeFormat(weekBegin), DateUtil.localDateTimeFormat(now), null, null);
            BigDecimal weekIncome = weekBills.stream()
                    .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            List<ServiceBill> lastWeekBills = serviceBillRepository
                    .listByTenant(DateUtil.localDateTimeFormat(lastWeekBegin), DateUtil.localDateTimeFormat(weekBegin),
                            null, null);
            BigDecimal lastWeekIncome = lastWeekBills.stream()
                    .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            double weekRate = lastWeekIncome.compareTo(BigDecimal.ZERO) == 0 ? 0.00 :
                    (weekIncome.subtract(lastWeekIncome)).divide(lastWeekIncome, RoundingMode.HALF_DOWN).doubleValue();
            JSONObject weekRateRateObject = new JSONObject();
            weekRateRateObject.put("name", "周收入环比");
            weekRateRateObject.put("value", MathUtils.round(weekRate, 2));
            weekRateRateObject.put("time", DateUtil.localDateFormat(weekBegin) + "-" + DateUtil.localDateFormat(now));
            result.put("weekRate", weekRateRateObject);
            return new BocloudResult(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("get order analysis overview error,", e);
            return BocloudResult.FAILED("查询失败！");
        }
    }

    @Override
    public BocloudResult billsAnalysis(Long vendorId, String time, String serviceCode, String condition) {
        Map<String, String> timeMap = DateUtil.timeToMap(time);
        if ("getTypeCount".equals(condition)) {
            return incomeTypeCount(timeMap.get("start"), timeMap.get("end"), vendorId);
        }
        if ("getTrend".equals(condition)) {
            List<String> timeList = DateUtil.timeToList(new JSONObject(MapTools.simpleMap("time", time)));
            if (timeList.size() <= 1) {
                return BocloudResult.FAILED("时间区间选择太短，请重新选择");
            }
            return incomeTrend(timeList, vendorId);
        }
        if ("getVendorCount".equals(condition)) {
            return vendorServiceIncome(timeMap.get("start"), timeMap.get("end"), serviceCode, vendorId);
        }
        if ("getTenantTop5".equals(condition)) {
            return tenantIncomeTop5(timeMap.get("start"), timeMap.get("end"), vendorId);
        }
        if ("getServiceCount".equals(condition)) {
            return serviceIncome(timeMap.get("start"), timeMap.get("end"), vendorId);
        }
        return BocloudResult.FAILED("不支持的查询类型");
    }

    private BocloudResult incomeTypeCount(String start, String end, Long vendorId) {
        JSONArray result = new JSONArray();
        try {
            Map<ChargeCommon.ChargeMode, List<ServiceOrder>> serviceOrders =
                    serviceOrderRepository.listByVendor(null, null, vendorId, null).stream()
                            .collect(groupingBy(ServiceOrder::getChargeMode));
            List<ServiceBill> allServiceBills = serviceBillRepository.listByOrder(start, end, null, null);
            for (Map.Entry<ChargeCommon.ChargeMode, List<ServiceOrder>> entry : serviceOrders.entrySet()) {
                JSONObject jsonObject = new JSONObject();
                List<ServiceOrder> orders = entry.getValue();
                BigDecimal totalIncome = BigDecimal.ZERO;
                for (ServiceOrder order : orders) {
                    BigDecimal income =
                            allServiceBills.stream().filter(serviceBill -> order.getId().equals(serviceBill.getOrderId()))
                                    .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                                    .reduce(BigDecimal.ZERO, BigDecimal::add);
                    totalIncome = totalIncome.add(income);
                }
                if (entry.getKey().equals(ChargeCommon.ChargeMode.PreCharge)) {
                    jsonObject.put("name", "预付费");
                } else {
                    jsonObject.put("name", "后付费");
                }
                jsonObject.put("value", totalIncome);
                result.add(jsonObject);
            }
            return new BocloudResult(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("get income type count error", e);
            return new BocloudResult(false, result, "查询失败");
        }
    }

    private BocloudResult incomeTrend(List<String> timeList, Long vendorId) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            HashMap<String, List<Object>> countMap = new HashMap<>(8);
            List<Object> total = new ArrayList<>();
            Map<String, List<ServiceBill>> serviceBills = serviceBillRepository
                    .listByTenant(timeList.get(0).substring(0, 10) + " 00:00:00",
                            timeList.get(timeList.size() - 1).substring(0, 10) + " 23:59:59", null, null).stream()
                    .filter(serviceBill -> {
                        if (null != vendorId && 0L != vendorId) {
                            return serviceBill.getVendorId().equals(vendorId);
                        }
                        return true;
                    }).peek(serviceBill -> {
                        if (serviceBill.getVendorId() == null) {
                            serviceBill.setVendorName("其他");
                        }
                    }).collect(groupingBy(ServiceBill::getVendorName));
            HashMap<String, List<Long>> billsMap = new HashMap<>(8);
            for (Map.Entry<String, List<ServiceBill>> entry : serviceBills.entrySet()) {
                String vendorName = entry.getKey();
                List<Long> BillsIds = entry.getValue().stream().map(ServiceBill::getId).collect(Collectors.toList());
                billsMap.put(vendorName, BillsIds);
            }
            List<BillItem> billItems = billItemRepository.list(timeList.get(0).substring(0, 10) + " 00:00:00",
                    timeList.get(timeList.size() - 1).substring(0, 10) + " 23:59:59", null);
            keys.addAll(timeList);
            for (String time : timeList) {
                BigDecimal totalIncome = BigDecimal.ZERO;
                for (Map.Entry<String, List<Long>> entry : billsMap.entrySet()) {
                    String vendorName = entry.getKey();
                    List<Long> vendorBills = entry.getValue();
                    List<BillItem> billsList = billItems.stream().filter(
                            billItem -> vendorBills.contains(billItem.getBillId()) && billItem.getGmtCreate()
                                    .before(DateUtils.stringToDate(DateUtil.timeProcess(time), "yyyy-MM-dd HH:mm:ss")))
                            .collect(Collectors.toList());
                    BigDecimal income = billsList.stream()
                            .map(billItem -> Optional.ofNullable(billItem.getMoney()).orElse(BigDecimal.ZERO))
                            .reduce(BigDecimal.ZERO, BigDecimal::add);
                    totalIncome = totalIncome.add(income);
                    //统计过了则更新 数据列表 没有统计的则新增数据列表
                    if (countMap.containsKey(vendorName)) {
                        List<Object> countNum = countMap.get(vendorName);
                        countNum.add(income);
                        countMap.put(vendorName, countNum);
                    } else {
                        ArrayList<Object> countNum = new ArrayList<>();
                        countNum.add(income);
                        countMap.put(vendorName, countNum);
                    }
                }
                total.add(totalIncome);
            }
            for (Map.Entry<String, List<Object>> entry : countMap.entrySet()) {
                ChargeTrendData chargeTrendData = new ChargeTrendData(entry.getKey(), entry.getValue());
                values.add(chargeTrendData);
            }
            values.add(new ChargeTrendData("合计(￥)", total));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get income trend error", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    private BocloudResult vendorServiceIncome(String start, String end, String serviceCode, Long vendorId) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            ArrayList<Object> data = new ArrayList<>();
            Map<String, List<ServiceBill>> serviceBills =
                    serviceBillRepository.listByCode(start, end, serviceCode, vendorId).stream().peek(serviceBill -> {
                        if (serviceBill.getVendorName() == null) {
                            serviceBill.setVendorName("其他");
                        }
                    }).collect(groupingBy(ServiceBill::getVendorName));
            for (Map.Entry<String, List<ServiceBill>> entry : serviceBills.entrySet()) {
                BigDecimal income = entry.getValue().stream()
                        .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                keys.add(entry.getKey());
                data.add(income);
            }
            values.add(new ChargeTrendData("收入(￥)", data));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get vendor service income error", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    private BocloudResult tenantIncomeTop5(String start, String end, Long vendorId) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            HashMap<String, BigDecimal> sortedMap = new HashMap<>(8);
            List<ServiceBill> serviceBills = serviceBillRepository.listByTenant(start, end, null, vendorId).stream()
                    .filter(serviceBill -> serviceBill.getTenantName() != null).collect(Collectors.toList());
            Map<String, List<ServiceBill>> tenantBillMap =
                    serviceBills.stream().collect(groupingBy(ServiceBill::getUserName));
            for (Map.Entry<String, List<ServiceBill>> tenantEntry : tenantBillMap.entrySet()) {
                String tenantName = tenantEntry.getKey();
                List<ServiceBill> tenantServiceBills = tenantEntry.getValue();
                BigDecimal income = tenantServiceBills.stream()
                        .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                sortedMap.put(tenantName, income);
            }
            List<Map.Entry<String, BigDecimal>> sortedResult =
                    sortedMap.entrySet().stream().sorted(((o1, o2) -> o2.getValue().compareTo(o1.getValue()))).limit(5)
                            .collect(Collectors.toList());
            ArrayList<Object> data = new ArrayList<>();
            for (Map.Entry<String, BigDecimal> entry : sortedResult) {
                keys.add(entry.getKey());
                data.add(entry.getValue());
            }
            values.add(new ChargeTrendData("收入(￥)", data));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get vendor tenant service income error", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    private BocloudResult serviceIncome(String start, String end, Long vendorId) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            ArrayList<Object> data = new ArrayList<>();
            Map<String, List<ServiceBill>> serviceBills =
                    serviceBillRepository.listByTenant(start, end, null, vendorId).stream()
                            .collect(groupingBy(ServiceBill::getServiceName));
            Set<Map.Entry<String, List<ServiceBill>>> tenantBillsEntry = serviceBills.entrySet();
            for (Map.Entry<String, List<ServiceBill>> entry : tenantBillsEntry) {
                BigDecimal income = entry.getValue().stream()
                        .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                keys.add(entry.getKey());
                data.add(income);
            }
            values.add(new ChargeTrendData("收入(￥)", data));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get vendor service income error", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    @Override
    public BocloudResult billsAnalysisNew(String time, String condition, String label) {
        try {
            JSONArray jsonValues = null;
            ChargeResultTrend chargeResultTrend = null;
            List<String> timeList = DateUtil.timeToList(JSONObject.parseObject(time, JSONObject.class));
            if (timeList.size() <= 1) {
                return BocloudResult.FAILED("时间区间选择太短，请重新选择");
            }
            if (BillAnalysisCondition.LABEL.getCondition().equals(condition)) {
                if (StringUtils.isEmpty(label)) {
                    return BocloudResult.FAILED("请选择标签信息");
                }
                jsonValues = labelServiceIncomeNew(timeList.get(0), timeList.get(timeList.size() - 1), label);
                chargeResultTrend = labelincomeTrendNew(timeList, label);
            } else {
                jsonValues = vendorServiceIncomeNew(timeList.get(0), timeList.get(timeList.size() - 1), condition);
                chargeResultTrend = incomeTrendNew(timeList, condition);
            }
            List<Object> res = new ArrayList<>();
            res.add(jsonValues);
            res.add(chargeResultTrend);
            return new BocloudResult<>(true, res, "账单分析查询成功");
        } catch (Exception e) {
            logger.error("get bills vendor analysis error", e.getMessage());
            return new BocloudResult(false, "账单分析查询失败");
        }

    }

    /**
     * @param label
     * @return com.bocloud.cos.model.ChargeResultTrend
     * @Author: lxy
     * @Description: 针对标签单独使用
     * @Date 2021/5/29 17:39
     * @Param timeList
     * @throws:
     **/
    private ChargeResultTrend labelincomeTrendNew(List<String> timeList, String label) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        HashMap<String, List<Object>> countMap = new HashMap<>(8);
        List<Object> total = new ArrayList<>();
        String start = timeList.get(0);
        String end = timeList.get(timeList.size() - 1);
        start = formatTime(start, " 00:00:00", ":00");
        end = formatTime(end, " 23:59:59", ":59");
        List<BillVmLabel> serviceBillList = billVmLabelRepository.listByLabelType(start, end, label);
        if (ListTool.isEmpty(serviceBillList)) {
            return new ChargeResultTrend(keys, values);
        }
        Map<String, List<BillVmLabel>> serviceBills = serviceBillList.stream().peek(serviceBill -> {
            if (serviceBill.getLabelValue() == null) {
                serviceBill.setLabelValue("其他");
            }
        }).collect(groupingBy(BillVmLabel::getLabelValue));

        if (serviceBills.size() == 0) {
            return new ChargeResultTrend(keys, values);
        }

        keys.addAll(timeList);
        for (String time : timeList) {
            BigDecimal totalIncome = BigDecimal.ZERO;
            for (Map.Entry<String, List<BillVmLabel>> entry : serviceBills.entrySet()) {
                String vendorName = entry.getKey();
                List<BillVmLabel> vendorBills = entry.getValue();
                time = formatTime(time, " 23:59:59", ":59");
                String finalTime = time;
                List<BillVmLabel> billsList = vendorBills.stream().filter(billItem -> !billItem.getGmtCreate()
                        .after(DateUtils.stringToDate(finalTime, "yyyy-MM-dd HH:mm:ss"))).collect(Collectors.toList());
                BigDecimal income =
                        billsList.stream().map(billItem -> Optional.ofNullable(billItem.getMoney()).orElse(BigDecimal.ZERO))
                                .reduce(BigDecimal.ZERO, BigDecimal::add);
                totalIncome = totalIncome.add(income);
                //统计过了则更新 数据列表 没有统计的则新增数据列表
                if (countMap.containsKey(vendorName)) {
                    List<Object> countNum = countMap.get(vendorName);
                    countNum.add(MathUtils.round(income.doubleValue(), 2));
                    countMap.put(vendorName, countNum);
                } else {
                    ArrayList<Object> countNum = new ArrayList<>();
                    countNum.add(MathUtils.round(income.doubleValue(), 2));
                    countMap.put(vendorName, countNum);
                }
            }
            total.add(MathUtils.round(totalIncome.doubleValue(), 2));
        }
        for (Map.Entry<String, List<Object>> entry : countMap.entrySet()) {
            ChargeTrendData chargeTrendData = new ChargeTrendData(entry.getKey(), entry.getValue());
            values.add(chargeTrendData);
        }
        values.add(new ChargeTrendData("合计(￥)", total));
        return new ChargeResultTrend(keys, values);
    }

    /**
     * @param start
     * @return com.alibaba.fastjson.JSONArray
     * @Author: lxy
     * @Description: 针对标签单独使用
     * @Date 2021/5/29 15:47
     * @Param end
     * @throws:
     **/
    private JSONArray labelServiceIncomeNew(String start, String end, String label) {
        JSONArray jsonArray = new JSONArray();
        start = formatTime(start, " 00:00:00", ":00");
        end = formatTime(end, " 23:59:59", ":59");
        List<BillVmLabel> billVmLabels = billVmLabelRepository.listByLabelType(start, end, label);
        if (ListTool.isEmpty(billVmLabels)) {
            return jsonArray;
        }
        Map<String, List<BillVmLabel>> serviceBills = billVmLabels.stream().peek(serviceBill -> {
            if (serviceBill.getLabelValue() == null) {
                serviceBill.setLabelValue("其他");
            }
        }).collect(groupingBy(BillVmLabel::getLabelValue));
        if (serviceBills.size() == 0) {
            return jsonArray;
        }
        for (Map.Entry<String, List<BillVmLabel>> entry : serviceBills.entrySet()) {
            JSONObject jsonObject = new JSONObject();
            BigDecimal income = entry.getValue().stream()
                    .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            jsonObject.put("name", entry.getKey());
            jsonObject.put("value", MathUtils.round(income.doubleValue(), 2));
            jsonArray.add(jsonObject);
        }
        return jsonArray;
    }

    private JSONArray vendorServiceIncomeNew(String start, String end, String condition) throws Exception {
        JSONArray jsonArray = new JSONArray();
        Map<String, List<ServiceBill>> serviceBills = new HashMap<>(8);
        start = formatTime(start, " 00:00:00", ":00");
        end = formatTime(end, " 23:59:59", ":59");
        List<ServiceBill> serviceBillList = serviceBillRepository.listByCode(start, end, null, null);
        if (ListTool.isEmpty(serviceBillList)) {
            return jsonArray;
        }
        //平台
        if (BillAnalysisCondition.VENDOR.getCondition().equals(condition)) {
            serviceBills = serviceBillList.stream().peek(serviceBill -> {
                if (serviceBill.getVendorName() == null) {
                    serviceBill.setVendorName("其他");
                }
            }).collect(groupingBy(ServiceBill::getVendorName));
        } else if (BillAnalysisCondition.TENANT.getCondition().equals(condition)) { //租户
            serviceBills = serviceBillList.stream().peek(serviceBill -> {
                if (serviceBill.getTenantName() == null) {
                    serviceBill.setTenantName("其他");
                }
            }).collect(groupingBy(ServiceBill::getTenantName));
        } else if (BillAnalysisCondition.BUSINESS.getCondition().equals(condition)) { //业务
            serviceBills = serviceBillList.stream().peek(serviceBill -> {
                if (serviceBill.getBusinessName() == null) {
                    serviceBill.setBusinessName("其他");
                }
            }).collect(groupingBy(ServiceBill::getBusinessName));
        } else if (BillAnalysisCondition.PROJECT.getCondition().equals(condition)) { //项目
            serviceBills = serviceBillList.stream().peek(serviceBill -> {
                if (serviceBill.getProjectName() == null) {
                    serviceBill.setProjectName("其他");
                }
            }).collect(groupingBy(ServiceBill::getProjectName));
        } else {
            throw new Exception("查询失败，请检查传入 condition 类型是否匹配，传入为：" + condition);
        }
        if (serviceBills.size() == 0) {
            return jsonArray;
        }
        for (Map.Entry<String, List<ServiceBill>> entry : serviceBills.entrySet()) {
            JSONObject jsonObject = new JSONObject();
            BigDecimal income = entry.getValue().stream()
                    .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            jsonObject.put("name", entry.getKey());
            jsonObject.put("value", MathUtils.round(income.doubleValue(), 2));
            jsonArray.add(jsonObject);
        }
        return jsonArray;

    }

    /**
     * @param s
     * @param s2
     * @return java.lang.String
     * @Author: lxy
     * @Description: 格式化时间
     * @Date 2021/7/22 18:28
     * @Param time
     * @throws:
     **/
    private String formatTime(String time, String s, String s2) {
        if (time.length() == 10) {
            time += s;
        } else if (time.length() == 16) {
            time += s2;
        }
        return time;
    }

    private ChargeResultTrend incomeTrendNew(List<String> timeList, String condition) throws Exception {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        HashMap<String, List<Object>> countMap = new HashMap<>(8);
        List<Object> total = new ArrayList<>();
        String start = timeList.get(0);
        String end = timeList.get(timeList.size() - 1);
        start = formatTime(start, " 00:00:00", ":00");
        end = formatTime(end, " 23:59:59", ":59");

        List<ServiceBill> serviceBillList = serviceBillRepository.listByTenant(start, end, null, null);
        if (ListTool.isEmpty(serviceBillList)) {
            return new ChargeResultTrend(keys, values);
        }
        Map<String, List<ServiceBill>> serviceBills = new HashMap<>(8);
        Map<Long, Date> serviceBillGmtDate =
                serviceBillList.stream().collect(Collectors.toMap(ServiceBill::getId, ServiceBill::getGmtCreate));

        //平台
        if (BillAnalysisCondition.VENDOR.getCondition().equals(condition)) {
            serviceBills = serviceBillList.stream().peek(serviceBill -> {
                if (serviceBill.getVendorId() == null) {
                    serviceBill.setVendorName("其他");
                }
            }).collect(groupingBy(ServiceBill::getVendorName));
        } else if (BillAnalysisCondition.TENANT.getCondition().equals(condition)) { //租户
            serviceBills = serviceBillList.stream().peek(serviceBill -> {
                if (serviceBill.getTenantName() == null) {
                    serviceBill.setTenantName("其他");
                }
            }).collect(groupingBy(ServiceBill::getTenantName));
        } else if (BillAnalysisCondition.BUSINESS.getCondition().equals(condition)) { // 业务
            serviceBills = serviceBillList.stream().peek(serviceBill -> {
                if (serviceBill.getBusinessName() == null) {
                    serviceBill.setBusinessName("其他");
                }
            }).collect(groupingBy(ServiceBill::getBusinessName));
        } else if (BillAnalysisCondition.PROJECT.getCondition().equals(condition)) { // 项目
            serviceBills = serviceBillList.stream().peek(serviceBill -> {
                if (serviceBill.getProjectName() == null) {
                    serviceBill.setProjectName("其他");
                }
            }).collect(groupingBy(ServiceBill::getProjectName));
        } else {
            throw new Exception("查询失败，请检查传入 condition 类型是否匹配，传入为：" + condition);
        }
        if (serviceBills.size() == 0) {
            return new ChargeResultTrend(keys, values);
        }
        HashMap<String, List<Long>> billsMap = new HashMap<>(8);
        for (Map.Entry<String, List<ServiceBill>> entry : serviceBills.entrySet()) {
            String vendorName = entry.getKey();
            List<Long> BillsIds = entry.getValue().stream().map(ServiceBill::getId).collect(Collectors.toList());
            billsMap.put(vendorName, BillsIds);
        }
        List<BillItem> billItems = billItemRepository.list(start, end, null);
        keys.addAll(timeList);
        for (String time : timeList) {
            BigDecimal totalIncome = BigDecimal.ZERO;
            for (Map.Entry<String, List<Long>> entry : billsMap.entrySet()) {
                String vendorName = entry.getKey();
                List<Long> vendorBills = entry.getValue();
                time = formatTime(time, " 23:59:59", ":59");
                String finalTime = time;
                List<BillItem> billsList = billItems.stream().filter(
                        //由于数据跑批是存在时间误差，所以取大订单的时间进行比较
                        billItem -> vendorBills.contains(billItem.getBillId()) && !serviceBillGmtDate
                                .get(billItem.getBillId()).after(DateUtils.stringToDate(finalTime, "yyyy-MM-dd HH:mm:ss")))
                        .collect(Collectors.toList());
                BigDecimal income =
                        billsList.stream().map(billItem -> Optional.ofNullable(billItem.getMoney()).orElse(BigDecimal.ZERO))
                                .reduce(BigDecimal.ZERO, BigDecimal::add);
                totalIncome = totalIncome.add(income);
                //统计过了则更新 数据列表 没有统计的则新增数据列表
                if (countMap.containsKey(vendorName)) {
                    List<Object> countNum = countMap.get(vendorName);
                    countNum.add(MathUtils.round(income.doubleValue(), 2));
                    countMap.put(vendorName, countNum);
                } else {
                    ArrayList<Object> countNum = new ArrayList<>();
                    countNum.add(MathUtils.round(income.doubleValue(), 2));
                    countMap.put(vendorName, countNum);
                }
            }
            total.add(MathUtils.round(totalIncome.doubleValue(), 2));
        }
        for (Map.Entry<String, List<Object>> entry : countMap.entrySet()) {
            ChargeTrendData chargeTrendData = new ChargeTrendData(entry.getKey(), entry.getValue());
            values.add(chargeTrendData);
        }
        values.add(new ChargeTrendData("合计(￥)", total));
        return new ChargeResultTrend(keys, values);
    }

    @Override
    public BocloudResult getUserTop5() {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            HashMap<String, Double> sortedMap = new HashMap<>(8);
            List<ServiceBill> serviceBills = serviceBillRepository.listByUser(null, null, null, null).stream()
                    .filter(serviceBill -> serviceBill.getUserName() != null).collect(Collectors.toList());
            Map<String, List<ServiceBill>> tenantBillMap =
                    serviceBills.stream().collect(groupingBy(ServiceBill::getUserName));
            for (Map.Entry<String, List<ServiceBill>> tenantEntry : tenantBillMap.entrySet()) {
                String tenantName = tenantEntry.getKey();
                List<ServiceBill> tenantServiceBills = tenantEntry.getValue();
                BigDecimal income = tenantServiceBills.stream()
                        .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                sortedMap.put(tenantName, MathUtils.round(income.doubleValue(), 2));
            }
            List<Map.Entry<String, Double>> sortedResult =
                    sortedMap.entrySet().stream().sorted(((o1, o2) -> o2.getValue().compareTo(o1.getValue()))).limit(5)
                            .collect(Collectors.toList());
            ArrayList<Object> data = new ArrayList<>();
            for (Map.Entry<String, Double> entry : sortedResult) {
                keys.add(entry.getKey());
                data.add(entry.getValue());
            }
            values.add(new ChargeTrendData("消费金额(元)", data));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "用户消费TOP5查询成功");
        } catch (Exception e) {
            logger.error("get userTop5 service income error", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "用户消费TOP5查询失败");
        }
    }

    @Override
    public BocloudResult billsAnalysisNewDownload(String time, String condition, String label) {
        SXSSFWorkbook workbook = null;
        try {
            BocloudResult bocloudResult = this.billsAnalysisNew(time, condition, label);
            if (bocloudResult.isFailed()) {
                return bocloudResult;
            }
            Object data = bocloudResult.getData();
            List<Object> objects = JSONArray.parseArray(JSONObject.toJSONString(bocloudResult.getData()), Object.class);
            String fileName = "";
            if (BillAnalysisCondition.VENDOR.getCondition().equals(condition)) {
                fileName = BillAnalysisCondition.VENDOR.getName();
            } else if (BillAnalysisCondition.TENANT.getCondition().equals(condition)) {
                fileName = BillAnalysisCondition.TENANT.getName();
            } else if (BillAnalysisCondition.BUSINESS.getCondition().equals(condition)) {
                fileName = BillAnalysisCondition.BUSINESS.getName();
            } else if (BillAnalysisCondition.PROJECT.getCondition().equals(condition)) {
                fileName = BillAnalysisCondition.PROJECT.getName();
            } else if (BillAnalysisCondition.LABEL.getCondition().equals(condition)) {
                fileName = BillAnalysisCondition.LABEL.getName();
            }
            fileName += "账单分析";
            List<JSONObject> jsObjects1 =
                    JSONArray.parseArray(JSONObject.toJSONString(objects.get(0)), JSONObject.class);
            JSONObject jsObjects2 = JSONObject.parseObject(JSONObject.toJSONString(objects.get(1)));
            //jsObjects1 扇形图数据 jsObjects2 折线图数据
            workbook = ExcelChartUtil.createAnalyzeExcel(fileName, jsObjects1, jsObjects2);
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            workbook.write(byteArrayOutputStream);
            byte[] ba = byteArrayOutputStream.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", fileName + System.currentTimeMillis() + ".xlsx");
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new BocloudResult(true, resultMap, "下载成功");
        } catch (Exception e) {
            logger.error("下载失败：", e);
            return new BocloudResult(false, "下载失败");
        } finally {
            try {
                if (null != workbook) {
                    workbook.close();
                }
            } catch (IOException e) {
                logger.info("文件流关闭失败{}", e.getMessage());
            }
        }
    }

}
