package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ApplyCategory;
import com.bocloud.cos.intf.CloudServiceService;
import com.bocloud.cos.model.BatchServiceFlowBean;
import com.bocloud.cos.model.ServiceFlowBean;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.internal.SmsInternalService;
import com.bocloud.cos.internel.entity.Permission;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 云服务业务操作实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月10日
 */
@Service
public class CloudServiceServiceImpl implements CloudServiceService {

    private static final Logger logger = LoggerFactory.getLogger(CloudServiceServiceImpl.class);
    private final ServiceCatalogRepository serviceCatalogRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceFlowConfigRepository flowConfigRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final ServiceSlaRepository serviceSlaRepository;
    private final ResourceQuotaRepository quotaRepository;
    private final SmsInternalService smsInternalService;
    private final OrderSnRepository snRepository;
    private final LockFactory lockFactory;
    private final ServiceTenantRepository serviceTenantRepository;

    public CloudServiceServiceImpl(ServiceCatalogRepository serviceCatalogRepository,
                                   CloudServiceRepository cloudServiceRepository, ServiceFlowConfigRepository flowConfigRepository,
                                   ServiceSkuRepository serviceSkuRepository, ServiceSlaRepository serviceSlaRepository,
                                   ResourceQuotaRepository quotaRepository, SmsInternalService smsInternalService, OrderSnRepository snRepository,
                                   LockFactory lockFactory, ServiceTenantRepository serviceTenantRepository) {
        this.serviceCatalogRepository = serviceCatalogRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.flowConfigRepository = flowConfigRepository;
        this.serviceSkuRepository = serviceSkuRepository;
        this.serviceSlaRepository = serviceSlaRepository;
        this.quotaRepository = quotaRepository;
        this.smsInternalService = smsInternalService;
        this.snRepository = snRepository;
        this.lockFactory = lockFactory;
        this.serviceTenantRepository = serviceTenantRepository;
    }

    @Override
    public BocloudResult<GridBean<CloudService>> list(Pager pager) {
        try {
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            sorter.put("gmtCreate", Common.ONE);
            int page = Optional.ofNullable(pager.getPage()).orElse(1);
            int rows = Optional.ofNullable(pager.getRows()).orElse(10);

            sorter.put("gmtCreate", Common.ONE);
            Long paramTenantId = null;
            for (Param pm : params) {
                Map<String, Object> param = pm.getParam();
                Object obj = param.get("tenantId");
                if (null == obj) {
                    continue;
                }
                paramTenantId = Long.valueOf(obj.toString());
                param.remove("tenantId");
                break;
            }
            int total = this.cloudServiceRepository.count(params, paramTenantId);
            List<CloudService> list = this.cloudServiceRepository.list(page, rows, sorter, params, paramTenantId);
            for (CloudService cloudService : list) {
                List<ServiceFlowBean> flowList = new ArrayList<>();
                Map<ApplyCategory, ServiceFlowConfig> flowConfigMap =
                        flowConfigRepository.getServiceFlowConfig(cloudService.getId()).stream()
                                .collect(Collectors.toMap(ServiceFlowConfig::getCategory, Function.identity()));
                flowList.add(new ServiceFlowBean(flowConfigMap.get(ApplyCategory.ApplicationOperate) != null ?
                        flowConfigMap.get(ApplyCategory.ApplicationOperate).getFlowId() : null,
                        ApplyCategory.ApplicationOperate));
                flowList.add(new ServiceFlowBean(flowConfigMap.get(ApplyCategory.AlterationOperate) != null ?
                        flowConfigMap.get(ApplyCategory.AlterationOperate).getFlowId() : null,
                        ApplyCategory.AlterationOperate));
                flowList.add(new ServiceFlowBean(flowConfigMap.get(ApplyCategory.ExtensionOperate) != null ?
                        flowConfigMap.get(ApplyCategory.ExtensionOperate).getFlowId() : null,
                        ApplyCategory.ExtensionOperate));
                flowList.add(new ServiceFlowBean(flowConfigMap.get(ApplyCategory.UnsubscribeOperate) != null ?
                        flowConfigMap.get(ApplyCategory.UnsubscribeOperate).getFlowId() : null,
                        ApplyCategory.UnsubscribeOperate));
                flowList.add(new ServiceFlowBean(flowConfigMap.get(ApplyCategory.ResourceComputerClone) != null ?
                        flowConfigMap.get(ApplyCategory.ResourceComputerClone).getFlowId() : null,
                        ApplyCategory.ResourceComputerClone));
                cloudService.setFlowList(flowList);
            }
            GridBean<CloudService> gridBean = new GridBean<>(page, rows, total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list cloud service error:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result remove(Long id, Long userId) {
        String path = CloudService.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudService service = cloudServiceRepository.query(id);
            Assert.notNull(service, "数据不存在！");
            List<Param> params = new ArrayList<>();
            params.add(new Param(MapTools.simpleMap("serviceId", id), Sign.EQ));
            int count = serviceSkuRepository.count(params);
            if (count > 0) {
                return new Result(false, "该服务下存在规格无法删除");
            }
            boolean result = cloudServiceRepository.remove(id, userId);
            if (result) {
                snRepository.removeByServiceId(id);
                cloudServiceRepository.deleteTenant(null,id);
            }
            return result ? new Result(true, "删除成功") : new Result(false, "删除失败");
        } catch (Exception e) {
            logger.error("delete CloudService fail:", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    public BocloudResult<CloudService> detail(Long id) {
        try {
            CloudService service = cloudServiceRepository.query(id);
            List<ServiceSla> slas = serviceSlaRepository.listByService(id, ServiceSla.Status.enable.name());
            if (!ListTool.isEmpty(slas)) {
                service.setSlaId(slas.get(0).getId());
            }
            return new BocloudResult<>(true, service, "查询成功");
        } catch (Exception e) {
            logger.error("Query cloud service detail error:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result create(CloudService service) {
        try {
            ServiceCatalog catalog = serviceCatalogRepository.query(service.getCatalogId());
//            Optional.ofNullable(catalog).orElseThrow(() -> new Exception("服务目录不存在"));
            if (null == catalog){
                return new BocloudResult<>(false,"服务目录不存在");
            }
            if ("operations".equals(catalog.getCode()) || "fabric".equals(catalog.getCode())) {
                service.setCode(catalog.getCode() + "." + service.getCode());
            }
            boolean flag = cloudServiceRepository.existsName(service.getName());
            if (flag) {
                return new Result(false, "服务名称已存在");
            }
            flag = cloudServiceRepository.existsCode(service.getCode());
            if (flag) {
                return new Result(false, "服务编码已存在");
            }
            flag = cloudServiceRepository.existsSn(service.getSn(), service.getCatalogId());
            if (flag) {
                return new Result(false, "当前服务目录下的服务编号已存在");
            }
            flag = smsInternalService.getLicenseService(service.getVendorType());
            if (! flag){
                return new Result(false, "未授权 "+ service.getVendorType() + " License，请联系管理员!");
            }
            service.setStatus(CloudService.Status.PUBLISHED.name());
            Integer maxId = cloudServiceRepository.getMaxId();
            if (null == maxId) {
                maxId = 0;
            }
            service.setSn(String.format("%02d", maxId));
            cloudServiceRepository.save(service);
            if ("operations".equals(catalog.getCode())) {
                //更新运维控制台权限关联
                List<Permission> permissions = smsInternalService.getPermission("运维服务");
                for (Permission permission : permissions) {
                    List<String> serviceCodes = JSONArray.parseArray(permission.getServiceCodes(), String.class);
                    serviceCodes = Optional.ofNullable(serviceCodes).orElse(new ArrayList<>());
                    serviceCodes.add(service.getCode());
                    permission.setServiceCodes(JSONObject.toJSONString(serviceCodes));
                    BocloudResult result = smsInternalService.updatePermission(permission);
                    if (result.isFailed()) {
                        logger.warn("change service permission error");
                    }
                }
            }
            if ("fabric".equals(catalog.getCode())) {
                //更新编排控制台权限关联
                List<Permission> permissions = smsInternalService.getPermission("资源编排");
                for (Permission permission : permissions) {
                    List<String> serviceCodes = JSONArray.parseArray(permission.getServiceCodes(), String.class);
                    serviceCodes = Optional.ofNullable(serviceCodes).orElse(new ArrayList<>());
                    serviceCodes.add(service.getCode());
                    permission.setServiceCodes(JSONObject.toJSONString(serviceCodes));
                    BocloudResult result = smsInternalService.updatePermission(permission);
                    if (result.isFailed()) {
                        logger.warn("change service permission error");
                    }
                }
            }
            return new Result(true, "新增成功");
        } catch (Exception e) {
            logger.error("create cloud service error:", e);
            return new Result(false, "新增失败：" + e.getMessage());
        }
    }

    @Transactional(rollbackFor = BoCloudDBException.class)
    @Override
    public Result modify(CloudService service, Long userId) {
        String path = CloudService.class.getSimpleName() + "_" + service.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudService object = cloudServiceRepository.query(service.getId());
            Assert.notNull(service, "数据不存在！");
            if (!object.getName().equals(service.getName()) && cloudServiceRepository.existsName(service.getName())) {
                return new Result(false, "服务已存在");
            }
            if (!object.getCode().equals(service.getCode()) && cloudServiceRepository.existsCode(service.getCode())) {
                return new Result(false, "服务编码已存在");
            }
            if (!object.getSn().equals(service.getSn()) && cloudServiceRepository
                    .existsSn(service.getSn(), service.getCatalogId())) {
                return new Result(false, "当前服务目录下的服务编号已存在");
            }
            object.setName(service.getName());
            object.setCode(service.getCode());
            object.setSn(service.getSn());
            object.setIconId(service.getIconId());
            object.setCatalogId(service.getCatalogId());
            object.setRemark(service.getRemark());
            object.setUnit(service.getUnit());
            object.setBillPolicy(service.getBillPolicy());
            object.setBillable(service.getBillable());
            object.setBasicPrice(service.getBasicPrice());
            object.setTemplateId(service.getTemplateId());
            object.setMenderId(userId);
            object.setGmtModify(new Date());
            boolean result = cloudServiceRepository.update(object);
            return result ? new Result(true, "修改成功") : new Result(false, "修改失败");
        } catch (Exception e) {
            logger.error("modify CloudService fail:", e);
            return new Result(false, "修改失败");
        }
    }

    /**
     * 查询租户可授权的服务
     * 创建租户时 给子租户用 授权最大范围受 租户自己限制 最小值0
     * 授权给项目 给项目使用 授权最大范围受 租户自己限制 最小值=项目使用量
     * 授权本租户 给自己租户使用 授权最大范围受 父租户限制 最小值=租户自身使用量
     *
     * @param tenantId
     * @param event
     * @param quotaAble
     * @param requestContext
     * @return
     */
    @Override
    public BocloudResult<List<CloudService>> getAbleServices(Long tenantId, String event, Boolean quotaAble,
                                                             RequestContext requestContext) {
        try {
            List<CloudService> list = new ArrayList<>();
            switch (event) {
                case "toCreateTenant":
                    list = getTenantCreateGrantAble(requestContext);
                    break;
                case "toAssignTenant":
                    if (Optional.ofNullable(quotaAble).orElse(false)) {
                        list = getServiceGrantScope(tenantId, requestContext);
                    } else {
                        list = getServiceGrantAble(tenantId, requestContext);
                    }
                    break;
                default:
                    break;

            }
            return new BocloudResult<>(true, list, "查询成功");
        } catch (Exception e) {
            logger.error("List service item by tenant fail:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 查询租户服务可授权范围(最大值 最小值)
     *
     * @param tenantId
     * @param context
     * @return
     */
    private List<CloudService> getServiceGrantScope(Long tenantId, RequestContext context) {
        List<CloudService> list = new ArrayList<>();
        if (context.getCatalog().equals(RequestContext.Catalog.Manager)) {
            //查询租户已经授权的服务
            Map<String, CloudService> tenantServices = cloudServiceRepository.listTenantService(tenantId).stream()
                    .collect(
                            Collectors.toMap(CloudService::getCode, Function.identity(), (service1, service2) -> service2));
            List<CloudService> allPublishService = this.cloudServiceRepository.listByStatus("PUBLISHED");
            //查询租户授权的服务配额
            Map<String, ResourceQuota> tenantQuota =
                    quotaRepository.list(tenantId, ResourceQuota.Category.Service.name(), null).stream()
                            .collect(Collectors.toMap(ResourceQuota::getCode, Function.identity()));
            for (CloudService cloudService : allPublishService) {
                //处理授权配额最大值最小值(无单位的服务不需要配额)
                if (cloudService.getUnit() == null || cloudService.getUnit().length() <= 0) {
                    cloudService.setReleasable(true);
                    CloudService tenantService = tenantServices.get(cloudService.getCode());
                    if (tenantService != null) {
                        //已经授权的服务check改为true
                        cloudService.setCheck(true);
                    }
                } else {
                    //设置服务配额默认值 默认最大99999 最小0 默认未授权 可释放配额
                    cloudService.setMaxQuota(99999999L);
                    cloudService.setMinQuota(0L);
                    cloudService.setCheck(false);
                    cloudService.setReleasable(true);
                    //查询当前服务是否授权
                    CloudService tenantService = tenantServices.get(cloudService.getCode());
                    if (tenantService != null) {
                        //已经授权的服务check改为true
                        cloudService.setCheck(true);
                        ResourceQuota quota = tenantQuota.get(cloudService.getCode());
                        //已经授权的服务需要修改配额最大值最小值及能否释放
                        if (quota != null) {
                            cloudService.setQuota(quota.getQuota());
                            // 有配额使用的服务不可释放
                            if (quota.getUsed() > 0) {
                                // 有授权和使用得不可以释放
                                cloudService.setReleasable(false);
                                //租户服务授权最小值受分配量限制（使用量在租户分配量里面）
                                cloudService.setMinQuota(quota.getUsed());
                            }
                        }
                    }
                }

                list.add(cloudService);
            }
        }
        return list;
    }

    /**
     * 查询租户可授权服务
     *
     * @param tenantId
     * @param context
     * @return
     */
    private List<CloudService> getServiceGrantAble(Long tenantId, RequestContext context) {
        List<CloudService> list = new ArrayList<>();
        if (context.getCatalog().equals(RequestContext.Catalog.Manager)) {
            //查询租户已经授权的服务
            Map<String, ServiceTenant> serviceTenants = serviceTenantRepository.listByTenantId(tenantId).stream()
                    .collect(Collectors.toMap(ServiceTenant::getServiceCode, Function.identity()));
            List<CloudService> allPublishService = this.cloudServiceRepository.listByStatus("PUBLISHED");
            for (CloudService cloudService : allPublishService) {
                //查询当前服务是否授权
                ServiceTenant serviceTenant = serviceTenants.get(cloudService.getCode());
                cloudService.setCheck(false);
                if (serviceTenant != null) {
                    //已经授权的服务check为true
                    cloudService.setCheck(true);
                }
                list.add(cloudService);
            }
        }
        return list;
    }

    /**
     * 查询租户创建服务可授权范围
     *
     * @param context
     * @return
     */
    private List<CloudService> getTenantCreateGrantAble(RequestContext context) {
        List<CloudService> list = new ArrayList<>();
        //管理端创建已经发布的服务都能选最大值99999 最小值0
        if (context.getCatalog().equals(RequestContext.Catalog.Manager)) {
            list = this.cloudServiceRepository.listByStatus("PUBLISHED");
            //开启配额才需要设置最大值最小值
            for (CloudService cloudService : list) {
                //有单位的才需要设置配额最大值最小值
                if (cloudService.getUnit() == null || cloudService.getUnit().length() <= 0) {
                    continue;
                }
                // 一级租户最大配额 999999L
                cloudService.setMaxQuota(99999999L);
                cloudService.setMinQuota(0L);
            }
        }
        return list;
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result updateStatus(Long id, String status, Long userId) {
        String path = CloudService.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudService service = cloudServiceRepository.query(id);
            Assert.notNull(service, "数据不存在！");
            String message = CloudService.Status.PUBLISHED.name().equalsIgnoreCase(status) ? "上架" : "下架";
            if (CloudService.Status.PUBLISHED.name().equalsIgnoreCase(status)
                && !smsInternalService.getLicenseServiceCheck(service.getVendorType())
            ){
                // 如果上架某个服务，需要先判断服务是否有License
                return new BocloudResult<>(false, "未授权 " + service.getVendorType() + " License，请联系管理员!");
            }
            service.setStatus(status);
            service.setMenderId(userId);
            service.setGmtModify(new Date());
            boolean flag = cloudServiceRepository.update(service);
            if (!flag) {
                return new Result(false, message + "失败");
            }
            return new Result(true, message + "成功");
        } catch (Exception e) {
            logger.error("update cloud service status error:", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result updateTemplate(Long id, String templateName, RequestContext context) {
        String path = CloudService.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            CloudService service = cloudServiceRepository.query(id);
            Assert.notNull(service, "数据不存在！");
            service.setTemplateName(templateName);
            service.setMenderId(context.getTarget());
            service.setGmtModify(new Date());
            boolean flag = cloudServiceRepository.update(service);
            if (!flag) {
                return new Result(false, "更新失败");
            }
            return new Result(true, "更新成功");
        } catch (Exception e) {
            logger.error("Modify cloud service template error:", e);
            return new Result(false, "更新失败");
        }
    }

    @Override
    public Result batchPublished(List<Long> ids, Long userId) {
        try {
            List<CloudService> cloudServices = cloudServiceRepository.listByIds(ids);
            for (CloudService cloudService : cloudServices) {
                cloudService.setStatus(CloudService.Status.PUBLISHED.name());
                cloudService.setMenderId(userId);
                cloudService.setGmtModify(new Date());
            }
            cloudServiceRepository.batchUpdate(cloudServices, cloudServices.size());
            return new Result(true, "批量上架服务成功");
        } catch (Exception e) {
            logger.error("batch update cloud service status error:", e);
            return new Result(false, "批量上架服务失败");
        }
    }

    @Override
    public Result batchUnpublished(List<Long> ids, Long userId) {
        try {
            List<CloudService> cloudServices = cloudServiceRepository.listByIds(ids);
            for (CloudService cloudService : cloudServices) {
                cloudService.setStatus(CloudService.Status.UNPUBLISHED.name());
                cloudService.setMenderId(userId);
                cloudService.setGmtModify(new Date());
            }
            cloudServiceRepository.batchUpdate(cloudServices, cloudServices.size());
            return new Result(true, "批量下架服务成功");
        } catch (Exception e) {
            logger.error("batch update cloud service status error:", e);
            return new Result(false, "批量下架服务失败");
        }
    }

    @Override
    public BocloudResult<List<CloudService>> listByCode(List<String> codes) {
        try {
            List<CloudService> cloudServices = cloudServiceRepository.listByCode(codes);
            return new BocloudResult<>(true, cloudServices, "获取服务列表成功");

        } catch (Exception e) {
            logger.error("get cloud service by code error,", e);
            return new BocloudResult<>(false, "获取服务列表失败");
        }
    }

    @Override
    public BocloudResult<List<CloudService>> getTenantService(Long id, RequestContext requestContext) {
        try {
            List<CloudService> cloudServices = cloudServiceRepository.listTenantService(id);
            return new BocloudResult<>(true, cloudServices, "查询成功");

        } catch (Exception e) {
            logger.error("get tenant's cloud service error,", e);
            return new BocloudResult<>(false, "获取租户服务列表失败");
        }
    }

    @Override
    public BocloudResult condition(String condition, RequestContext context) {
        if (StringUtils.isEmpty(condition)) {
            return new BocloudResult<>(false, "参数格式错误");
        }
        try {
            JSONObject object = JSONObject.parseObject(condition);
            if ("queryByCode".equalsIgnoreCase(object.getString("condition"))) {
                CloudService service = cloudServiceRepository.queryByCode(object.getString("code"));
                List<ServiceSla> slas =
                        serviceSlaRepository.listByService(service.getId(), ServiceSla.Status.enable.name());
                if (!slas.isEmpty()) {
                    service.setSlaId(slas.get(0).getId());
                }
                return new BocloudResult<>(true, service, "查询成功");
            }
            if ("queryByCatalog".equalsIgnoreCase(object.getString("condition"))) {
                List<String> catalogArray = JSONObject.parseArray(object.getString("catalogs"), String.class);
                if (ListTool.isEmpty(catalogArray)) {
                    return new BocloudResult<>(false, "参数格式错误");
                }
                List<CloudService> services = cloudServiceRepository.queryByCatalog(catalogArray);
                return new BocloudResult<>(true, services, "查询成功");
            }
            if ("getServiceType".equalsIgnoreCase(object.getString("condition"))) {
                String catalog = object.getString("catalog");
                Set<String> serviceType = this.getServiceType(catalog, context);
                return new BocloudResult<>(true, serviceType, "查询成功");
            }
        } catch (Exception e) {
            logger.error("list by condition error", e);
        }
        return new BocloudResult<>(false, "未知类型");
    }

    private Set<String> getServiceType(String catalog, RequestContext context) {
        try {
            List<CloudService> cloudServices = cloudServiceRepository.listTenantService(context.getTenant());
            List<CloudService> server = new ArrayList<>();
            switch (catalog) {
                case "server":
                    for (CloudService cloudService : cloudServices) {
                        String suffix = cloudService.getCode().substring(cloudService.getCode().lastIndexOf(".") + 1);
                        if ("server".equals(suffix)) {
                            server.add(cloudService);
                        }
                    }
                    break;
                case "volume":
                    for (CloudService cloudService : cloudServices) {
                        String suffix = cloudService.getCode().substring(cloudService.getCode().lastIndexOf(".") + 1);
                        if ("volume".equals(suffix) || "cbs".equals(suffix) || "disk".equals(suffix)) {
                            server.add(cloudService);
                        }
                    }
                    break;
                case "slb":
                    for (CloudService cloudService : cloudServices) {
                        String suffix = cloudService.getCode().substring(cloudService.getCode().lastIndexOf(".") + 1);
                        if ("slb".equals(suffix)) {
                            server.add(cloudService);
                        }
                    }
                    break;
                case "vpc":
                    for (CloudService cloudService : cloudServices) {
                        String suffix = cloudService.getCode().substring(cloudService.getCode().lastIndexOf(".") + 1);
                        if ("vpc".equals(suffix)) {
                            server.add(cloudService);
                        }
                    }
                    break;
                case "eip":
                    for (CloudService cloudService : cloudServices) {
                        String suffix = cloudService.getCode().substring(cloudService.getCode().lastIndexOf(".") + 1);
                        if ("eip".equals(suffix)) {
                            server.add(cloudService);
                        }
                    }
                    break;
                default:
                    break;
            }
            return server.stream().map(CloudService::getVendorType).collect(Collectors.toSet());
        } catch (Exception e) {
            logger.error("list permission");
            return null;
        }
    }

    @Override
    public BocloudResult<List<String>> vendors(Long catalogId, Long tenantId) {
        try {
            List<CloudService> cloudServices = cloudServiceRepository.listTenantService(tenantId);
            List<String> vendorTypes =
                    cloudServices.stream().filter(cloudService -> cloudService.getCatalogId().equals(catalogId))
                            .map(CloudService::getVendorType).distinct().collect(Collectors.toList());
            return new BocloudResult<>(true, vendorTypes, "查询失败");
        } catch (Exception e) {
            logger.error("get tenant's service vendor type error,", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult<List<CloudService>> vendorService(String vendor, Long catalogId, Long tenantId) {
        try {
            List<CloudService> cloudServices = cloudServiceRepository.listTenantService(tenantId);
            List<CloudService> vendorService = cloudServices.stream().filter(
                    cloudService -> cloudService.getCatalogId().equals(catalogId) && cloudService.getVendorType()
                            .equalsIgnoreCase(vendor)).collect(Collectors.toList());
            return new BocloudResult<>(true, vendorService, "查询失败");
        } catch (Exception e) {
            logger.error("get tenant' vendor service error,", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public Result configFlow(Long id, List<ServiceFlowBean> serviceFlowBeans, RequestContext context) {
        try {
            // 先解绑服务已经绑定的流程
            flowConfigRepository.deleteById(id);
            ArrayList<ServiceFlowConfig> serviceFlowConfigs = new ArrayList<>();
            for (ServiceFlowBean bean : serviceFlowBeans) {
                if (null != bean.getFlowId() && 0L != bean.getFlowId()) {
                    ServiceFlowConfig serviceFlowConfig =
                            new ServiceFlowConfig(id, bean.getFlowId(), new Date(), context.getTarget(),
                                    bean.getCategory());
                    serviceFlowConfigs.add(serviceFlowConfig);
                }
            }
            if (!serviceFlowConfigs.isEmpty()) {
                flowConfigRepository.batchSave(serviceFlowConfigs, serviceFlowConfigs.size());
            }
            return new Result(true, "流程绑定成功");
        } catch (Exception e) {
            logger.error("cloud service config flow error,", e);
            return new Result(false, "服务流程配置失败");
        }
    }

    @Override
    public Long getFlowIdByServiceId(Long id, ApplyCategory category) {
        return flowConfigRepository.getServiceFlowId(id, category);
    }

    @Override
    public Result batchConfigFlow(BatchServiceFlowBean serviceFlowBeans, RequestContext context) {
        try {
            // 先解绑服务已经绑定的流程s
            List<Long> serviceIds = serviceFlowBeans.getServiceIds();
            flowConfigRepository.deleteByIds(serviceIds);
            ArrayList<ServiceFlowConfig> serviceFlowConfigs = new ArrayList<>();
            List<ServiceFlowBean> serviceFlows = serviceFlowBeans.getServiceFlows();
            for (Long serviceId : serviceIds) {
                for (ServiceFlowBean flowBean : serviceFlows) {
                    if (null != flowBean.getFlowId() && 0L != flowBean.getFlowId()) {
                        ServiceFlowConfig serviceFlowConfig =
                                new ServiceFlowConfig(serviceId, flowBean.getFlowId(), new Date(), context.getTarget(),
                                        flowBean.getCategory());
                        serviceFlowConfigs.add(serviceFlowConfig);
                    }
                }
            }
            if (!serviceFlowConfigs.isEmpty()) {
                flowConfigRepository.batchSave(serviceFlowConfigs, serviceFlowConfigs.size());
            }
            return new Result(true, "流程批量绑定成功");
        } catch (Exception e) {
            logger.error("cloud service config flow error,", e);
            return new Result(false, "服务流程批量配置失败");
        }
    }
}
