package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.*;
import com.bocloud.cos.entity.Cos;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.ServiceOrder;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.enums.ResourceCategory;
import com.bocloud.cos.intf.CosService;
import com.bocloud.cos.model.ApplicationParam;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.OrderMessage;
import com.bocloud.cos.model.OrderMessage.Operation;
import com.bocloud.cos.params.CosParams;
import com.bocloud.cos.repository.CosRepository;
import com.bocloud.cos.internal.CopTaskService;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 运维服务实现
 *
 * @author weiwei
 */
@Service
public class CosServiceImpl implements CosService {

    private static Logger logger = LoggerFactory.getLogger(CosServiceImpl.class);
    @Autowired
    private CosRepository cosRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private CopTaskService copTaskService;

    @Override
    public BocloudResult list(int page, int rows, List<Param> params, Map<String, String> sorter, boolean simple,
                              RequestContext context) {
        List<Cos> list = null;
        int total = 0;
        GridBean gridBean = null;
        try {
            params = null == params ? new ArrayList<>() : params;
            sorter = null == sorter ? new HashMap<>() : sorter;
            sorter.put("gmtCreate", Common.ONE);
            if (Catalog.User == context.getCatalog()) {
                params.add(new Param(MapTools.simpleMap("gmtCatalog", context.getCatalog().name()), Sign.EQ));
                params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
                params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
            } else if (Catalog.Tenant == context.getCatalog()) {
                params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
            }
            total = cosRepository.count(params);
            if (simple) {
                List<SimpleBean> beans = cosRepository.list(params, sorter);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                list = cosRepository.list(page, rows, sorter, params);
                gridBean = GridHelper.getBean(page, rows, total, list);
            }
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detail(Long id) {
        try {
            Cos cos = cosRepository.query(id);
            if (null == cos) {
                return new BocloudResult(false, "查询失败");
            }
            return new BocloudResult(true, cos, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public BocloudResult remove(Long id, RequestContext context) {
        String path = Cos.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Cos cos = cosRepository.query(id);
            Assert.notNull(cos, "数据不存在！");
            cosRepository.remove(id, context.getTarget());
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult create(ApplicationParam param, RequestContext context) {
        ServiceOrder order = param.getOrder();
        List<OrderItem> items = order.getItems();
        if (ListTool.isEmpty(items)) {
            return new BocloudResult(false, "订单数据错误！");
        }
        DurationModel duration = param.getEmption().getDuration();
        DurationModel.Mode mode = duration.getMode();
        int amount = Optional.ofNullable(duration.getAmount()).orElse(0);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        if (amount > 0) {
            if (mode.equals(DurationModel.Mode.Month)) {
                calendar.add(Calendar.MONTH, amount);
            } else {
                calendar.add(Calendar.YEAR, amount);
            }
        } else {
            calendar.setTime(DateTools.stringToDateTime("2099-12-31 00:00:00"));
        }
        Cos cos = new Cos();
        cos.setStatus("available");
        cos.setUuid(items.get(0).getSn());
        cos.setCreatorId(context.getTarget());
        cos.setTenantId(context.getTenant());
        cos.setGmtCatalog(context.getCatalog().name());
        cos.setExpiredTime(calendar.getTime());
        cos.setInventory(JSONObject.toJSONString(param));
        if (null != param.getElements() && !param.getElements().isEmpty()) {
            ElementModel element = param.getElements().get(0);
            cos.setSpec(JSONObject.toJSONString(element.getSpecs()));
        }
        cos.setUsed(0);
        JSONObject configs = param.getConfigs();
        cos.setCategoryId(configs.getLong("categoryId"));
        cos.setSkuId(configs.getLong("skuId"));
        cos.setServiceCode(param.getService());
        cos.setName(configs.getString("serviceName") + "_" + System.currentTimeMillis());
        try {
            // FIXME: 2021/5/6 这里的次数从哪里获取
            cos.setTimes(1);
            cosRepository.save(cos);
            OrderMessage message =
                    new OrderMessage(cos.getUuid(), Operation.Application, OrderItemStatus.OpenedSuccess, null,
                            "运维服务【" + configs.getString("serviceName") + "】订购成功", cos.getGmtCreate(), cos.getExpiredTime(),
                            null, ResourceCategory.Operations.name(), context);
            amqpTemplate.convertAndSend("resource.result.queue", message);
            return new BocloudResult(true, "创建成功");
        } catch (Exception e) {
            OrderMessage message =
                    new OrderMessage(cos.getUuid(), Operation.Application, OrderItemStatus.OpenedFailure, null,
                            "运维服务【" + configs.getString("serviceName") + "】订购失败", cos.getGmtCreate(), cos.getExpiredTime(),
                            null, ResourceCategory.Operations.name(), context);
            amqpTemplate.convertAndSend("resource.result.queue", message);
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult execute(CosParams params, RequestContext context) {
        Long cosId = params.getCosId();
        String path = Cos.class.getSimpleName() + "_" + cosId;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Cos cos = cosRepository.query(cosId);
            Assert.notNull(cos, "数据不存在！");
            if ("unavailable".equalsIgnoreCase(cos.getStatus())) {
                return new BocloudResult(false, "该服务实例处于不可用状态");
            }
            JSONObject task = params.getTask();
            task.put("name", cos.getName().substring(0, cos.getName().indexOf("_")));
            task.put("cosId", cosId);
            BocloudResult result = copTaskService.execute(task, context.getApiKey());
            if (result.isFailed()) {
                return result;
            }
            //			cos.setUsed(cos.getUsed() + 1);
            //			if (cos.getUsed() >= cos.getTimes()) {
            //				cos.setStatus("unavailable");
            //			}
            //			cos.setMenderId(context.getTarget());
            //			cos.setGmtModify(new Date());
            //			cosRepository.update(cos);
            return new BocloudResult(true, "运维任务下发成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "运维任务下发失败");
        }
    }

}
