package com.bocloud.cos.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.serializer.SerializerFeature;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Sign;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.Document;
import com.bocloud.cos.intf.DocumentService;
import com.bocloud.cos.repository.DocumentRepository;
import com.bocloud.service.lock.LockFactory;
import io.jsonwebtoken.lang.Assert;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 使用文档业务层
 *
 * @author tyl
 * @update fanzx
 * @since 2021/3/8 15:04
 */
@Service
public class DocumentServiceImpl implements DocumentService {

    Logger logger = LoggerFactory.getLogger(Document.class);
    private final LockFactory lockFactory;
    private final DocumentRepository documentRepository;
    @Value("${upload.storage.path:/home/cmp}")
    private String storagePath;
    @Value("${upload.storage.host:http://10.40.20.145:60006}")
    private String storageHost;

    public DocumentServiceImpl(LockFactory lockFactory, DocumentRepository documentRepository) {
        this.lockFactory = lockFactory;
        this.documentRepository = documentRepository;
    }

    @Override
    public BocloudResult upload(MultipartFile multipartFile) {
        try {
            String originalFilename = multipartFile.getOriginalFilename();
            org.springframework.util.Assert.notNull(originalFilename, "文件名不能为空");
            String fileName =
                    System.currentTimeMillis() + originalFilename.substring(originalFilename.lastIndexOf("."));
            String filePath = "/attachment/docs/files/";
            String path = storagePath + filePath;
            File file = new File(path, fileName);
            if (!file.exists()) {
                file.mkdirs();
            }
            multipartFile.transferTo(file);
            String resultPath = filePath + fileName;
//            String resultPath = storageHost + filePath + fileName;
            return new BocloudResult(true, resultPath, "图片上传成功");
        } catch (Exception e) {
            logger.error("update document picture error,", e);
            return BocloudResult.FAILED("图片上传失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult delete(List<String> paths) {
        try {
            List<String> errorPath = new ArrayList<>();
            for (String path : paths) {
                File file = new File(path.substring(path.indexOf("/docs/files/")));
                if (file.exists()) {
                    if (!file.delete()) {
                        errorPath.add(path);
                    }
                } else {
                    logger.warn("the picture in path [{}] is not exists", path);
                }
            }
            if (errorPath.isEmpty()) {
                return new BocloudResult(true, null, "文档图片删除成功");
            } else {
                return errorPath.size() == paths.size() ? new BocloudResult(false, null, "文档图片删除失败") :
                        new BocloudResult(false, errorPath, "文档图片删除异常");
            }
        } catch (Exception e) {
            logger.error("remove document pictures error error ,", e);
            return BocloudResult.FAILED("文档图片删除失败" + e.getMessage());
        }
    }

    /**
     * 新增文档
     *
     * @param document
     * @param context
     * @return
     */
    @Override
    public BocloudResult create(Document document, RequestContext context) {
        try {
            document.setGmtCreate(new Date());
            document.setCreatorId(context.getTarget());
            document.setDeleted(false);
            // 1. 先判断parentId是否为0,如果为0则代表是父节点
            if (0 == document.getParentId()) {
                // 2. 判断库中是否已经存在parentId=0 如果存在 则添加失败
                Document father = documentRepository.selectParent(document.getCategory());
                Assert.isNull(father, "该分类已经存在");
                document.setStatus(Document.Status.PUBLISHED.name());
                // 如果为父节点 则进行特殊添加
                documentRepository.save(document);
                return new BocloudResult(true, null, "新增父文档成功");
            }
            // 如果不为父节点 则去查询该子节点名称是否存在
            // 获取到父节点下所有的子节点信息
            Document existName = documentRepository.queryByName(document.getName(), document.getParentId(), null);
            Assert.isNull(existName, "该文档已经存在");
            // 如果名称不重复 则进行添加
            document.setStatus(Document.Status.UNPUBLISHED.name());
            documentRepository.save(document);
            return new BocloudResult(true, null, "新增文档成功");
        } catch (Exception e) {
            logger.error("create document error", e);
            return BocloudResult.FAILED("新增文档失败:" + e.getMessage());
        }
    }

    /**
     * 查询文档树
     *
     * @param paramList
     * @param context
     * @return
     */
    @Override
    public BocloudResult list(List<Param> paramList, RequestContext context) {
        try {
            // 查询文档树列表
            Map<Long, List<Document>> categoryMap = documentRepository.listAllCategory(paramList).stream()
                    .collect(Collectors.groupingBy(Document::getParentId));
            List<Document> rootDocument = Optional.ofNullable(categoryMap.get(0L)).orElse(new ArrayList<>());
            buildCategoryTree(rootDocument, categoryMap);
            return new BocloudResult(true, rootDocument, "查询分类成功");
        } catch (Exception e) {
            logger.error("select document error", e);
            return BocloudResult.FAILED("查询文档树失败:" + e.getMessage());
        }
    }

    /**
     * 递归查询文档树
     *
     * @param rootDocument
     * @param categoryMap
     */
    private void buildCategoryTree(List<Document> rootDocument, Map<Long, List<Document>> categoryMap) {
        for (Document category : rootDocument) {
            List<Document> children = categoryMap.get(category.getId());
            if (children != null && !children.isEmpty()) {
                buildCategoryTree(children, categoryMap);
                category.setChildren(children);
            } else {
                category.setChildren(new ArrayList<>(0));
            }
        }
    }

    /**
     * 修改文档信息
     *
     * @param id
     * @param document
     * @return
     */
    @Override
    public BocloudResult modify(Long id, Document document, RequestContext context) {
        try {
            // 先获取是否存在于数据库中
            Document exist = documentRepository.queryByName(null, document.getParentId(), document.getId());
            Assert.notNull(exist, "操作文档不存在");
            // 修改操作文档
            exist.setStatus(document.getStatus());
            exist.setGmtModify(new Date());
            exist.setMenderId(context.getTarget());
            exist.setPriority(document.getPriority());
            exist.setContent(document.getContent());
            exist.setName(document.getName());
            // 文档展示内容
            exist.setCategory(document.getCategory());
            documentRepository.update(exist);
            return new BocloudResult(true, exist, "修改成功");
        } catch (Exception e) {
            logger.error("update document error" + e);
            return BocloudResult.FAILED("修改文档失败!" + e.getMessage());
        }
    }

    /**
     * 删除使用文档
     *
     * @param id
     * @param context
     * @return
     */
    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        try {
            // 先获取是否存在于数据库中
            Document exist = documentRepository.queryByName(null, null, id);
            Assert.notNull(exist, "操作文档不存在");
            Map<Long, List<Document>> categoryMap = documentRepository.listAllCategory(null).stream()
                    .collect(Collectors.groupingBy(Document::getParentId));
            List<Document> rootDocument = Optional.ofNullable(categoryMap.get(id)).orElse(new ArrayList<>());
            buildCategoryTree(rootDocument, categoryMap);
            for (Document document : rootDocument) {
                documentRepository.update(document.getId(), context.getTarget());
                if (document.getChildren() != null) {
                    deleted(document.getChildren(), context);
                }
            }
            // 删除操作文档
            documentRepository.update(exist.getId(), context.getTarget());
            return new BocloudResult(true, exist, "删除文档成功!");
        } catch (Exception e) {
            logger.error("update document error" + e);
            return BocloudResult.FAILED("删除文档失败!" + e.getMessage());
        }
    }

    /**
     * 查询文档详细信息
     *
     * @param id
     * @return
     */
    @Override
    public BocloudResult detail(Long id) {
        try {
            Document document = documentRepository.detail(id);
            return new BocloudResult(true, document, "查询文档详情成功");
        } catch (Exception e) {
            logger.error("get document detail error,", e);
            return BocloudResult.FAILED("查询文档详情失败:" + e.getMessage());
        }
    }

    /**
     * 模糊搜索查询树
     *
     * @param name
     * @param context
     * @return
     */
    @Override
    public BocloudResult search(String name, String category, String status, RequestContext context) {
        try {
            List<Param> params = Arrays.asList(new Param(MapTools.simpleMap("category", category), Sign.EQ), new Param(MapTools.simpleMap("status", status), Sign.EQ));
            Map<Long, List<Document>> categoryMap = documentRepository.listAllCategory(params).stream()
                    .collect(Collectors.groupingBy(Document::getParentId));
            // rootDocument是获取到整个列表树
            List<Document> rootDocument = Optional.ofNullable(categoryMap.get(0L)).orElse(new ArrayList<>());
            buildCategoryTree(rootDocument, categoryMap);
            // 获取到整个列表树之后 判断name是否包含 如果包含的话 则放到一个新对象中
            // 创建一个新的Document对象 用于保存遍历出来的数据
            List<Document> result = new ArrayList<>();
            // 如果name不为空 则进行查询
            if (!"".equals(name)) {
                searchTree(rootDocument, name, result);
                JSONArray objects = JSONArray.parseArray(JSON.toJSONString(result, SerializerFeature.DisableCircularReferenceDetect));
                return new BocloudResult(true, objects, "查询文档树成功!");
            } else {
                List<Param> paramList = Collections.singletonList(new Param(MapTools.simpleMap("category", category), Sign.EQ));
                // 如果name为空 则查询所有内容
                return this.list(paramList, context);
            }
        } catch (Exception e) {
            logger.error("document search error,", e);
            return BocloudResult.FAILED("查询文档树失败:" + e.getMessage());
        }
    }

    /**
     * 管理端控制自服务展示
     *
     * @param display
     * @param context
     * @return
     */
    @Override
    public BocloudResult serviceDisplay(Boolean display, RequestContext context) {
        try {
            documentRepository.display(display, context);
            return new BocloudResult(true, "文档修改成功");
        } catch (Exception e) {
            logger.error("get document detail error,", e);
            return BocloudResult.FAILED("文档修改失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult getDisplay(RequestContext context) {
        try {
            Document document = documentRepository.queryDisplay(context);
            return new BocloudResult(true, document.getDisplay(), "查询文档状态成功");
        } catch (Exception e) {
            logger.error("get document detail error,", e);
            return BocloudResult.FAILED("查询文档状态失败:" + e.getMessage());
        }
    }

    /**
     * 递归模糊查询文档树
     *
     * @param rootDocument 查询出来的树列表
     * @param name         模糊搜索name
     * @param result       结果集
     */
    private void searchTree(List<Document> rootDocument, String name, List<Document> result) {
        // 遍历树
        rootDocument.forEach(document -> {
            if (document.getName().contains(name)) {
                result.add(document);
                searchTree(document.getChildren(), name, result);
            } else {
                // 如果不包含 则查询子级是否包含
                searchTree(document.getChildren(), name, result);
            }
        });
    }

    /**
     * 递归删除
     *
     * @param documents
     * @param context
     */
    public void deleted(List<Document> documents, RequestContext context) {
        documents.forEach(document -> {
            documentRepository.update(document.getId(), context.getTarget());
            if (document.getChildren() != null) {
                document.getChildren().forEach(child -> {
                    documentRepository.update(child.getId(), context.getTarget());
                    List<Document> children = document.getChildren();
                    if (children != null) {
                        deleted(children, context);
                    }
                });
            }
        });
    }
}
