package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.utils.DateUtils;
import com.bocloud.cos.utils.MathUtils;
import com.bocloud.cos.enums.BocloudStatus;
import com.bocloud.cos.enums.VendorType;
import com.bocloud.cos.intf.ExternalService;
import com.bocloud.cos.model.ChargeResultTrend;
import com.bocloud.cos.model.ChargeTrendData;
import com.bocloud.cos.internal.ImsInternalService;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.cos.internel.entity.CloudServer;
import com.bocloud.cos.internel.entity.CloudVendor;
import com.bocloud.cos.internel.entity.Volume;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 对外展示大屏业务
 *
 * @author tyl
 * @since 2020/12/23 15:12
 */
@Service
public class ExternalServiceImpl implements ExternalService {
    private final ImsInternalService imsInternalService;
    private final Logger logger = LoggerFactory.getLogger(ExternalServiceImpl.class);

    public ExternalServiceImpl(ImsInternalService imsInternalService) {
        this.imsInternalService = imsInternalService;
    }

    @Override
    public BocloudResult resourceOverview(Long dcId, RequestContext context) {
        Map<String, Object> result = new HashMap<>(8);
        try {
            List<CloudServer> cloudServerList = imsInternalService.listVmsByDcId(dcId, false, context);
            Integer cpuNum =
                    cloudServerList.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0))
                            .reduce(Integer::sum).orElse(0);
            BigDecimal memNum = cloudServerList.stream()
                    .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                    .reduce(BigDecimal.ZERO, BigDecimal::add);
            List<Volume> volumes = getVolumes(dcId, cloudServerList);
            Integer diskNum =
                    volumes.stream().map(volume -> Optional.ofNullable(volume.getSize()).orElse(0)).reduce(Integer::sum)
                            .orElse(0);
            result.put("cpu", cpuNum);
            result.put("mem", MathUtils.round(memNum.doubleValue() / 1024D, 2));
            result.put("disk", MathUtils.round(diskNum / 1024D, 2));
            result.put("vms", cloudServerList.size());
            return new BocloudResult(true, result, "获取资源总览信息成功");
        } catch (Exception e) {
            logger.error("get cloud platform list error,", e);
            return new BocloudResult(false, result, "获取资源总览信息失败");
        }
    }

    @Override
    public BocloudResult vmsTrend(Long dcId, RequestContext context) {
        List<String> timeList = DateUtil.timeToList(new JSONObject());
        List<String> keys = new ArrayList<>(timeList);
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            List<CloudServer> cloudServerList = imsInternalService.listVmsByDcId(dcId, false, context);
            Set<String> vendorTypes =
                    imsInternalService.listCloudVendor(null, context).stream().map(CloudVendor::getType)
                            .filter(s -> !s.isEmpty()).collect(Collectors.toSet());
            HashMap<String, List<Object>> dataMap = new HashMap<>();
            for (String vendorType : vendorTypes) {
                dataMap.put(vendorType, new ArrayList<>());
            }
            for (String time : timeList) {
                //筛选目标云主机 在 指定时间前创建 指定条件主机下 非创建中的云主机
                List<CloudServer> targetVms = cloudServerList.stream().filter(
                                cloudServer -> !BocloudStatus.Server.BUILDING.name().equals(cloudServer.getStatus()) && cloudServer
                                        .getGmtCreate()
                                        .before(DateUtils.stringToDate(time.substring(0, 10) + " 23:59:59", "yyyy-MM-dd HH:mm:ss"))
                                        && cloudServer.getGmtCreate()
                                        .after(DateUtils.stringToDate(time.substring(0, 10) + " 00:00:00", "yyyy-MM-dd HH:mm:ss")))
                        .collect(Collectors.toList());
                Map<String, List<CloudServer>> vmsMap =
                        targetVms.stream().filter(cloudServer -> cloudServer.getVendorType() != null)
                                .collect(Collectors.groupingBy(CloudServer::getVendorType));
                for (Map.Entry<String, List<Object>> entry : dataMap.entrySet()) {
                    String vendorType = entry.getKey();
                    List<Object> data = entry.getValue();
                    List<CloudServer> servers = vmsMap.get(vendorType);
                    Integer serverNumber = servers != null ? servers.size() : 0;
                    data.add(serverNumber);
                }
            }
            for (Map.Entry<String, List<Object>> entry : dataMap.entrySet()) {
                String key = entry.getKey();
                List<Object> data = entry.getValue();
                String name = VendorType.valueOf(key).getName();
                values.add(new ChargeTrendData(name, data));
            }
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询云主机申请趋势成功");
        } catch (Exception e) {
            logger.error("get current vendor apply info error,", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询云主机申请趋势失败");
        }
    }

    @Override
    public BocloudResult resourceTrend(Long dcId, RequestContext context) {
        List<String> timeList = DateUtil.timeToList(new JSONObject());
        List<String> keys = new ArrayList<>(timeList);
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            List<CloudServer> cloudServerList = imsInternalService.listVmsByDcId(dcId, false, context);
            List<Volume> volumes = getVolumes(dcId, cloudServerList);
            cloudServerList.removeIf(cloudServer -> VendorType.H3C.name().equals(cloudServer.getVendorType()));
            List<Object> cpuData = new ArrayList<>();
            List<Object> memData = new ArrayList<>();
            List<Object> diskData = new ArrayList<>();
            for (String time : timeList) {
                //筛选目标云主机 在 指定时间前创建 指定条件主机下 非创建中的云主机
                List<CloudServer> targetVms = cloudServerList.stream().filter(
                                cloudServer -> !BocloudStatus.Server.BUILDING.name().equals(cloudServer.getStatus()) && cloudServer
                                        .getGmtCreate()
                                        .before(DateUtils.stringToDate(time.substring(0, 10) + " 23:59:59", "yyyy-MM-dd HH:mm:ss"))
                                        && cloudServer.getGmtCreate()
                                        .after(DateUtils.stringToDate(time.substring(0, 10) + " 00:00:00", "yyyy-MM-dd HH:mm:ss")))
                        .collect(Collectors.toList());
                Integer cpuNum =
                        targetVms.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0))
                                .reduce(Integer::sum).orElse(0);
                cpuData.add(cpuNum);
                BigDecimal memNum = targetVms.stream()
                        .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal.ZERO, BigDecimal::add);
                memData.add(memNum.doubleValue());
                //筛选指定云主机下的磁盘--由于存在数据盘，所以改用取volum信息，没有用server对应的disk信息
                List<Long> serverIds = targetVms.stream().map(CloudServer::getId).collect(Collectors.toList());
                List<Volume> volumeList = volumes.stream().filter(volume -> serverIds.contains(volume.getServerId()))
                        .collect(Collectors.toList());
                Integer diskNum = volumeList.stream().map(volume -> Optional.ofNullable(volume.getSize()).orElse(0))
                        .reduce(Integer::sum).orElse(0);
                diskData.add(diskNum);
            }
            values.add(new ChargeTrendData("CPU(C)", cpuData));
            values.add(new ChargeTrendData("内存(GB)", memData));
            values.add(new ChargeTrendData("磁盘(GB)", diskData));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询当前平台资源使用趋势成功");
        } catch (Exception e) {
            logger.error("get current vendor resource used info error,", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询当前平台资源使用趋势失败");
        }
    }

    @Override
    public BocloudResult tenantResourceCount(Long dcId, RequestContext context) {
        JSONArray result = new JSONArray();
        try {
            //根据dcId取出对应volume
            List<CloudServer> cloudServers = imsInternalService.listVmsByDcId(dcId, false, context);
            List<Volume> volumes = getVolumes(dcId, cloudServers);
            Map<String, List<CloudServer>> serverMap =
                    cloudServers.stream().filter(cloudServer -> cloudServer.getTenantName() != null)
                            .collect(Collectors.groupingBy(CloudServer::getTenantName));
            for (Map.Entry<String, List<CloudServer>> entry : serverMap.entrySet()) {
                String tenantName = entry.getKey();
                List<CloudServer> servers = entry.getValue();
                int cupTotal = servers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0))
                        .reduce(Integer::sum).orElse(0);
                Double memTotal = servers.stream()
                        .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal::add).orElse(BigDecimal.ZERO).doubleValue();
                //单独处理硬盘
                List<Long> serverIds = servers.stream().map(CloudServer::getId).collect(Collectors.toList());
                List<Volume> volumesRes = volumes.stream().filter(volume -> serverIds.contains(volume.getServerId()))
                        .collect(Collectors.toList());
                int diskTotal = volumesRes.stream().map(volume -> Optional.ofNullable(volume.getSize()).orElse(0))
                        .reduce(Integer::sum).orElse(0);
                JSONObject tenantObject = new JSONObject();
                tenantObject.put("name", tenantName);
                tenantObject.put("number", cloudServers.size());
                tenantObject.put("cpu", cupTotal);
                tenantObject.put("mem", memTotal);
                tenantObject.put("disk", diskTotal);
                result.add(tenantObject);
            }
            return new BocloudResult(true, result, "获取租户计算资源统计成功");
        } catch (Exception e) {
            logger.error("get tenant resource info error,", e);
            return new BocloudResult(false, result, "获取租户计算资源统计失败");
        }
    }

    /**
     * @return java.util.List<com.bocloud.internel.entity.Volume>
     * @Author: lxy
     * @Description: 根据dcId取出对应volume
     * @Date 2021/7/5 14:28
     * @Param dcId
     * @throws:
     **/
    private List<Volume> getVolumes(Long dcId, List<CloudServer> cloudServerList) {
        List<Volume> volumes = new ArrayList<>();
        //取出dc下所有的vendorId
        List<CloudVendor> cloudVendors = imsInternalService.listCloudVendor(null);
        //筛选指定云主机下的磁盘--由于存在数据盘，所以改用取volum信息，没有用server对应的disk信息
        List<Long> serverIds = cloudServerList.stream().map(CloudServer::getId).collect(Collectors.toList());
        if (!ListTool.isEmpty(cloudVendors)) {
            List<Long> dcVendorIds = new ArrayList<>();
            if (null != dcId) {
                dcVendorIds = cloudVendors.stream()
                        .filter(cloudVendor -> null != cloudVendor.getDcId() && cloudVendor.getDcId().equals(dcId))
                        .map(CloudVendor::getId).distinct().collect(Collectors.toList());
            } else {
                dcVendorIds =
                        cloudVendors.stream().filter(cloudVendor -> null != cloudVendor.getDcId()).map(CloudVendor::getId)
                                .distinct().collect(Collectors.toList());
            }
            //单独处理云硬盘，可能出现数据盘的情况
            if (!ListTool.isEmpty(dcVendorIds)) {
                for (Long vendorId : dcVendorIds) {
                    BocloudResult serviceVolume = imsInternalService.getVolume(vendorId);
                    if (serviceVolume.getData() != null) {
                        List<Volume> volumeList =
                                JSONArray.parseArray(JSONObject.toJSONString(serviceVolume.getData()), Volume.class);
                        volumeList = volumeList.stream().filter(volume1 -> serverIds.contains(volume1.getServerId()))
                                .collect(Collectors.toList());
                        volumes.addAll(volumeList);
                    }
                }
            }
        }
        return volumes;
    }

    @Override
    public BocloudResult projectResourceCount(Long dcId, RequestContext context) {
        JSONArray result = new JSONArray();
        try {
            //根据dcId取出对应volume
            List<CloudServer> cloudServers = imsInternalService.listVmsByDcId(dcId, false, context);
            List<Volume> volumes = getVolumes(dcId, cloudServers);
            Map<String, List<CloudServer>> serverMap =
                    cloudServers.stream().filter(cloudServer -> cloudServer.getProjectName() != null)
                            .collect(Collectors.groupingBy(CloudServer::getProjectName));
            for (Map.Entry<String, List<CloudServer>> entry : serverMap.entrySet()) {
                String projectName = entry.getKey();
                List<CloudServer> servers = entry.getValue();
                int cupTotal = servers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0))
                        .reduce(Integer::sum).orElse(0);
                Double memTotal = servers.stream()
                        .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal::add).orElse(BigDecimal.ZERO).doubleValue();
                //单独处理硬盘
                List<Long> serverIds = servers.stream().map(CloudServer::getId).collect(Collectors.toList());
                List<Volume> volumesRes = volumes.stream().filter(volume -> serverIds.contains(volume.getServerId()))
                        .collect(Collectors.toList());
                int diskTotal = volumesRes.stream().map(volume -> Optional.ofNullable(volume.getSize()).orElse(0))
                        .reduce(Integer::sum).orElse(0);
                JSONObject tenantObject = new JSONObject();
                tenantObject.put("name", projectName);
                tenantObject.put("number", cloudServers.size());
                tenantObject.put("cpu", cupTotal);
                tenantObject.put("mem", memTotal);
                tenantObject.put("disk", diskTotal);
                result.add(tenantObject);
            }
            return new BocloudResult(true, result, "获取项目计算资源统计成功");
        } catch (Exception e) {
            logger.error("get project resource info error,", e);
            return new BocloudResult(false, result, "获取项目计算资源统计失败");
        }
    }

    @Override
    public BocloudResult businessResourceCount(Long dcId, RequestContext context) {
        JSONArray result = new JSONArray();
        try {
            //根据dcId取出对应volume
            List<CloudServer> cloudServers = imsInternalService.listVmsByDcId(dcId, false, context);
            List<Volume> volumes = getVolumes(dcId, cloudServers);
            Map<String, List<CloudServer>> serverMap =
                    cloudServers.stream().filter(cloudServer -> cloudServer.getBusinessName() != null)
                            .collect(Collectors.groupingBy(CloudServer::getBusinessName));
            for (Map.Entry<String, List<CloudServer>> entry : serverMap.entrySet()) {
                String businessName = entry.getKey();
                List<CloudServer> servers = entry.getValue();
                int cupTotal = servers.stream().map(cloudServer -> Optional.ofNullable(cloudServer.getCpu()).orElse(0))
                        .reduce(Integer::sum).orElse(0);
                double memTotal = servers.stream()
                        .map(cloudServer -> Optional.ofNullable(cloudServer.getMemory()).orElse(BigDecimal.ZERO))
                        .reduce(BigDecimal::add).orElse(BigDecimal.ZERO).doubleValue();
                //单独处理硬盘
                List<Long> serverIds = servers.stream().map(CloudServer::getId).collect(Collectors.toList());
                List<Volume> volumesRes = volumes.stream().filter(volume -> serverIds.contains(volume.getServerId()))
                        .collect(Collectors.toList());
                int diskTotal = volumesRes.stream().map(volume -> Optional.ofNullable(volume.getSize()).orElse(0))
                        .reduce(Integer::sum).orElse(0);
                JSONObject tenantObject = new JSONObject();
                tenantObject.put("name", businessName);
                tenantObject.put("number", cloudServers.size());
                tenantObject.put("cpu", cupTotal);
                tenantObject.put("mem", memTotal);
                tenantObject.put("disk", diskTotal);
                result.add(tenantObject);
            }
            return new BocloudResult(true, result, "获取业务计算资源统计成功");
        } catch (Exception e) {
            logger.error("get business resource info error,", e);
            return new BocloudResult(false, result, "获取业务计算资源统计失败");
        }
    }
}
