package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.ServiceOrder;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.cos.enums.ChargeCommon.OrderStatus;
import com.bocloud.cos.internal.ImsInternalService;
import com.bocloud.cos.internal.SmsTenantService;
import com.bocloud.cos.intf.OrderAnalysisService;
import com.bocloud.cos.model.ChargeResultTrend;
import com.bocloud.cos.model.ChargeTrendData;
import com.bocloud.cos.repository.ServiceOrderRepository;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.cos.utils.MathUtils;
import com.bocloud.cos.internel.entity.CloudVendor;
import com.bocloud.cos.internel.entity.Tenant;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

import static com.bocloud.cos.enums.ChargeCommon.ChargeMode.PostCharge;
import static com.bocloud.cos.enums.ChargeCommon.ChargeMode.PreCharge;
import static com.bocloud.cos.enums.ChargeCommon.OrderStatus.*;
import static java.util.stream.Collectors.groupingBy;
import static java.util.stream.Collectors.toMap;

/**
 * 订单分析业务类
 *
 * @author tyl
 * @time 2020/6/23 周二 11:19
 */
@Service
public class OrderAnalysisServiceImpl implements OrderAnalysisService {
    private final Logger logger = LoggerFactory.getLogger(OrderAnalysisServiceImpl.class);
    private final SmsTenantService tenantService;
    private final ServiceOrderRepository serviceOrderRepository;
    private final ImsInternalService imsInternalService;
    private final ThreadPoolTaskExecutor taskExecutor;

    public OrderAnalysisServiceImpl(SmsTenantService tenantService, ServiceOrderRepository serviceOrderRepository,
                                    ImsInternalService imsInternalService, ThreadPoolTaskExecutor taskExecutor) {
        this.tenantService = tenantService;
        this.serviceOrderRepository = serviceOrderRepository;
        this.imsInternalService = imsInternalService;
        this.taskExecutor = taskExecutor;
    }

    @Override
    public BocloudResult getOrdersOverview(Long vendorId, String time, String startString, String endString, Boolean owner, RequestContext context) {
        try {
            context = owner ? context : null;
            JSONObject resultObject = new JSONObject();
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime todayBegin = LocalDateTime.of(now.getYear(), now.getMonth(), now.getDayOfMonth(), 0, 0, 0, 0);
            LocalDateTime end = todayBegin.plus(1, ChronoUnit.DAYS);
            //获取总订单
            int totalNum = serviceOrderRepository.listByVendor(null, DateUtil.localDateTimeFormat(now), null, context).size();
            JSONObject totalObject = new JSONObject();
            totalObject.put("name", "总订单数");
            totalObject.put("value", totalNum);
            resultObject.put("totalNum", totalObject);
            //获取日订单
            int dayOrders = serviceOrderRepository
                    .listByVendor(DateUtil.localDateTimeFormat(todayBegin), DateUtil.localDateTimeFormat(now), null, context).size();
            JSONObject dayNum = new JSONObject();
            dayNum.put("name", "日订单数");
            dayNum.put("value", dayOrders);
            dayNum.put("time", DateUtil.localDateFormat(now));
            resultObject.put("dayNum", dayNum);
            //获取月订单
            LocalDateTime monthBegin = LocalDateTime.of(now.getYear(), now.getMonth(), 1, 0, 0, 0, 0);
            int monthOrders = serviceOrderRepository
                    .listByVendor(DateUtil.localDateTimeFormat(monthBegin), DateUtil.localDateTimeFormat(now), null, context).size();
            JSONObject monthNum = new JSONObject();
            monthNum.put("name", "月订单数");
            monthNum.put("value", monthOrders);
            monthNum.put("time", DateUtil.localDateFormat(now));
            resultObject.put("monthNum", monthNum);

            //获取日环比 环比增长率=（本期数-上期数）/上期数×100%
            LocalDateTime yesterday = todayBegin.minus(1, ChronoUnit.DAYS);
            int lastDayOrders = serviceOrderRepository
                    .listByVendor(DateUtil.localDateTimeFormat(yesterday), DateUtil.localDateTimeFormat(todayBegin), null, context)
                    .size();
            double dayRate = lastDayOrders == 0 ? 0.00 : 1.00 * (dayOrders - lastDayOrders) / lastDayOrders;
            JSONObject dayRateObject = new JSONObject();
            dayRateObject.put("name", "日订单环比");
            dayRateObject.put("value", MathUtils.round(dayRate, 2));
            dayRateObject.put("time", DateUtil.localDateFormat(now));
            resultObject.put("dayRate", dayRateObject);
            LocalDateTime weekBegin = end.minus(1, ChronoUnit.WEEKS);
            LocalDateTime lastWeekBegin = weekBegin.minus(1, ChronoUnit.WEEKS);
            int weekNum = serviceOrderRepository
                    .listByVendor(DateUtil.localDateTimeFormat(weekBegin), DateUtil.localDateTimeFormat(now), null, context).size();
            int lastWeekNum = serviceOrderRepository
                    .listByVendor(DateUtil.localDateTimeFormat(lastWeekBegin), DateUtil.localDateTimeFormat(weekBegin),
                            null, context).size();
            double weekRate = lastWeekNum == 0 ? 0.00 : 1.00 * (weekNum - lastWeekNum) / lastWeekNum;
            JSONObject weekRateRateObject = new JSONObject();
            weekRateRateObject.put("name", "周订单环比");
            weekRateRateObject.put("value", MathUtils.round(weekRate, 2));
            weekRateRateObject.put("time", DateUtil.localDateFormat(weekBegin) + "-" + DateUtil.localDateFormat(now));
            resultObject.put("weekRate", weekRateRateObject);
            String startTime;
            String endTime;
            if (StringUtils.hasText(time)) {
                Map<String, String> timeMap = DateUtil.timeToMap(time);
                startTime = timeMap.get("start");
                endTime = timeMap.get("end");
            } else {
                startTime = startString;
                endTime = endString;
            }
            RequestContext finalContext = context;
            Future<JSONArray> vendorOrdersSubmit =
                    taskExecutor.submit(() -> this.getVendorOrdersCount(startTime, endTime, vendorId, finalContext));
            Future<JSONArray> tenantOrdersSubmit =
                    taskExecutor.submit(() -> this.getTenantOrdersCount(startTime, endTime, vendorId, finalContext));
            Future<JSONArray> statusOrdersSubmit =
                    taskExecutor.submit(() -> this.getOrderStatusCount(startTime, endTime, vendorId, finalContext));
            Future<JSONArray> typeOrdersSubmit =
                    taskExecutor.submit(() -> this.getOrderTypeCount(startTime, endTime, vendorId, finalContext));
            Future<JSONArray> resourceOrdersSubmit =
                    taskExecutor.submit(() -> this.getResourceOrdersCount(startTime, endTime, vendorId, finalContext));

            resultObject.put("vendorOrders", vendorOrdersSubmit.get());
            resultObject.put("tenantOrders", tenantOrdersSubmit.get());
            resultObject.put("statusOrders", statusOrdersSubmit.get());
            resultObject.put("typeOrders", typeOrdersSubmit.get());
            resultObject.put("resourceOrders", resourceOrdersSubmit.get());
            return new BocloudResult(true, resultObject, "查询成功");
        } catch (Exception e) {
            logger.error("get order analysis overview error,", e);
            return BocloudResult.FAILED("查询失败！");
        }
    }

    @Override
    public BocloudResult ordersCount() {
        try {
            List<JSONObject> list = new ArrayList<>();
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime dayBegin = LocalDateTime.of(now.getYear(), now.getMonth(), now.getDayOfMonth(), 0, 0, 0, 0);
            //获取总订单
            int totalOrders = serviceOrderRepository.listByVendor(null, DateUtil.localDateTimeFormat(now), null, null).size();
            JSONObject totalNum = new JSONObject();
            totalNum.put("name", "总订单数");
            totalNum.put("value", totalOrders);
            list.add(totalNum);
            //获取月订单
            LocalDateTime monthBegin = LocalDateTime.of(now.getYear(), now.getMonth(), 1, 0, 0, 0, 0);
            int monthOrders = serviceOrderRepository
                    .listByVendor(DateUtil.localDateTimeFormat(monthBegin), DateUtil.localDateTimeFormat(now), null, null).size();
            JSONObject monthNum = new JSONObject();
            monthNum.put("name", "本月订单数");
            monthNum.put("value", monthOrders);
            list.add(monthNum);
            return new BocloudResult(true, list, "查询成功");
        } catch (Exception e) {
            logger.error("get count of orders error,", e);
            return BocloudResult.FAILED("查询失败！");
        }
    }

    @Override
    public BocloudResult vendorOrderAnalyse(String condition, String time, String startTime, String endTime) {
        if ("countByVendor".equals(condition)) {
            return this.getVendorOrders(startTime, endTime);
        }
        if ("getTop5".equals(condition)) {
            return this.getVendorOrdersTop5(startTime, endTime);
        }
        if ("getTrend".equals(condition)) {
            Map<String, Object> timeMap = MapTools.simpleMap("time", time);
            timeMap.put("startTime", startTime);
            timeMap.put("endTime", endTime);
            List<String> timeList = DateUtil.timeToList(new JSONObject(timeMap));
            if (timeList.size() <= 1) {
                return BocloudResult.FAILED("时间区间选择太短，请重新选择");
            }
            return this.getVendorOrdersTrend(timeList);
        }
        return BocloudResult.FAILED("不支持的查询类型");
    }

    /**
     * 获取平台订单总数
     *
     * @param start
     * @param end
     * @param vendorId
     * @param context
     * @return
     */
    private JSONArray getVendorOrdersCount(String start, String end, Long vendorId, RequestContext context) {
        JSONArray resourceCount = new JSONArray();
        try {
            Map<String, List<ServiceOrder>> resourceService =
                    serviceOrderRepository.listByVendor(start, end, vendorId, context).stream().peek(serviceOrder -> {
                        if (serviceOrder.getVendorName() == null) {
                            serviceOrder.setVendorName("其他");
                        }
                    }).collect(groupingBy(ServiceOrder::getVendorName));
            Set<Map.Entry<String, List<ServiceOrder>>> resourceEntrySet = resourceService.entrySet();
            for (Map.Entry<String, List<ServiceOrder>> resourceEntry : resourceEntrySet) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("name", resourceEntry.getKey());
                jsonObject.put("value", resourceEntry.getValue().size());
                resourceCount.add(jsonObject);
            }
        } catch (Exception e) {
            logger.error("get vendor orders count error ", e);
        }
        return resourceCount;
    }

    /**
     * 获取不同平台服务订单数
     *
     * @param start
     * @param end
     * @return
     */
    private BocloudResult getVendorOrders(String start, String end) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            Map<String, List<ServiceOrder>> ordersMap =
                    serviceOrderRepository.listByVendor(start, end, null, null).stream().peek(serviceOrder -> {
                        if (serviceOrder.getVendorName() == null) {
                            serviceOrder.setVendorName("其他");
                        }
                    }).collect(groupingBy(ServiceOrder::getVendorName));
            ArrayList<Map.Entry<String, List<ServiceOrder>>> entryArrayList = new ArrayList<>(ordersMap.entrySet());
            List<Object> data = new ArrayList<>();
            for (Map.Entry<String, List<ServiceOrder>> entry : entryArrayList) {
                keys.add(entry.getKey());
                data.add(entry.getValue().size());
            }
            ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
            values.add(chargeTrendData);
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get vendor service orders top5 error,", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    /**
     * 获取平台订单Top5
     *
     * @param start
     * @param end
     * @return
     */
    private BocloudResult getVendorOrdersTop5(String start, String end) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            Map<String, List<ServiceOrder>> ordersMap =
                    serviceOrderRepository.listByTenantId(start, end, null).stream().peek(serviceOrder -> {
                        if (serviceOrder.getVendorName() == null) {
                            serviceOrder.setVendorName("其他");
                        }
                    }).collect(groupingBy(ServiceOrder::getVendorName));
            ArrayList<Map.Entry<String, List<ServiceOrder>>> entryArrayList = new ArrayList<>(ordersMap.entrySet());
            //根据订单量排序取前5
            List<Map.Entry<String, List<ServiceOrder>>> sortedEntry =
                    entryArrayList.stream().sorted(((o1, o2) -> o2.getValue().size() - o1.getValue().size())).limit(5)
                            .collect(Collectors.toList());
            ArrayList<Object> data = new ArrayList<>();
            for (Map.Entry<String, List<ServiceOrder>> entry : sortedEntry) {
                keys.add(entry.getKey());
                data.add(entry.getValue().size());
            }
            ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
            values.add(chargeTrendData);
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get vendor service orders top5 error,", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    /**
     * 获取平台订单增长趋势
     *
     * @return
     */
    private BocloudResult getVendorOrdersTrend(List<String> timeList) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            keys.addAll(timeList);
            HashMap<String, List<Object>> countMap = new HashMap<>();
            List<CloudVendor> vendorList = imsInternalService.getVendorList();
            ArrayList<Object> total = new ArrayList<>();
            List<Future<List<ServiceOrder>>> futures = new ArrayList<>();
            for (String time : timeList) {
                Future<List<ServiceOrder>> future = taskExecutor
                        .submit(() -> serviceOrderRepository.listByVendor(null, DateUtil.timeProcess(time), null, null));
                futures.add(future);
            }
            for (Future<List<ServiceOrder>> future : futures) {
                List<ServiceOrder> orderList = future.get();
                long totalNum = 0L;
                Map<String, List<ServiceOrder>> serviceOrders = orderList.stream().peek(serviceOrder -> {
                    if (serviceOrder.getVendorName() == null) {
                        serviceOrder.setVendorName("其他");
                    }
                }).collect(groupingBy(ServiceOrder::getVendorName));
                for (CloudVendor cloudVendor : vendorList) {
                    String vendorName = cloudVendor.getName();
                    List<ServiceOrder> orders =
                            Optional.ofNullable(serviceOrders.get(vendorName)).orElse(new ArrayList<>());
                    totalNum += orders.size();
                    //统计过了则更新 数据列表 没有统计的则新增数据列表
                    if (countMap.containsKey(vendorName)) {
                        List<Object> countNum = countMap.get(vendorName);
                        countNum.add(orders.size());
                        countMap.put(vendorName, countNum);
                    } else {
                        ArrayList<Object> countNum = new ArrayList<>();
                        countNum.add(orders.size());
                        countMap.put(vendorName, countNum);
                    }
                }
                List<ServiceOrder> unKnowOrders =
                        Optional.ofNullable(serviceOrders.get("其他")).orElse(new ArrayList<>());
                totalNum += unKnowOrders.size();
                //统计过了则更新 数据列表 没有统计的则新增数据列表
                if (countMap.containsKey("其他")) {
                    List<Object> countNum = countMap.get("其他");
                    countNum.add(unKnowOrders.size());
                    countMap.put("其他", countNum);
                } else {
                    ArrayList<Object> countNum = new ArrayList<>();
                    countNum.add(unKnowOrders.size());
                    countMap.put("其他", countNum);
                }
                total.add(totalNum);
            }
            //计算总数
            for (Map.Entry<String, List<Object>> entry : countMap.entrySet()) {
                ChargeTrendData chargeTrendData = new ChargeTrendData(entry.getKey(), entry.getValue());
                values.add(chargeTrendData);
            }
            values.add(new ChargeTrendData("合计(个)", total));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get vendor orders count error ", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }

    }

    @Override
    public BocloudResult tenantOrderAnalyse(Long tenantId, String condition, String time, String startTime,
                                            String endTime) {
        if ("getTop5".equals(condition)) {
            return this.getTenantOrdersTop5(startTime, endTime);
        }
        if ("getTrend".equals(condition)) {
            List<String> timeList = DateUtil.timeToList(new JSONObject(MapTools.simpleMap("time", time)));
            if (timeList.size() <= 1) {
                return BocloudResult.FAILED("时间区间选择太短，请重新选择");
            }
            return this.getTenantOrdersTrend(timeList);
        }
        if ("countByVendor".equals(condition)) {
            return this.getTenantOrdersByVendor(startTime, endTime, tenantId);
        }
        return BocloudResult.FAILED("不支持的查询类型");
    }

    /**
     * 获取租户订单总数
     *
     * @param start
     * @param end
     * @param vendorId
     * @param context
     * @return
     */
    private JSONArray getTenantOrdersCount(String start, String end, Long vendorId, RequestContext context) {
        JSONArray resourceCount = new JSONArray();
        try {
            Map<Long, List<ServiceOrder>> resourceService =
                    serviceOrderRepository.listByVendor(start, end, vendorId, context).stream()
                            .filter(serviceOrder -> serviceOrder.getTenantId() != null)
                            .collect(groupingBy(ServiceOrder::getTenantId));
            Set<Long> tenantIds = resourceService.keySet();
            Map<Long, String> tenantMap = tenantService.listByIds(new ArrayList<>(tenantIds), "NORMAL").stream()
                    .collect(toMap(Tenant::getId, Tenant::getName));
            for (Map.Entry<Long, List<ServiceOrder>> tenantEntry : resourceService.entrySet()) {
                JSONObject jsonObject = new JSONObject();
                Long tenantId = tenantEntry.getKey();
                List<ServiceOrder> orders = tenantEntry.getValue();
                jsonObject.put("name", tenantMap.get(tenantId));
                jsonObject.put("value", orders.size());
                resourceCount.add(jsonObject);
            }
        } catch (Exception e) {
            logger.error("get tenant orders count error ", e);
        }
        return resourceCount;
    }

    /**
     * 获取租户订单top5
     *
     * @param start
     * @param end
     * @return
     */
    private BocloudResult getTenantOrdersTop5(String start, String end) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            //查询所有订单  并且按订单内租户id分组
            List<ServiceOrder> serviceOrders = serviceOrderRepository.listByTenantId(start, end, null);
            Map<Long, List<ServiceOrder>> orderMap =
                    serviceOrders.stream().filter(order -> order.getTenantId() != null && 0L != order.getTenantId())
                            .collect(groupingBy(ServiceOrder::getTenantId));
            Set<Long> tenantIds = orderMap.keySet();
            //获取所有一级租户信息 并收集他的子租户id和他自身id
            Map<Long, String> tenantsMap = tenantService.listByIds(new ArrayList<>(tenantIds), "").stream()
                    .collect(Collectors.toMap(Tenant::getId, Tenant::getName));

            HashMap<String, List<ServiceOrder>> tenantOrderMap = new HashMap<>();
            for (Map.Entry<Long, List<ServiceOrder>> orderEntry : orderMap.entrySet()) {
                Long tenantId = orderEntry.getKey();
                List<ServiceOrder> tenantOrders = orderEntry.getValue();
                tenantOrderMap.put(tenantsMap.get(tenantId), tenantOrders);
            }
            //根据订单量排序取前5
            List<Map.Entry<String, List<ServiceOrder>>> sortedEntry =
                    tenantOrderMap.entrySet().stream().sorted((o1, o2) -> o2.getValue().size() - o1.getValue().size())
                            .limit(5).collect(Collectors.toList());
            List<Object> data = new ArrayList<>();
            for (Map.Entry<String, List<ServiceOrder>> entry : sortedEntry) {
                keys.add(entry.getKey());
                data.add(entry.getValue().size());
            }
            ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
            values.add(chargeTrendData);
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get tenant service orders top5 error,", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    /**
     * 获取租户订单趋势
     *
     * @return
     */
    private BocloudResult getTenantOrdersTrend(List<String> timeList) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            //计数完后初始化开始时间
            keys.addAll(timeList);
            HashMap<String, List<Object>> countMap = new HashMap<>();
            List<Future<List<ServiceOrder>>> futures = new ArrayList<>();
            List<Tenant> tenants = tenantService.list(new Pager(1, Integer.MAX_VALUE, null, null));
            Map<Long, String> tenantMap = tenants.stream().collect(toMap(Tenant::getId, Tenant::getName));
            Set<Long> tenantSet = serviceOrderRepository.listByTenantId(timeList.get(0).substring(0, 10) + " 00:00:00",
                    timeList.get(timeList.size() - 1).substring(0, 10) + " 23:59:59", null).stream()
                    .map(ServiceOrder::getTenantId).collect(Collectors.toSet());
            for (String time : timeList) {
                Future<List<ServiceOrder>> future = taskExecutor
                        .submit(() -> serviceOrderRepository.listByTenantId(null, DateUtil.timeProcess(time), null));
                futures.add(future);
            }
            ArrayList<Object> total = new ArrayList<>();
            for (Future<List<ServiceOrder>> future : futures) {
                List<ServiceOrder> orders = future.get();
                Map<Long, List<ServiceOrder>> serviceOrders =
                        orders.stream().filter(serviceOrder -> serviceOrder.getTenantId() != null)
                                .collect(groupingBy(ServiceOrder::getTenantId));
                long totalNum = 0L;
                for (Long tenantId : tenantSet) {
                    String tenantName = tenantMap.get(tenantId);
                    List<ServiceOrder> tenantOrders =
                            Optional.ofNullable(serviceOrders.get(tenantId)).orElse(new ArrayList<>());
                    //统计过了则更新 数据列表 没有统计的则新增数据列表
                    totalNum += tenantOrders.size();
                    if (countMap.containsKey(tenantName)) {
                        List<Object> countNum = countMap.get(tenantName);
                        countNum.add(tenantOrders.size());
                        countMap.put(tenantName, countNum);
                    } else {
                        ArrayList<Object> countNum = new ArrayList<>();
                        countNum.add(tenantOrders.size());
                        countMap.put(tenantName, countNum);
                    }
                }
                total.add(totalNum);
            }
            for (Map.Entry<String, List<Object>> entry : countMap.entrySet()) {
                values.add(new ChargeTrendData(entry.getKey(), entry.getValue()));
            }
            values.add(new ChargeTrendData("合计(个)", total));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get tenant orders  count error ", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }

    }

    /**
     * 获取租户不同平台订单数统计
     *
     * @param start
     * @param end
     * @param tenantId
     * @return
     */
    private BocloudResult getTenantOrdersByVendor(String start, String end, Long tenantId) {

        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            Map<String, List<ServiceOrder>> ordersMap =
                    serviceOrderRepository.listByTenantId(start, end, tenantId).stream().peek(serviceOrder -> {
                        if (serviceOrder.getVendorName() == null) {
                            serviceOrder.setVendorName("其他");
                        }
                    }).collect(groupingBy(ServiceOrder::getVendorName));
            Set<Map.Entry<String, List<ServiceOrder>>> entries = ordersMap.entrySet();
            List<Object> data = new ArrayList<>();
            for (Map.Entry<String, List<ServiceOrder>> entry : entries) {
                keys.add(entry.getKey());
                data.add(entry.getValue().size());
            }
            ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
            values.add(chargeTrendData);
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get tenant orders by vendor error," + e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    @Override
    public BocloudResult orderStatusAnalyse(String time, String startTime, String endTime, String status,
                                            String condition) {
        if ("getTrend".equals(condition)) {
            Map<String, Object> timeMap = MapTools.simpleMap("time", time);
            timeMap.put("startTime", startTime);
            timeMap.put("endTime", endTime);
            List<String> timeList = DateUtil.timeToList(new JSONObject(timeMap));
            if (timeList.size() <= 1) {
                return BocloudResult.FAILED("时间区间选择太短，请重新选择");
            }
            return this.getOrderStatusTrend(timeList);
        } else if ("countByVendor".equals(condition)) {
            return this.getOrderStatusByVendor(startTime, endTime, status);
        }
        return new BocloudResult(false, null, "查询失败");
    }

    /**
     * 获取订单状态统计
     *
     * @param start
     * @param end
     * @param vendorId
     * @param context
     * @return
     */
    private JSONArray getOrderStatusCount(String start, String end, Long vendorId, RequestContext context) {
        JSONArray resourceCount = new JSONArray();
        try {
            Map<OrderStatus, List<ServiceOrder>> resourceService =
                    serviceOrderRepository.listByVendor(start, end, vendorId, context).stream()
                            .collect(groupingBy(ServiceOrder::getOrderStatus));
            for (OrderStatus status : OrderStatus.values()) {
                JSONObject jsonObject = new JSONObject();
                List<ServiceOrder> orders = Optional.ofNullable(resourceService.get(status)).orElse(new ArrayList<>());
                switch (status) {
                    case paid:
                        jsonObject.put("name", "已支付");
                        break;
                    case audit:
                        jsonObject.put("name", "审批中");
                        break;
                    case unpay:
                        jsonObject.put("name", "未支付");
                        break;
                    case canceld:
                        jsonObject.put("name", "已取消");
                        break;
                    default:
                        break;
                }
                jsonObject.put("value", orders.size());
                resourceCount.add(jsonObject);
            }
        } catch (Exception e) {
            logger.error("get orders status count error ", e);
        }
        return resourceCount;
    }

    /**
     * 获取订单状态趋势
     *
     * @return
     */
    private BocloudResult getOrderStatusTrend(List<String> timeList) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            keys.addAll(timeList);
            HashMap<String, List<Object>> countMap = new HashMap<>();
            List<Future<List<ServiceOrder>>> futures = new ArrayList<>();
            for (String time : timeList) {
                Future<List<ServiceOrder>> future =
                        taskExecutor.submit(() -> serviceOrderRepository.listByStatus(null, DateUtil.timeProcess(time), null));
                futures.add(future);
            }
            ArrayList<Object> total = new ArrayList<>();
            for (Future<List<ServiceOrder>> future : futures) {
                List<ServiceOrder> orderList = future.get();
                total.add(orderList.size());
                Map<OrderStatus, List<ServiceOrder>> statusListMap =
                        orderList.stream().collect(groupingBy(ServiceOrder::getOrderStatus));
                List<ServiceOrder> unPayOrders =
                        Optional.ofNullable(statusListMap.get(unpay)).orElse(new ArrayList<>());
                if (countMap.containsKey("未支付")) {
                    List<Object> countNum = countMap.get("未支付");
                    countNum.add(unPayOrders.size());
                    countMap.put("未支付", countNum);
                } else {
                    ArrayList<Object> countNum = new ArrayList<>();
                    countNum.add(unPayOrders.size());
                    countMap.put("未支付", countNum);
                }
                List<ServiceOrder> paidOrders = Optional.ofNullable(statusListMap.get(paid)).orElse(new ArrayList<>());
                if (countMap.containsKey("已支付")) {
                    List<Object> countNum = countMap.get("已支付");
                    countNum.add(paidOrders.size());
                    countMap.put("已支付", countNum);
                } else {
                    ArrayList<Object> countNum = new ArrayList<>();
                    countNum.add(paidOrders.size());
                    countMap.put("已支付", countNum);
                }
                List<ServiceOrder> cancelOrders =
                        Optional.ofNullable(statusListMap.get(canceld)).orElse(new ArrayList<>());
                if (countMap.containsKey("已取消")) {
                    List<Object> countNum = countMap.get("已取消");
                    countNum.add(cancelOrders.size());
                    countMap.put("已取消", countNum);
                } else {
                    ArrayList<Object> countNum = new ArrayList<>();
                    countNum.add(cancelOrders.size());
                    countMap.put("已取消", countNum);
                }
                List<ServiceOrder> auditOrders =
                        Optional.ofNullable(statusListMap.get(audit)).orElse(new ArrayList<>());
                if (countMap.containsKey("审批中")) {
                    List<Object> countNum = countMap.get("审批中");
                    countNum.add(auditOrders.size());
                    countMap.put("审批中", countNum);
                } else {
                    ArrayList<Object> countNum = new ArrayList<>();
                    countNum.add(auditOrders.size());
                    countMap.put("审批中", countNum);
                }
            }
            for (Map.Entry<String, List<Object>> entry : countMap.entrySet()) {
                values.add(new ChargeTrendData(entry.getKey(), entry.getValue()));
            }
            values.add(new ChargeTrendData("合计(个)", total));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get orders type count error ", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    /**
     * 获取平台订单状态统计
     *
     * @param start
     * @param end
     * @param status
     * @return
     */
    private BocloudResult getOrderStatusByVendor(String start, String end, String status) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            Map<String, List<ServiceOrder>> typeOrders =
                    serviceOrderRepository.listByStatus(start, end, status).stream().peek(serviceOrder -> {
                        if (serviceOrder.getVendorName() == null) {
                            serviceOrder.setVendorName("其他");
                        }
                    }).collect(groupingBy(ServiceOrder::getVendorName));
            Set<Map.Entry<String, List<ServiceOrder>>> typeEntries = typeOrders.entrySet();
            List<Object> data = new ArrayList<>();
            for (Map.Entry<String, List<ServiceOrder>> entry : typeEntries) {
                keys.add(entry.getKey());
                data.add(entry.getValue().size());
            }
            ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
            values.add(chargeTrendData);
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get service order status by vendor error," + e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    @Override
    public BocloudResult orderTypeAnalyse(String time, String startTime, String endTime, String type,
                                          String condition) {
        if ("getTrend".equals(condition)) {
            Map<String, Object> timeMap = MapTools.simpleMap("time", time);
            timeMap.put("startTime", startTime);
            timeMap.put("endTime", endTime);
            List<String> timeList = DateUtil.timeToList(new JSONObject(timeMap));
            if (timeList.size() <= 1) {
                return BocloudResult.FAILED("时间区间选择太短，请重新选择");
            }
            return this.getOrderTypeTrend(timeList);
        } else if ("countByVendor".equals(condition)) {
            return this.getOrderTypeByVendor(startTime, endTime, type);
        }
        return new BocloudResult(false, null, "查询失败");
    }

    /**
     * 获取订单类型统计
     *
     * @param start
     * @param end
     * @param vendorId
     * @param context
     * @return
     */
    private JSONArray getOrderTypeCount(String start, String end, Long vendorId, RequestContext context) {
        JSONArray resourceCount = new JSONArray();
        try {
            Map<ChargeCommon.ChargeMode, List<ServiceOrder>> resourceService =
                    serviceOrderRepository.listByVendor(start, end, vendorId, context).stream()
                            .collect(groupingBy(ServiceOrder::getChargeMode));
            Set<Map.Entry<ChargeCommon.ChargeMode, List<ServiceOrder>>> resourceEntrySet = resourceService.entrySet();
            for (Map.Entry<ChargeCommon.ChargeMode, List<ServiceOrder>> resourceEntry : resourceEntrySet) {
                JSONObject jsonObject = new JSONObject();
                if (PreCharge.equals(resourceEntry.getKey())) {
                    jsonObject.put("name", "预付费");
                } else {
                    jsonObject.put("name", "后付费");
                }
                jsonObject.put("value", resourceEntry.getValue().size());
                resourceCount.add(jsonObject);
            }
        } catch (Exception e) {
            logger.error("get orders type count error ", e);
        }
        return resourceCount;
    }

    /**
     * 获取订单类型趋势
     *
     * @return
     */
    private BocloudResult getOrderTypeTrend(List<String> timeList) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            keys.addAll(timeList);
            HashMap<String, List<Object>> countMap = new HashMap<>();
            List<Future<List<ServiceOrder>>> futures = new ArrayList<>();
            for (String time : timeList) {
                Future<List<ServiceOrder>> future =
                        taskExecutor.submit(() -> serviceOrderRepository.listByStatus(null, DateUtil.timeProcess(time), null));
                futures.add(future);
            }
            ArrayList<Object> total = new ArrayList<>();
            for (Future<List<ServiceOrder>> future : futures) {
                List<ServiceOrder> orderList = future.get();
                total.add(orderList.size());
                Map<ChargeCommon.ChargeMode, List<ServiceOrder>> chargeModeListMap =
                        orderList.stream().collect(groupingBy(ServiceOrder::getChargeMode));
                List<ServiceOrder> postChargeOrders =
                        Optional.ofNullable(chargeModeListMap.get(PostCharge)).orElse(new ArrayList<>());
                List<ServiceOrder> preChargeOrders =
                        Optional.ofNullable(chargeModeListMap.get(PreCharge)).orElse(new ArrayList<>());
                if (countMap.containsKey("后付费")) {
                    List<Object> countNum = countMap.get("后付费");
                    countNum.add(postChargeOrders.size());
                    countMap.put("后付费", countNum);
                } else {
                    ArrayList<Object> countNum = new ArrayList<>();
                    countNum.add(preChargeOrders.size());
                    countMap.put("后付费", countNum);
                }
                if (countMap.containsKey("预付费")) {
                    List<Object> countNum = countMap.get("预付费");
                    countNum.add(preChargeOrders.size());
                    countMap.put("预付费", countNum);
                } else {
                    ArrayList<Object> countNum = new ArrayList<>();
                    countNum.add(preChargeOrders.size());
                    countMap.put("预付费", countNum);
                }
            }
            for (Map.Entry<String, List<Object>> entry : countMap.entrySet()) {
                values.add(new ChargeTrendData(entry.getKey(), entry.getValue()));
            }
            values.add(new ChargeTrendData("合计(个)", total));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get orders type count error ", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    /**
     * 获取订单类型列表
     *
     * @param start
     * @param end
     * @param type
     * @return
     */
    private BocloudResult getOrderTypeByVendor(String start, String end, String type) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            Map<String, List<ServiceOrder>> typeOrders =
                    serviceOrderRepository.listByType(start, end, type).stream().peek(serviceOrder -> {
                        if (serviceOrder.getVendorName() == null) {
                            serviceOrder.setVendorName("其他");
                        }
                    }).collect(groupingBy(ServiceOrder::getVendorName));
            Set<Map.Entry<String, List<ServiceOrder>>> typeEntries = typeOrders.entrySet();
            List<Object> data = new ArrayList<>();
            for (Map.Entry<String, List<ServiceOrder>> entry : typeEntries) {
                keys.add(entry.getKey());
                data.add(entry.getValue().size());
            }
            ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
            values.add(chargeTrendData);
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get service order type by vendor error," + e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    @Override
    public BocloudResult resourceOrderAnalyse(String time, String startTime, String endTime, String serviceCode,
                                              String condition) {
        if ("getTrend".equals(condition)) {
            Map<String, Object> timeMap = MapTools.simpleMap("time", time);
            timeMap.put("startTime", startTime);
            timeMap.put("endTime", endTime);
            List<String> timeList = DateUtil.timeToList(new JSONObject(timeMap));
            if (timeList.size() <= 1) {
                return BocloudResult.FAILED("时间区间选择太短，请重新选择");
            }
            return this.getResourceOrdersTrend(timeList);
        }
        if ("getTop5".equals(condition)) {
            return this.getResourceOrdersTop5(startTime, endTime, serviceCode);
        }
        if ("countByVendor".equals(condition)) {
            return this.getResourceOrdersByVendor(startTime, endTime, serviceCode);
        }
        return BocloudResult.FAILED("不支持的查询类型");
    }

    /**
     * 获取资源订单统计
     *
     * @param start
     * @param end
     * @param vendorId
     * @param context
     * @return
     */
    private JSONArray getResourceOrdersCount(String start, String end, Long vendorId, RequestContext context) {
        JSONArray resourceCount = new JSONArray();
        try {
            Map<String, List<ServiceOrder>> resourceService =
                    serviceOrderRepository.listByVendor(start, end, vendorId, context).stream()
                            .collect(groupingBy(ServiceOrder::getServiceName));
            Set<Map.Entry<String, List<ServiceOrder>>> resourceEntrySet = resourceService.entrySet();
            for (Map.Entry<String, List<ServiceOrder>> resourceEntry : resourceEntrySet) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("name", resourceEntry.getKey());
                jsonObject.put("value", resourceEntry.getValue().size());
                resourceCount.add(jsonObject);
            }
        } catch (Exception e) {
            logger.error("get resource orders count error ", e);
        }
        return resourceCount;
    }

    /**
     * 获取资源订单增长趋势
     *
     * @return
     */
    private BocloudResult getResourceOrdersTrend(List<String> timeList) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            keys.addAll(timeList);
            List<ServiceOrder> allServiceOrders = serviceOrderRepository
                    .listByCode(null, timeList.get(timeList.size() - 1).substring(0, 10) + " 23:59:59", null);
            List<String> serviceName =
                    allServiceOrders.stream().map(ServiceOrder::getServiceName).distinct().collect(Collectors.toList());
            HashMap<String, List<Object>> countMap = new HashMap<>();
            ArrayList<Object> total = new ArrayList<>();
            for (String time : timeList) {
                List<ServiceOrder> orderList = allServiceOrders.stream().filter(
                        serviceOrder -> serviceOrder.getGmtCreate()
                                .before(DateUtil.timeDateProcess(time)))
                        .collect(Collectors.toList());
                total.add(orderList.size());
                Map<String, List<ServiceOrder>> serviceOrderMap =
                        orderList.stream().filter(order -> order.getServiceName() != null).collect(groupingBy(ServiceOrder::getServiceName));
                for (String name : serviceName) {
                    int orderNum = Optional.ofNullable(serviceOrderMap.get(name)).orElse(new ArrayList<>()).size();
                    if (countMap.containsKey(name)) {
                        List<Object> countNum = countMap.get(name);
                        countNum.add(orderNum);
                        countMap.put(name, countNum);
                    } else {
                        ArrayList<Object> countNum = new ArrayList<>();
                        countNum.add(orderNum);
                        countMap.put(name, countNum);
                    }
                }
            }
            for (Map.Entry<String, List<Object>> entry : countMap.entrySet()) {
                values.add(new ChargeTrendData(entry.getKey(), entry.getValue()));
            }
            values.add(new ChargeTrendData("合计(个)", total));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get resource orders count error ", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    /**
     * 获取资源订单Top5
     *
     * @param start
     * @param end
     * @param serviceCode
     * @return
     */
    private BocloudResult getResourceOrdersTop5(String start, String end, String serviceCode) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            Map<String, List<ServiceOrder>> resourceOrders =
                    serviceOrderRepository.listByCode(start, end, serviceCode).stream()
                            .collect(groupingBy(ServiceOrder::getServiceName));
            List<List<ServiceOrder>> collect =
                    resourceOrders.values().stream().sorted((o1, o2) -> o2.size() - o1.size()).limit(5)
                            .collect(Collectors.toList());
            List<Object> data = new ArrayList<>();
            for (List<ServiceOrder> serviceOrders : collect) {
                keys.add(serviceOrders.get(0).getServiceName());
                data.add(serviceOrders.size());
            }
            ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
            values.add(chargeTrendData);
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get resource orders top5 error,", e);
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    /**
     * 获取资源订单列表
     *
     * @param start
     * @param end
     * @param serviceCode
     * @return
     */
    private BocloudResult getResourceOrdersByVendor(String start, String end, String serviceCode) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            Map<String, List<ServiceOrder>> vendorOrders =
                    serviceOrderRepository.listByCode(start, end, serviceCode).stream().peek(serviceOrder -> {
                        if (serviceOrder.getVendorName() == null) {
                            serviceOrder.setVendorName("其他");
                        }
                    }).collect(groupingBy(ServiceOrder::getVendorName));
            Set<Map.Entry<String, List<ServiceOrder>>> vendorOrderEntry = vendorOrders.entrySet();
            List<Object> data = new ArrayList<>();
            for (Map.Entry<String, List<ServiceOrder>> entry : vendorOrderEntry) {
                keys.add(entry.getKey());
                data.add(entry.getValue().size());
            }
            ChargeTrendData chargeTrendData = new ChargeTrendData("订单数量(个)", data);
            values.add(chargeTrendData);
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get resource orders by vendor error,", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }
}
