package com.bocloud.cos.service;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.IDFactory;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.Payment;
import com.bocloud.cos.intf.PaymentService;
import com.bocloud.cos.repository.PaymentRepository;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 支付记录业务实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年3月10日
 */
@Service
public class PaymentServiceImpl implements PaymentService {

    private static Logger logger = LoggerFactory.getLogger(PaymentServiceImpl.class);
    private final PaymentRepository paymentRepository;
    private final LockFactory lockFactory;

    public PaymentServiceImpl(PaymentRepository paymentRepository, LockFactory lockFactory) {
        this.paymentRepository = paymentRepository;
        this.lockFactory = lockFactory;
    }

    @Override
    public BocloudResult<GridBean<Payment>> list(Pager pager, RequestContext context) {
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            if (Catalog.Tenant == context.getCatalog()) {
                params.add(new Param(MapTools.simpleMap(Common.TENANTID, context.getTenant()), Sign.EQ));
            } else if (Catalog.User == context.getCatalog()) {
                params.add(new Param(MapTools.simpleMap(Common.USERID, context.getTarget()), Sign.EQ));
            }
            int total = paymentRepository.count(params);
            List<Payment> list = paymentRepository.list(pager.getPage(), pager.getRows(), sorter, params);
            GridBean<Payment> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result create(Payment payment, RequestContext context) {
        try {
            payment.setSn(IDFactory.instance().timestamp());
            payment.setGmtCreate(new Date());
            payment.setUserId(context.getTarget());
            payment.setTenantId(context.getTenant());
            paymentRepository.save(payment);
            return new Result(true, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message,", e);
            return new Result(false, "创建失败");
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result modify(Payment payment, RequestContext context) {
        String path = Payment.class.getSimpleName() + "_" + payment.getId();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Payment database = paymentRepository.query(payment.getId());
            Assert.notNull(database, "数据不存在！");
            BeanUtils.copyProperties(payment, database);
            database.setGmtModify(new Date());
            boolean flag = paymentRepository.update(database);
            if (!flag) {
                return new Result(false, "修改失败");
            }
            return new Result(true, "修改成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    public BocloudResult<Payment> detail(Long id) {
        try {
            Payment payment = paymentRepository.query(id);
            if (null == payment) {
                return new BocloudResult<>(false, "数据不存在");
            }
            return new BocloudResult<>(true, payment, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result remove(Long id, RequestContext context) {
        String path = Payment.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Payment payment = paymentRepository.query(id);
            Assert.notNull(payment, "数据不存在！");
            Boolean flag = paymentRepository.remove(id);
            return flag ? new Result(true, "删除成功") : new Result(false, "删除失败");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, "删除失败");
        }
    }
}
