package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.ServiceOrder;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.enums.*;
import com.bocloud.cos.enums.ChargeCommon.ChargeMode;
import com.bocloud.cos.internal.FmsFlowService;
import com.bocloud.cos.internal.ImsInternalService;
import com.bocloud.cos.internal.SmsTenantService;
import com.bocloud.cos.intf.ResourceExtensionService;
import com.bocloud.cos.intf.ResourceQuotaService;
import com.bocloud.cos.intf.ServiceOrderService;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.DurationModel.Mode;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.ExtensionParam;
import com.bocloud.cos.operate.*;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.utils.IdGenerator;
import com.bocloud.cos.internel.entity.CloudServer;
import com.bocloud.cos.internel.entity.Tenant;
import com.bocloud.cos.internel.entity.Volume;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 服务操作业务实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年4月15日
 */
@Service
public class ResourceExtensionServiceImpl implements ResourceExtensionService {

    private static final Logger logger = LoggerFactory.getLogger(ResourceExtensionServiceImpl.class);
    private final ServiceOrderRepository serviceOrderRepository;
    private final OrderItemRepository orderItemRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final ResourceQuotaRepository resourceQuotaRepository;
    private final SmsTenantService tenantService;
    private final ResourceQuotaService resourceQuotaService;
    private final ServiceBillRepository billRepository;
    private final ImsInternalService imsInternalService;
    private final LockFactory lockFactory;
    private final ServiceFlowConfigRepository serviceFlowConfigRepository;
    private final FmsFlowService fmsFlowService;
    private final ServiceOrderService orderService;

    public ResourceExtensionServiceImpl(ServiceOrderRepository serviceOrderRepository, OrderItemRepository orderItemRepository,
                                        CloudServiceRepository cloudServiceRepository, ServiceSkuRepository serviceSkuRepository,
                                        ResourceQuotaRepository resourceQuotaRepository, SmsTenantService tenantService,
                                        ResourceQuotaService resourceQuotaService, ServiceBillRepository billRepository,
                                        ImsInternalService imsInternalService, LockFactory lockFactory,
                                        ServiceFlowConfigRepository serviceFlowConfigRepository, FmsFlowService fmsFlowService, ServiceOrderService orderService) {
        this.serviceOrderRepository = serviceOrderRepository;
        this.orderItemRepository = orderItemRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.serviceSkuRepository = serviceSkuRepository;
        this.resourceQuotaRepository = resourceQuotaRepository;
        this.tenantService = tenantService;
        this.resourceQuotaService = resourceQuotaService;
        this.billRepository = billRepository;
        this.imsInternalService = imsInternalService;
        this.lockFactory = lockFactory;
        this.serviceFlowConfigRepository = serviceFlowConfigRepository;
        this.fmsFlowService = fmsFlowService;
        this.orderService = orderService;
    }


    private Map<String, Object> getParams(String category, String params, String theme, RequestContext context) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("ApplyCategory", category);
        map.put("ApplyParams", params);
        map.put("ApplyTheme", theme);
        map.put("ApplyContext", context);
        return map;
    }

    @Override
    public BocloudResult process(ExtensionParam extensionParam, RequestContext context) {
        try {
            // 服务资源实例id
            String instance = extensionParam.getInstance();
            /** 根据sn查询是否有处于中间状态的订单项（如变更中、延期中、退订中等），如有，则直接RETURN **/
            String result = orderService.underwayCheck(instance);
            if (null != result) {
                return new BocloudResult<>(false, result);
            }

            OrderItem item = orderItemRepository.queryBySn(instance);
            Assert.notNull(item, "订单明细未找到");
            ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
            //延期时长
            Integer extension = extensionParam.getExtension();
            DurationModel duration = new DurationModel(extensionParam.getMode(), extension);
            Assert.notNull(order, "订单未找到");
            if (ChargeMode.PreCharge == order.getChargeMode()) {
                Long tenantId = null;
                Long userId = null;
                if (Catalog.User == context.getCatalog()) {
                    userId = context.getTarget();
                } else if (Catalog.Tenant == context.getCatalog()) {
                    tenantId = context.getTenant();
                }
                int count = billRepository
                        .count(ChargeCommon.BillStatus.unpay.name(), ChargeMode.PreCharge.name(), tenantId, userId,
                                order.getId());
                if (count > 0) {
                    return new BocloudResult<>(false, "当前订单存在未支付的账单，请先支付");
                }
            }
            Assert.isTrue(!ChargeMode.PostCharge.equals(order.getChargeMode()), "后付费订单不能进行延期操作");
            CloudService service = cloudServiceRepository.queryByCode(item.getServiceCode());
            Assert.notNull(service, "订单明细关联的服务未找到");
            extensionParam.setService(service.getCode());
            if (null != item.getSkuId() && 0L != item.getSkuId()) {
                ServiceSku sku = serviceSkuRepository.query(item.getSkuId());
                Assert.notNull(sku, "订单明细规格未找到");
            }
            ResourceOperate extensionOperate = ResourceOperateFactory.getInstance(extensionParam.getService());
            Assert.notNull(extensionOperate, "不支持延期操作");
            extensionParam.setContext(context);
            extensionParam.setCategory(ServiceResourceMapping.getResourceCategory(extensionParam.getService()).name());
            extensionParam.setOldStatus(item.getStatus());
            ServiceOrder extensionOrder = new ServiceOrder();
            extensionOrder.setServiceCode(extensionParam.getService());
            // 生成订单号，增加分布式锁
            String path = IdGenerator.class.getSimpleName() + "_ordersn";
            try (BocloudACLock lock = lockFactory.getACLock(path)) {
                Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
                extensionOrder.setSn(IdGenerator.getInstance().nextId());
            } catch (Exception e) {
                logger.error("Get error message：{}", e.getMessage());
                return new BocloudResult<>(false, "资源延期失败 : " + e.getMessage());
            }
            extensionOrder.setChargeMode(order.getChargeMode());
            extensionOrder.setSlaAgreed(order.getSlaAgreed());
            extensionOrder.setOrderStatus(ChargeCommon.OrderStatus.unpay);
            extensionOrder.setType(ChargeCommon.Type.extension);
            extensionOrder.setGmtCreate(new Date());
            extensionOrder.setAutoRenew(order.getAutoRenew());
            extensionOrder.setVendorId(order.getVendorId());
            extensionOrder.setBusinessId(order.getBusinessId());
            extensionOrder.setProjectId(order.getProjectId());
            extensionOrder.setUserId(order.getUserId());
            extensionOrder.setTenantId(order.getTenantId());
            extensionOrder.setResourceStatus(ChargeCommon.ResourceStatus.creating);
            extensionOrder.setApplyParam(JSONObject.toJSONString(extensionParam));
            serviceOrderRepository.save(extensionOrder);
            Tenant tenant = tenantService.detail(context.getTenant());
            if (extensionOperate instanceof PaasResourceOperate) {
                //找到所有资源订单项 做延期计费
                List<OrderItem> orderItems = orderItemRepository.listByOrderId(item.getOrderId());
                for (OrderItem orderItem : orderItems) {
                    ServiceSku serviceSku = serviceSkuRepository.query(orderItem.getSkuId());
                    CloudService cloudService = cloudServiceRepository.queryByCode(orderItem.getServiceCode());
                    List<ElementModel> elementModels =
                            JSONArray.parseArray(orderItem.getElements(), ElementModel.class);
                    ElementModel element = elementModels.get(0);
                    BigDecimal unitPrice =
                            OrderPriceUtils.getUnitPrice(cloudService, serviceSku, element.getSpecs(), duration);
                    Integer count = Optional.ofNullable(element.getCount()).orElse(1);
                    unitPrice = unitPrice.multiply(BigDecimal.valueOf(count));
                    OrderItem extensionItem = new OrderItem();
                    extensionItem.setCatalog(OrderItem.Catalog.complex);
                    extensionItem.setSkuId(orderItem.getSkuId());
                    extensionItem.setSn(orderItem.getSn());
                    extensionItem.setServiceCode(serviceSku.getServiceCode());
                    extensionItem.setStatus(OrderItemStatus.Extensioning);
                    DurationModel durationModel = new DurationModel();
                    durationModel.setMode(extensionParam.getMode());
                    durationModel.setAmount(extension);
                    extensionItem.setDuration(JSONObject.toJSONString(durationModel));
                    extensionItem.setDeleted(false);
                    extensionItem.setResources("[]");
                    extensionItem.setEffectiveTime(orderItem.getExpiredTime());

                    // 获取最后一次有效的订单项
                    OrderItem lastOrderItem = orderItemRepository.queryLastSuccessBySn(orderItem.getSn());
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(lastOrderItem.getExpiredTime());
                    if (duration.getMode().equals(Mode.Month)) {
                        calendar.add(Calendar.MONTH, extension);
                    } else {
                        calendar.add(Calendar.YEAR, extension);
                    }
                    extensionItem.setExpiredTime(calendar.getTime());
                    extensionItem.setBasicPrice(unitPrice);
                    extensionItem.setElements(lastOrderItem.getElements());
                    extensionItem.setFinalPrice(unitPrice.multiply(tenant.getDiscount()));
                    extensionItem.setOrderId(extensionOrder.getId());
                    orderItemRepository.save(extensionItem);
                }
            } else if (!(ServiceResourceMapping.getResourceCategory(service.getCode()) == ResourceCategory.Computer)) {
                OrderItem targetItem = orderItemRepository.queryLastSuccessBySn(instance);
                ServiceSku targetSku = serviceSkuRepository.query(targetItem.getSkuId());
                List<ElementModel> elementModels = JSONArray.parseArray(targetItem.getElements(), ElementModel.class);
                ElementModel element = elementModels.get(0);
                BigDecimal targetPrice = OrderPriceUtils.getUnitPrice(service, targetSku, element.getSpecs(), duration);
                OrderItem extensionItem = new OrderItem();
                extensionItem.setCatalog(OrderItem.Catalog.complex);
                extensionItem.setSkuId(targetItem.getSkuId());
                extensionItem.setSn(targetItem.getSn());
                extensionItem.setServiceCode(targetItem.getServiceCode());
                extensionItem.setStatus(OrderItemStatus.Extensioning);
                extensionItem.setElements(targetItem.getElements());
                DurationModel durationModel = new DurationModel();
                durationModel.setMode(extensionParam.getMode());
                extensionItem.setEffectiveTime(targetItem.getExpiredTime());
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(targetItem.getExpiredTime());
                if (duration.getMode().equals(Mode.Month)) {
                    calendar.add(Calendar.MONTH, extension);
                } else {
                    calendar.add(Calendar.YEAR, extension);
                }
                extensionItem.setExpiredTime(calendar.getTime());
                durationModel.setAmount(extension);
                extensionItem.setDuration(JSONObject.toJSONString(durationModel));
                extensionItem.setDeleted(false);
                extensionItem.setResources("[]");
                extensionItem.setBasicPrice(targetPrice);
                extensionItem.setFinalPrice(targetPrice.multiply(tenant.getDiscount()));
                extensionItem.setOrderId(extensionOrder.getId());
                orderItemRepository.save(extensionItem);
            } else {
                //找到对应机器和磁盘 做延期计费
                CloudServer server = imsInternalService.queryBySn(instance);
                OrderItem serverItem = orderItemRepository.queryLastSuccessBySn(instance);
                ServiceSku serverSku = serviceSkuRepository.query(serverItem.getSkuId());
                List<ElementModel> elementModels = JSONArray.parseArray(serverItem.getElements(), ElementModel.class);
                ElementModel element = elementModels.get(0);
                BigDecimal serverPrice = OrderPriceUtils.getUnitPrice(service, serverSku, element.getSpecs(), duration);
                OrderItem serverExtension = new OrderItem();
                serverExtension.setCatalog(OrderItem.Catalog.complex);
                serverExtension.setSkuId(server.getSkuId());
                serverExtension.setSn(server.getUuid());
                serverExtension.setServiceCode(serverSku.getServiceCode());
                serverExtension.setStatus(OrderItemStatus.Extensioning);
                serverExtension.setElements(serverItem.getElements());
                DurationModel durationModel = new DurationModel();
                durationModel.setMode(extensionParam.getMode());
                serverExtension.setEffectiveTime(serverItem.getExpiredTime());
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(serverItem.getExpiredTime());
                if (duration.getMode().equals(Mode.Month)) {
                    calendar.add(Calendar.MONTH, extension);
                } else {
                    calendar.add(Calendar.YEAR, extension);
                }
                serverExtension.setExpiredTime(calendar.getTime());
                durationModel.setAmount(extension);
                serverExtension.setDuration(JSONObject.toJSONString(durationModel));
                serverExtension.setDeleted(false);
                serverExtension.setResources("[]");
                serverExtension.setBasicPrice(serverPrice);
                serverExtension.setFinalPrice(serverPrice.multiply(tenant.getDiscount()));
                serverExtension.setOrderId(extensionOrder.getId());
                orderItemRepository.save(serverExtension);
                // 查询磁盘列表
                List<Volume> volumes;
                if (serverItem.getServiceCode().contains("smartx.")) {
                    volumes = imsInternalService.getVolumesForSmartX(server.getId(), server.getVendorId());
                } else {
                    volumes = imsInternalService.listByVmId(server.getId());
                }
                for (Volume volume : volumes) {
                    if (!CloudProviderType.VMWARE.name().equals(server.getVendorType())
                            && (null == volume.getSkuId() || 0L == volume.getSkuId() || !volume.getBootVolume())) {
                        continue;
                    }
                    OrderItem volumeItem = orderItemRepository.queryLastSuccessBySn(volume.getUuid());
                    CloudService cloudService = cloudServiceRepository.queryByCode(volumeItem.getServiceCode());
                    ServiceSku volumeSku = serviceSkuRepository.query(volumeItem.getSkuId());
                    List<ElementModel> volumeElements =
                            JSONArray.parseArray(volumeItem.getElements(), ElementModel.class);
                    ElementModel elementModel = volumeElements.get(0);
                    BigDecimal volumePrice =
                            OrderPriceUtils.getUnitPrice(cloudService, volumeSku, elementModel.getSpecs(), duration);
                    OrderItem volumeOrder = new OrderItem();
                    volumeOrder.setCatalog(OrderItem.Catalog.complex);
                    volumeOrder.setSkuId(volume.getSkuId());
                    volumeOrder.setSn(volume.getUuid());
                    volumeOrder.setElements(volumeItem.getElements());
                    volumeOrder.setServiceCode(volumeItem.getServiceCode());
                    volumeOrder.setStatus(OrderItemStatus.Extensioning);
                    volumeOrder.setDuration(JSONObject.toJSONString(durationModel));
                    volumeOrder.setEffectiveTime(serverExtension.getEffectiveTime());
                    volumeOrder.setExpiredTime(serverExtension.getExpiredTime());
                    volumeOrder.setDeleted(false);
                    volumeOrder.setResources("[]");
                    volumeOrder.setBasicPrice(volumePrice);
                    volumeOrder.setFinalPrice(volumePrice.multiply(tenant.getDiscount()));
                    volumeOrder.setOrderId(extensionOrder.getId());
                    orderItemRepository.save(volumeOrder);
                }
            }
            Long flowId = serviceFlowConfigRepository.getServiceFlowId(service.getId(), ApplyCategory.ExtensionOperate);
            if (null == flowId) {
                if (duration.getMode().equals(Mode.Year)) {
                    extensionParam.setExtension(extensionParam.getExtension() * 12);
                }
                extensionOrder.setApplyParam(JSONObject.toJSONString(extensionParam));
                serviceOrderRepository.update(extensionOrder);
                return new BocloudResult<>(true, extensionOrder.getSn(), "申请已提交,支付后即可进行资源延期操作");
            } else {
                List<OrderItem> extensionItems = orderItemRepository.listByOrderId(extensionOrder.getId());
                for (OrderItem exItem : extensionItems) {
                    exItem.setStatus(OrderItemStatus.Audit);
                    orderItemRepository.update(exItem);
                }
                extensionOrder.setOrderStatus(ChargeCommon.OrderStatus.audit);
                serviceOrderRepository.update(extensionOrder);
                Map<String, Object> content =
                        getParams(ApplyCategory.ExtensionOperate.name(), JSONObject.toJSONString(extensionParam),
                                "申请审核服务实例【" + extensionParam.getInstanceName() + "】延期", context);
                content.put("order", JSONObject.toJSONString(extensionOrder));
                content.put("requestContext", JSONObject.toJSONString(context));
                BocloudResult execution = fmsFlowService.startInstanceById(flowId, context, extensionOrder.getProjectId(), content);
                if (execution.isFailed()) {
                    return new BocloudResult<>(false, "服务实例延期审核流程启动失败");
                }
                return new BocloudResult<>(true, "服务实例延期审核流程启动成功");
            }
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "申请提交失败 : " + e.getMessage());
        }
    }

}
