package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.ServiceOrder;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.enums.*;
import com.bocloud.cos.internal.FmsFlowService;
import com.bocloud.cos.internal.ImsInternalService;
import com.bocloud.cos.internal.SmsTenantService;
import com.bocloud.cos.intf.ResourceUnsubscribeService;
import com.bocloud.cos.intf.ServiceOrderService;
import com.bocloud.cos.model.CancelationParam;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.operate.*;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.utils.IdGenerator;
import com.bocloud.cos.internel.entity.CloudServer;
import com.bocloud.cos.internel.entity.Tenant;
import com.bocloud.cos.internel.entity.Volume;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 服务操作业务实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2020年4月15日
 */
@Service
public class ResourceUnsubscribeServiceImpl implements ResourceUnsubscribeService {

    private static final Logger logger = LoggerFactory.getLogger(ResourceUnsubscribeServiceImpl.class);
    private final ServiceOrderRepository serviceOrderRepository;
    private final AmqpTemplate amqpTemplate;
    private final OrderItemRepository orderItemRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceSkuRepository serviceSkuRepository;
    private final SmsTenantService tenantService;
    private final ImsInternalService imsInternalService;
    private final LockFactory lockFactory;
    private final ServiceFlowConfigRepository serviceFlowConfigRepository;
    private final FmsFlowService fmsFlowService;
    private final ServiceOrderService orderService;

    public ResourceUnsubscribeServiceImpl(ServiceOrderRepository serviceOrderRepository, AmqpTemplate amqpTemplate,
                                          OrderItemRepository orderItemRepository, CloudServiceRepository cloudServiceRepository,
                                          ServiceSkuRepository serviceSkuRepository, SmsTenantService tenantService,
                                          ImsInternalService imsInternalService, LockFactory lockFactory,
                                          ServiceFlowConfigRepository serviceFlowConfigRepository, FmsFlowService fmsFlowService, ServiceOrderService orderService) {
        this.serviceOrderRepository = serviceOrderRepository;
        this.amqpTemplate = amqpTemplate;
        this.orderItemRepository = orderItemRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.serviceSkuRepository = serviceSkuRepository;
        this.tenantService = tenantService;
        this.imsInternalService = imsInternalService;
        this.lockFactory = lockFactory;
        this.serviceFlowConfigRepository = serviceFlowConfigRepository;
        this.fmsFlowService = fmsFlowService;
        this.orderService = orderService;
    }


    private Map<String, Object> getParams(String category, String params, String theme, RequestContext context) {
        Map<String, Object> map = new HashMap<>(16);
        map.put("ApplyCategory", category);
        map.put("ApplyParams", params);
        map.put("ApplyTheme", theme);
        map.put("ApplyContext", context);
        return map;
    }

    @Override
    @Transactional
    public Result process(CancelationParam cancelationParam, RequestContext context) {
        // 服务资源实例id
        String instance = cancelationParam.getInstance();
        /** 根据sn查询是否有处于中间状态的订单项（如变更中、延期中、退订中等），如有，则直接RETURN **/
        String result = orderService.underwayCheck(cancelationParam.getInstance());
        if (null != result) {
            return new BocloudResult<>(false, result);
        }

        // 检测资源操作
        OrderItem item = orderItemRepository.queryBySn(instance);
        Assert.notNull(item, "订单明细未找到");
        ServiceOrder order = serviceOrderRepository.query(item.getOrderId());
        Assert.notNull(order, "订单未找到");
        String code = item.getServiceCode();
        if (null != item.getSkuId() && 0L != item.getSkuId()) {
            ServiceSku sku = serviceSkuRepository.query(item.getSkuId());
            Assert.notNull(sku, "订单明细规格未找到");
            CloudService service = cloudServiceRepository.query(sku.getServiceId());
            Assert.notNull(service, "订单明细关联的服务未找到");
            code = service.getCode();
        }
        DurationModel duration = JSONObject.parseObject(item.getDuration(), DurationModel.class);
        CloudService service = cloudServiceRepository.queryByCode(code);
        cancelationParam.setService(service.getCode());
        ResourceOperate resourceOperate = ResourceOperateFactory.getInstance(code);
        Assert.notNull(resourceOperate, "该服务不支持此操作");
        cancelationParam.setCategory(ServiceResourceMapping.getResourceCategory(code).name());
        cancelationParam.setService(code);
        cancelationParam.setContext(context);
        cancelationParam.setOldStatus(item.getStatus());
        ServiceOrder cancelationOrder = new ServiceOrder();
        cancelationOrder.setServiceCode(cancelationParam.getService());
        // 生成订单号，增加分布式锁
        String path = IdGenerator.class.getSimpleName() + "_ordersn";
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(1, TimeUnit.MINUTES), "生成订单号请求超时");
            cancelationOrder.setSn(IdGenerator.getInstance().nextId());
        } catch (Exception e) {
            throw new RuntimeException("资源退订失败 : " + e.getMessage(), e);
        }
        cancelationOrder.setChargeMode(order.getChargeMode());
        cancelationOrder.setSlaAgreed(order.getSlaAgreed());
        cancelationOrder.setOrderStatus(ChargeCommon.OrderStatus.paid);
        cancelationOrder.setType(ChargeCommon.Type.unsubscribe);
        cancelationOrder.setGmtCreate(new Date());
        cancelationOrder.setAutoRenew(order.getAutoRenew());
        cancelationOrder.setVendorId(order.getVendorId());
        cancelationOrder.setBusinessId(order.getBusinessId());
        cancelationOrder.setProjectId(order.getProjectId());
        cancelationOrder.setUserId(order.getUserId());
        cancelationOrder.setTenantId(order.getTenantId());
        cancelationOrder.setResourceStatus(ChargeCommon.ResourceStatus.canceld);
        cancelationOrder.setApplyParam(JSONObject.toJSONString(cancelationParam));
        serviceOrderRepository.save(cancelationOrder);
        Tenant tenant = tenantService.detail(context.getTenant());
        if (resourceOperate instanceof PaasResourceOperate) {
            //找到所有资源订单项 做退订
            List<OrderItem> orderItems = orderItemRepository.listByOrderId(item.getOrderId());
            for (OrderItem orderItem : orderItems) {
                ServiceSku serviceSku = serviceSkuRepository.query(orderItem.getSkuId());
                CloudService cloudService = cloudServiceRepository.queryByCode(orderItem.getServiceCode());
                List<ElementModel> elementModels = JSONArray.parseArray(orderItem.getElements(), ElementModel.class);
                ElementModel element = elementModels.get(0);
                BigDecimal unitPrice = OrderPriceUtils.getUnitPrice(cloudService, serviceSku, element.getSpecs(), duration);
                Integer count = Optional.ofNullable(element.getCount()).orElse(1);
                unitPrice = unitPrice.multiply(BigDecimal.valueOf(count));
                OrderItem cancelationItem = new OrderItem();
                cancelationItem.setCatalog(OrderItem.Catalog.complex);
                cancelationItem.setSkuId(orderItem.getSkuId());
                cancelationItem.setSn(orderItem.getSn());
                cancelationItem.setServiceCode(orderItem.getServiceCode());
                cancelationItem.setStatus(OrderItemStatus.Unsubscribing);
                cancelationItem.setDeleted(false);
                cancelationItem.setResources("[]");
                cancelationItem.setBasicPrice(unitPrice);
                cancelationItem.setElements(orderItem.getElements());
                cancelationItem.setDuration(orderItem.getDuration());
                cancelationItem.setEffectiveTime(new Date());
                cancelationItem.setExpiredTime(orderItem.getExpiredTime());
                BigDecimal elementRatio = element.getRatio() == null ? BigDecimal.ONE : element.getRatio();
                cancelationItem.setFinalPrice(unitPrice.multiply(tenant.getDiscount()).multiply(elementRatio));
                cancelationItem.setOrderId(cancelationOrder.getId());
                orderItemRepository.save(cancelationItem);
            }
        } else if (!ServiceResourceMapping.getResourceCategory(code).equals(ResourceCategory.Computer)) {
            OrderItem targetItem = orderItemRepository.queryLastSuccessBySn(instance);
            ServiceSku targetSku = serviceSkuRepository.query(targetItem.getSkuId());
            List<ElementModel> elementModels = JSONArray.parseArray(targetItem.getElements(), ElementModel.class);
            ElementModel element = elementModels.get(0);
            BigDecimal targetPrice = OrderPriceUtils.getUnitPrice(service, targetSku, element.getSpecs(), duration);
            OrderItem cancelationItem = new OrderItem();
            cancelationItem.setCatalog(OrderItem.Catalog.complex);
            cancelationItem.setSkuId(targetItem.getSkuId());
            cancelationItem.setSn(targetItem.getSn());
            cancelationItem.setServiceCode(targetItem.getServiceCode());
            cancelationItem.setStatus(OrderItemStatus.Unsubscribing);
            cancelationItem.setElements(targetItem.getElements());
            cancelationItem.setEffectiveTime(new Date());
            cancelationItem.setExpiredTime(targetItem.getExpiredTime());
            cancelationItem.setDuration(targetItem.getDuration());
            cancelationItem.setDeleted(false);
            cancelationItem.setResources("[]");
            cancelationItem.setBasicPrice(targetPrice);
            BigDecimal elementRatio = element.getRatio() == null ? BigDecimal.ONE : element.getRatio();
            cancelationItem.setFinalPrice(targetPrice.multiply(tenant.getDiscount()).multiply(elementRatio));
            cancelationItem.setOrderId(cancelationOrder.getId());
            orderItemRepository.save(cancelationItem);
        } else {
            //找到对应机器和磁盘 做退订计费
            CloudServer server = imsInternalService.queryBySn(instance);
            OrderItem serverItem = orderItemRepository.queryLastSuccessBySn(instance);
            ServiceSku serverSku = serviceSkuRepository.query(serverItem.getSkuId());
            List<ElementModel> elementModels = JSONArray.parseArray(serverItem.getElements(), ElementModel.class);
            ElementModel element = elementModels.get(0);
            BigDecimal serverPrice = OrderPriceUtils.getUnitPrice(service, serverSku, element.getSpecs(), duration);
            OrderItem cancelationItem = new OrderItem();
            cancelationItem.setCatalog(OrderItem.Catalog.complex);
            cancelationItem.setSkuId(server.getSkuId());
            cancelationItem.setSn(server.getUuid());
            cancelationItem.setServiceCode(serverItem.getServiceCode());
            cancelationItem.setStatus(OrderItemStatus.Unsubscribing);
            cancelationItem.setElements(serverItem.getElements());
            cancelationItem.setDuration(serverItem.getDuration());
            cancelationItem.setEffectiveTime(new Date());
            cancelationItem.setExpiredTime(serverItem.getExpiredTime());
            cancelationItem.setDeleted(false);
            cancelationItem.setResources("[]");
            cancelationItem.setBasicPrice(serverPrice);
            cancelationItem.setFinalPrice(serverPrice);
            if (null != tenant && null != tenant.getDiscount()) {
                cancelationItem.setFinalPrice(serverPrice.multiply(tenant.getDiscount()));
            }
            cancelationItem.setOrderId(cancelationOrder.getId());
            orderItemRepository.save(cancelationItem);
            // 查询磁盘列表
            List<Volume> volumes;
            if (serverItem.getServiceCode().contains("smartx.")) {
                volumes = imsInternalService.getVolumesForSmartX(server.getId(), server.getVendorId());
                //虚拟机挂在的虚拟卷会被同时退订，共享卷除外
                if (CollectionUtils.isNotEmpty(volumes)) {
                    volumes = volumes.stream().filter(volume -> volume.getMultiattach() == null || !volume.getMultiattach()).collect(Collectors.toList());
                }
            } else {
                volumes = imsInternalService.listByVmId(server.getId());
                if (!CloudProviderType.VMWARE.name().equals(server.getVendorType())) {
                    volumes = volumes.stream().filter(volume -> null != volume.getSkuId() && 0L != volume.getSkuId() && volume.getBootVolume())
                            .collect(Collectors.toList());
                }
            }
            for (Volume volume : volumes) {
                OrderItem volumeItem = orderItemRepository.queryLastSuccessBySn(volume.getUuid());
                CloudService cloudService = cloudServiceRepository.queryByCode(volumeItem.getServiceCode());
                ServiceSku volumeSku = serviceSkuRepository.query(volumeItem.getSkuId());
                List<ElementModel> volumeElements = JSONArray.parseArray(volumeItem.getElements(), ElementModel.class);
                ElementModel elementModel = volumeElements.get(0);
                BigDecimal volumePrice = OrderPriceUtils.getUnitPrice(cloudService, volumeSku, elementModel.getSpecs(), duration);
                OrderItem volumeOrder = new OrderItem();
                volumeOrder.setCatalog(OrderItem.Catalog.complex);
                volumeOrder.setSkuId(volume.getSkuId());
                volumeOrder.setSn(volume.getUuid());
                volumeOrder.setElements(volumeItem.getElements());
                volumeOrder.setDuration(volumeItem.getDuration());
                volumeOrder.setServiceCode(volumeItem.getServiceCode());
                volumeOrder.setStatus(OrderItemStatus.Unsubscribing);
                volumeOrder.setEffectiveTime(cancelationItem.getEffectiveTime());
                volumeOrder.setExpiredTime(cancelationItem.getExpiredTime());
                volumeOrder.setDeleted(false);
                volumeOrder.setResources("[]");
                volumeOrder.setBasicPrice(volumePrice);
                volumeOrder.setFinalPrice(volumePrice.multiply(tenant.getDiscount()));
                volumeOrder.setOrderId(cancelationOrder.getId());
                orderItemRepository.save(volumeOrder);
            }
        }
        Long flowId = serviceFlowConfigRepository.getServiceFlowId(service.getId(), ApplyCategory.UnsubscribeOperate);
        if (null == flowId) {
            cancelationOrder.setApplyParam(JSONObject.toJSONString(cancelationParam));
            serviceOrderRepository.update(cancelationOrder);
            logger.info("退订消息：{}", JSONObject.toJSONString(cancelationParam));
            if (resourceOperate instanceof PaasResourceOperate) {
                amqpTemplate.convertAndSend("paas.resource.deletion.queue", cancelationParam);
            } else {
                if ("vmware.nsx.vpc".equals(code)) {
                    amqpTemplate.convertAndSend("network.resource.deletion.queue", cancelationParam);
                } else {
                    amqpTemplate.convertAndSend("iaas.resource.deletion.queue", cancelationParam);
                }
            }
            return new BocloudResult<>(true, cancelationOrder.getSn(), "申请已提交");
        } else {
            List<OrderItem> cancelationItems = orderItemRepository.listByOrderId(cancelationOrder.getId());
            for (OrderItem orderItem : cancelationItems) {
                orderItem.setStatus(OrderItemStatus.Audit);
                orderItemRepository.update(orderItem);
            }
            cancelationOrder.setOrderStatus(ChargeCommon.OrderStatus.audit);
            serviceOrderRepository.update(cancelationOrder);
            logger.info("【服务退订】申请审核服务实例参数: {}", JSONObject.toJSONString(cancelationParam));
            Map<String, Object> content = getParams(ApplyCategory.UnsubscribeOperate.name(), JSONObject.toJSONString(cancelationParam),
                    "申请审核服务实例【" + cancelationParam.getInstanceName() + "】退订", context);
            content.put("order", JSONObject.toJSONString(cancelationOrder));
            content.put("requestContext", JSONObject.toJSONString(context));
            ServiceOrder serviceOrder = serviceOrderRepository.query(item.getOrderId());
            BocloudResult execution = fmsFlowService.startInstanceById(flowId, context, serviceOrder.getProjectId(), content);
            if (execution.isFailed()) {
                return new Result(false, "服务实例退订审核流程启动失败");
            }
            return new Result(true, "服务实例退订审核流程启动成功");
        }
    }

    @Override
    public Result empty(RequestContext context) {
        try {
            List<CloudServer> servers = imsInternalService.listServers(context);
            logger.info("云主机列表：{}", JSONObject.toJSON(servers));
            Assert.notEmpty(servers, "回收站无云主机需要清空");
            for (CloudServer server : servers) {
                OrderItem item = orderItemRepository.queryBySn(server.getUuid());
                if (OrderItemStatus.Unsubscribing == item.getStatus()) {
                    continue;
                }
                CancelationParam params = new CancelationParam();
                params.setInstance(server.getUuid());
                this.process(params, context);
            }
            return new Result(true, "清空回收站任务已下发，请等待执行结果...");
        } catch (Exception e) {
            logger.error("Get error message:{}", e.getMessage());
            return new Result(false, e.getMessage());
        }
    }

}
