package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.*;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.entity.Ros;
import com.bocloud.cos.entity.ServiceOrder;
import com.bocloud.cos.enums.OrderItemStatus;
import com.bocloud.cos.enums.ResourceCategory;
import com.bocloud.cos.intf.RosService;
import com.bocloud.cos.model.ApplicationParam;
import com.bocloud.cos.model.DurationModel;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.OrderMessage;
import com.bocloud.cos.model.OrderMessage.Operation;
import com.bocloud.cos.params.RosParams;
import com.bocloud.cos.repository.RosRepository;
import com.bocloud.cos.internal.CopTaskService;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 资源编排服务实现
 *
 * @author weiwei
 */
@Service
public class RosServiceImpl implements RosService {

    private static Logger logger = LoggerFactory.getLogger(RosServiceImpl.class);
    @Autowired
    private RosRepository rosRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private CopTaskService copTaskService;

    @Override
    public BocloudResult list(int page, int rows, List<Param> params, Map<String, String> sorter, boolean simple,
                              RequestContext context) {
        List<Ros> list = null;
        int total = 0;
        GridBean gridBean = null;
        try {
            params = null == params ? new ArrayList<>() : params;
            sorter = null == sorter ? new HashMap<>() : sorter;
            sorter.put("gmtCreate", Common.ONE);
            if (Catalog.User == context.getCatalog()) {
                params.add(new Param(MapTools.simpleMap("gmtCatalog", context.getCatalog().name()), Sign.EQ));
                params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
                params.add(new Param(MapTools.simpleMap("creatorId", context.getTarget()), Sign.EQ));
            } else if (Catalog.Tenant == context.getCatalog()) {
                params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
            }
            total = rosRepository.count(params);
            if (simple) {
                List<SimpleBean> beans = rosRepository.list(params, sorter);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                list = rosRepository.list(page, rows, sorter, params);
                gridBean = GridHelper.getBean(page, rows, total, list);
            }
            return new BocloudResult(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult detail(Long id) {
        try {
            Ros ros = rosRepository.query(id);
            if (null == ros) {
                return new BocloudResult(false, "查询失败");
            }
            return new BocloudResult(true, ros, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public BocloudResult remove(Long id, RequestContext context) {
        String path = Ros.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Ros ros = rosRepository.query(id);
            Assert.notNull(ros, "数据不存在！");
            rosRepository.remove(id, context.getTarget());
            return new BocloudResult(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "删除失败");
        }
    }

    @Override
    public BocloudResult create(ApplicationParam param, RequestContext context) {
        ServiceOrder order = param.getOrder();
        List<OrderItem> items = order.getItems();
        if (ListTool.isEmpty(items)) {
            return new BocloudResult(false, "订单数据错误！");
        }
        DurationModel duration = param.getEmption().getDuration();
        DurationModel.Mode mode = duration.getMode();
        int amount = Optional.ofNullable(duration.getAmount()).orElse(0);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        if (amount > 0) {
            if (mode.equals(DurationModel.Mode.Month)) {
                calendar.add(Calendar.MONTH, amount);
            } else {
                calendar.add(Calendar.YEAR, amount);
            }
        } else {
            calendar.setTime(DateTools.stringToDateTime("2099-12-31 00:00:00"));
        }
        Ros ros = new Ros();
        ros.setStatus("available");
        ros.setUuid(items.get(0).getSn());
        ros.setCreatorId(context.getTarget());
        ros.setTenantId(context.getTenant());
        ros.setGmtCatalog(context.getCatalog().name());
        ros.setExpiredTime(calendar.getTime());
        ros.setInventory(JSONObject.toJSONString(param));
        if (null != param.getElements() && !param.getElements().isEmpty()) {
            ElementModel element = param.getElements().get(0);
            ros.setSpec(JSONObject.toJSONString(element.getSpecs()));
        }
        ros.setUsed(0);
        JSONObject configs = param.getConfigs();
        ros.setCategoryId(configs.getLong("categoryId"));
        ros.setSkuId(configs.getLong("skuId"));
        ros.setServiceCode(param.getService());
        ros.setName(configs.getString("serviceName") + "_" + System.currentTimeMillis());
        try {
            // FIXME: 2021/5/6 这里的次数从哪里获取
            ros.setTimes(1);
            rosRepository.save(ros);
            OrderMessage message = new OrderMessage(ros.getUuid(), Operation.Application, OrderItemStatus.OpenedSuccess,
                    null, "资源编排服务【" + configs.getString("serviceName") + "】订购成功", ros.getGmtCreate(),
                    ros.getExpiredTime(), null, ResourceCategory.Operations.name(), context);
            amqpTemplate.convertAndSend("resource.result.queue", message);
            return new BocloudResult(true, "创建成功");
        } catch (Exception e) {
            OrderMessage message = new OrderMessage(ros.getUuid(), Operation.Application, OrderItemStatus.OpenedFailure,
                    null, "资源编排服务【" + configs.getString("serviceName") + "】订购失败", ros.getGmtCreate(),
                    ros.getExpiredTime(), null, ResourceCategory.Operations.name(), context);
            amqpTemplate.convertAndSend("resource.result.queue", message);
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "创建失败");
        }
    }

    @Override
    public BocloudResult execute(RosParams params, RequestContext context) {
        Long rosId = params.getRosId();
        String path = Ros.class.getSimpleName() + "_" + rosId;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Ros ros = rosRepository.query(rosId);
            Assert.notNull(ros, "数据不存在！");
            if ("unavailable".equalsIgnoreCase(ros.getStatus())) {
                return new BocloudResult(false, "该服务实例处于不可用状态");
            }
            JSONObject task = params.getTask();
            task.put("name", ros.getName().substring(0, ros.getName().indexOf("_")));
            task.put("rosId", rosId);
            BocloudResult result = copTaskService.execute(task, context.getApiKey());
            if (result.isFailed()) {
                return result;
            }
//			ros.setUsed(ros.getUsed() + 1);
//			if (ros.getUsed() >= ros.getTimes()) {
//				ros.setStatus("unavailable");
//			}
//			ros.setMenderId(context.getTarget());
//			ros.setGmtModify(new Date());
//			rosRepository.update(ros);
            return new BocloudResult(true, "资源编排任务下发成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e);
            return new BocloudResult(false, "资源编排任务下发失败");
        }
    }

}
