package com.bocloud.cos.service;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ServiceCatalog;
import com.bocloud.cos.entity.ServiceSla;
import com.bocloud.cos.entity.ServiceTenant;
import com.bocloud.cos.intf.ServiceCatalogService;
import com.bocloud.cos.repository.*;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 服务目录业务操作实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月10日
 */
@Service
public class ServiceCatalogServiceImpl implements ServiceCatalogService {

    private static final Logger logger = LoggerFactory.getLogger(ServiceCatalogServiceImpl.class);
    private final ServiceCatalogRepository serviceCatalogRepository;
    private final ServiceSlaRepository serviceSlaRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final OrderSnRepository snRepository;
    private final ServiceTenantRepository serviceTenantRepository;

    public ServiceCatalogServiceImpl(ServiceCatalogRepository serviceCatalogRepository, ServiceSlaRepository serviceSlaRepository,
                                     CloudServiceRepository cloudServiceRepository, OrderSnRepository snRepository,
                                     ServiceTenantRepository serviceTenantRepository) {
        this.serviceCatalogRepository = serviceCatalogRepository;
        this.serviceSlaRepository = serviceSlaRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.snRepository = snRepository;
        this.serviceTenantRepository  = serviceTenantRepository;
    }

    @Override
    public BocloudResult<GridBean<ServiceCatalog>> list(Pager pager) {
        Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
        List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
        int total = this.serviceCatalogRepository.count(params);
        List<ServiceCatalog> list = this.serviceCatalogRepository.list(pager.getPage(), pager.getRows(), sorter, params);
        GridBean<ServiceCatalog> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
        return new BocloudResult<>(true, gridBean, "查询成功");
    }

    @Override
    @Transactional
    public Result remove(Long id, Long userId) {
        Boolean result = serviceCatalogRepository.remove(id, userId);
        if (result) {
            snRepository.removeByCatalogId(id);
        }
        return new Result(true, "删除成功");
    }

    @Override
    public BocloudResult<ServiceCatalog> detail(Long id) {
        ServiceCatalog serviceCatalog = serviceCatalogRepository.query(id);
        return new BocloudResult<>(true, serviceCatalog, "查询成功");
    }

    @Override
    public Result create(ServiceCatalog serviceCatalog) {
        Integer maxId = serviceCatalogRepository.getMaxId();
        if (null == maxId) {
            maxId = 0;
        }
        serviceCatalog.setSn(String.format("%02d", maxId));
        serviceCatalogRepository.save(serviceCatalog);
        return new Result(true, "新增成功");
    }

    @Override
    public BocloudResult list(Long tenantId) {
        if (null == tenantId) {
            return new BocloudResult<>(false, "当前租户状态异常，请重新操作！");
        }
        BocloudResult<GridBean<ServiceCatalog>> bocloudResult = this.list(new Pager());
        if (bocloudResult.isFailed()) {
            return bocloudResult;
        }
        GridBean<ServiceCatalog> gridBean = bocloudResult.getData();
        List<ServiceCatalog> serviceCatalogList = gridBean.getRows();
        if (null == serviceCatalogList || serviceCatalogList.size() == 0) {
            return new BocloudResult<>(false, "没有可用服务！");
        }
        for (int i = serviceCatalogList.size() - 1; i >= 0; i--) {
            ServiceCatalog serviceCatalog = serviceCatalogList.get(i);
            List<CloudService> cloudServices = cloudServiceRepository.listByCategory(serviceCatalog.getId(),
                    tenantId);
            if (null != cloudServices && cloudServices.size() > 0) {
                serviceCatalog.setCloudServices(cloudServices);
            } else {
                serviceCatalogList.remove(serviceCatalog);
            }
        }
        return new BocloudResult<>(true, serviceCatalogList, "查询成功");
    }

    @Override
    public List<CloudService> listPublishedServicesByCatalog(String code, Long tenantId) {
        ServiceCatalog serviceCatalog = serviceCatalogRepository.getByCode(code);
        List<Long> serviceIds = serviceTenantRepository.listByTenantId(tenantId).stream().map(ServiceTenant::getServiceId)
                .collect(Collectors.toList());
        List<CloudService> cloudServices = cloudServiceRepository.listPublishedServicesByCatalog(serviceCatalog.getId()).stream()
                .filter(cloudService -> serviceIds.contains(cloudService.getId())).collect(Collectors.toList());
        for (CloudService service : cloudServices) {
            List<ServiceSla> slas = serviceSlaRepository.listByService(service.getId(), ServiceSla.Status.enable.name());
            if (!ListTool.isEmpty(slas)) {
                service.setSlaId(slas.get(0).getId());
            }
        }
        return cloudServices;
    }
}
