package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ServiceCatalog;
import com.bocloud.cos.entity.ServiceCategory;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.intf.ServiceCategoryService;
import com.bocloud.cos.repository.*;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 服务类型业务操作实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月10日
 */
@Service
public class ServiceCategoryServiceImpl implements ServiceCategoryService {

    private static final Logger logger = LoggerFactory.getLogger(ServiceCategoryServiceImpl.class);
    private final ServiceCategoryRepository serviceCategoryRepository;
    private final ServiceCatalogRepository catalogRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final LockFactory lockFactory;
    private final OrderSnRepository snRepository;
    private final ServiceSkuRepository serviceSkuRepository;

    public ServiceCategoryServiceImpl(ServiceCategoryRepository serviceCategoryRepository, ServiceCatalogRepository catalogRepository, CloudServiceRepository cloudServiceRepository, LockFactory lockFactory, OrderSnRepository snRepository, ServiceSkuRepository serviceSkuRepository) {
        this.serviceCategoryRepository = serviceCategoryRepository;
        this.catalogRepository = catalogRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.lockFactory = lockFactory;
        this.snRepository = snRepository;
        this.serviceSkuRepository = serviceSkuRepository;
    }

    @Override
    public BocloudResult<GridBean<ServiceCategory>> list(Pager pager) {
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            int total = this.serviceCategoryRepository.count(params);
            List<ServiceCategory> list = this.serviceCategoryRepository.list(pager.getPage(), pager.getRows(), sorter, params);
            GridBean<ServiceCategory> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query error:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result remove(Long id, Long userId) {
        try {

            ServiceSku serviceSku = serviceSkuRepository.getByCategoryId(id);
            if (serviceSku != null && serviceSku.getSpec() != null){
                return new Result(false,"该服务类型下存在规格无法删除");
            }

            Boolean result = serviceCategoryRepository.remove(id, userId);
            if (result) {
                snRepository.removeByCategoryId(id);
            }
            return new Result(true, "删除成功");
        } catch (Exception e) {
            logger.error("Query error:", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    public BocloudResult<ServiceCategory> detail(Long id) {
        try {
            ServiceCategory serviceCategory = serviceCategoryRepository.query(id);
            return new BocloudResult<>(true, serviceCategory, "查询成功");
        } catch (Exception e) {
            logger.error("Query error:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result create(ServiceCategory category, RequestContext context) {
        try {
            category.setCreatorId(context.getTarget());
            category.setMenderId(context.getTarget());
            boolean flag = serviceCategoryRepository.existsName(category.getName(), category.getServiceId());
            if (flag) {
                return new Result(false, "类型名称已存在");
            }
            flag = serviceCategoryRepository.existsSn(category.getSn(), category.getServiceId());
            if (flag) {
                return new Result(false, "当前服务下的类型编号已存在");
            }
            category.setReserved(false);
            Integer maxId = serviceCategoryRepository.getMaxId();
            if (null == maxId) {
                maxId = 0;
            }
            category.setSn(String.format("%02d", maxId));
            serviceCategoryRepository.save(category);
            return new Result(true, "新增成功");
        } catch (Exception e) {
            logger.error("Query error:", e);
            return new Result(false, "新增失败");
        }
    }

    @Override
    public Result modify(Long id, ServiceCategory category, RequestContext context) {
        String path = ServiceCategory.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceCategory object = serviceCategoryRepository.query(id);
            Assert.notNull(object, "数据不存在！");
            if (!object.getName().equals(category.getName())
                    && serviceCategoryRepository.existsName(category.getName(), category.getServiceId())) {
                return new Result(false, "名称已存在");
            }
            if (!object.getSn().equals(category.getSn())
                    && serviceCategoryRepository.existsSn(category.getSn(), category.getServiceId())) {
                return new Result(false, "当前服务目录下的云服务sn已存在");
            }
            object.setName(category.getName());
            object.setCode(category.getCode());
            object.setSn(category.getSn());
            object.setRemark(category.getRemark());
            object.setGmtModify(new Date());
            object.setMenderId(context.getTarget());
            serviceCategoryRepository.update(object);
            return new Result(true, "修改成功");
        } catch (Exception e) {
            logger.error("Modify ServiceCategory fail:", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    public BocloudResult<List<ServiceCategory>> listByCode(String code) {
        try {
            List<ServiceCategory> categories = serviceCategoryRepository.listByServiceCode(code);
            return new BocloudResult<>(true, categories, "查询成功");
        } catch (Exception e) {
            logger.error("Query error:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult<List<ServiceCategory>> list(List<Param> params) {
        try {
            if (null == params) {
                params = Lists.newArrayList();
            }
            List<ServiceCategory> categories = this.serviceCategoryRepository.list(params);
            return new BocloudResult<>(true, categories, "查询成功");
        } catch (Exception e) {
            logger.error("Get service skus error：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult<JSONArray> tree() {
        try {
            List<ServiceCategory> serviceCategories = serviceCategoryRepository.listAll();
            Map<String, List<ServiceCategory>> map = serviceCategories.stream().filter(serviceCategory -> serviceCategory.getServiceName() != null).collect(Collectors.groupingBy(ServiceCategory::getServiceName));
            //排序
            List<ServiceCatalog> catalogSortList = catalogRepository.listAll().stream().sorted(Comparator.comparing(ServiceCatalog::getSn)).collect(Collectors.toList());
            Map<Long, List<CloudService>> serviceMap = cloudServiceRepository.listByStatus(CloudService.Status.PUBLISHED.name()).stream().collect(Collectors.groupingBy(CloudService::getCatalogId));
            JSONArray result = new JSONArray();
            ListIterator<ServiceCatalog> catalogListIterator = catalogSortList.listIterator();
            while (catalogListIterator.hasNext()) {
                ServiceCatalog catalog = catalogListIterator.next();
                JSONObject catalogObject = new JSONObject();
                catalogObject.put("id", catalog.getId());
                catalogObject.put("name", catalog.getName());
                JSONArray serviceArray = new JSONArray();
                List<CloudService> cloudServices = Optional.ofNullable(serviceMap.get(catalog.getId())).orElse(new ArrayList<>());
                if (!cloudServices.isEmpty()) {
                    cloudServices.sort(Comparator.comparing(CloudService::getSn));
                    ListIterator<CloudService> cloudServiceListIterator = cloudServices.listIterator();
                    while (cloudServiceListIterator.hasNext()) {
                        CloudService cloudService = cloudServiceListIterator.next();
                        JSONObject serviceObject = new JSONObject();
                        serviceObject.put("id", cloudService.getId());
                        serviceObject.put("name", cloudService.getName());
                        List<ServiceCategory> categories = Optional.ofNullable(map.get(cloudService.getName())).orElse(new ArrayList<>());
                        if (!categories.isEmpty()) {
                            categories.sort(Comparator.comparing(ServiceCategory::getSn));
                            serviceObject.put("children", categories);
                        } else {
                            cloudServiceListIterator.remove();
                            continue;
                        }
                        serviceArray.add(serviceObject);
                    }
                } else {
                    catalogListIterator.remove();
                    continue;
                }
                catalogObject.put("children", serviceArray);
                result.add(catalogObject);
            }
            return new BocloudResult<>(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("list service category tree error:", e);
            return new BocloudResult<>(false, "查询失败：" + e.getMessage());
        }
    }
}
