package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.intf.ServiceSkuService;
import com.bocloud.cos.params.SkuParams;
import com.bocloud.cos.repository.*;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * 服务产品业务操作实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月10日
 */
@Service
public class ServiceSkuServiceImpl implements ServiceSkuService {

    private static final Logger logger = LoggerFactory.getLogger(ServiceSkuServiceImpl.class);
    private final ServiceSkuRepository serviceSkuRepository;
    private final SkuSpecRepository skuSpecRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final LockFactory lockFactory;
    private final OrderSnRepository snRepository;
    private final ServiceCategoryRepository categoryRepository;
    private final ServiceCatalogRepository catalogRepository;

    @Autowired
    public ServiceSkuServiceImpl(ServiceSkuRepository serviceSkuRepository, SkuSpecRepository skuSpecRepository,
                                 CloudServiceRepository cloudServiceRepository, LockFactory lockFactory, OrderSnRepository snRepository,
                                 ServiceCategoryRepository categoryRepository, ServiceCatalogRepository catalogRepository) {
        this.serviceSkuRepository = serviceSkuRepository;
        this.skuSpecRepository = skuSpecRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.lockFactory = lockFactory;
        this.snRepository = snRepository;
        this.categoryRepository = categoryRepository;
        this.catalogRepository = catalogRepository;
    }

    @Override
    public BocloudResult<GridBean<ServiceSku>> list(Pager pager) {
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            sorter.put("gmtCreate", Common.ONE);
            int total = this.serviceSkuRepository.count(params);
            List<ServiceSku> list = this.serviceSkuRepository.list(pager.getPage(), pager.getRows(), params, sorter);
            GridBean<ServiceSku> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query service sku list error:{}", e.getMessage());
            return new BocloudResult<>(false, e.getMessage());
        }
    }

    @Override
    public BocloudResult<ServiceSku> detail(Long id) {
        try {
            ServiceSku sku = this.serviceSkuRepository.query(id);
            assert Objects.nonNull(sku) : "数据不存在";
            return new BocloudResult<>(true, sku, "查询成功");
        } catch (Exception e) {
            logger.error("Get service sku error：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result create(SkuParams skuParams, RequestContext context) {
        try {
            ServiceSku sku = new ServiceSku();
            BeanUtils.copyProperties(skuParams, sku);
            Assert.notNull(sku.getSpec(), "服务产品规格为空");
            Assert.notNull(sku.getServiceId(), "服务ID为空");
            Assert.notNull(sku.getCategoryId(), "服务类型为空");
            CloudService service = cloudServiceRepository.query(sku.getServiceId());
            sku.setServiceCode(service.getCode());
            // sku.setSn(IDFactory.instance().timestamp());
            Integer maxId = serviceSkuRepository.getMaxId();
            maxId = Optional.ofNullable(maxId).orElse(0);
            sku.setSn(String.format("%02d", maxId));
            sku.setStatus(ServiceSku.Status.enable.name());
            boolean exists = serviceSkuRepository.exists("tariff_code", sku.getTariffCode());
            Assert.isTrue(!exists, "服务产品资费代码已存在");
            exists = serviceSkuRepository.exists("code", sku.getCode());
            Assert.isTrue(!exists, "服务产品代码已存在");
            exists = serviceSkuRepository.exists("spec", sku.getSpec(), sku.getCategoryId());
            Assert.isTrue(!exists, "当前产品类型下的产品规格已存在");
            sku.setCreatorId(context.getTarget());
            sku.setGmtCreate(new Date());
            serviceSkuRepository.save(sku);
            List<SkuSpec> specs = JSONArray.parseArray(sku.getSpec(), SkuSpec.class);
            if (!ListTool.isEmpty(specs)) {
                // 保存规格字典
                List<SkuSpec> buildSpecs = buildSpecs(specs, sku.getId(), sku.getCode());
                boolean success = skuSpecRepository.batchSave(buildSpecs, buildSpecs.size());
                if (!success) {
                    serviceSkuRepository.remove(sku.getId(), context.getTarget());
                    return new Result(false, "创建失败");
                }
            }
            // 生成订单号记录
            this.generateOrderSn(sku);
            return new Result(true, "创建成功");
        } catch (Exception e) {
            logger.error("Create service sku error：", e);
            return new Result(false, e.getMessage());
        }
    }

    /**
     * 生成订单号记录
     *
     * @param sku
     * @throws Exception
     */
    private void generateOrderSn(ServiceSku sku) throws Exception {
        ServiceCategory category = categoryRepository.query(sku.getCategoryId());
        if (null == category) {
            return;
        }
        ServiceCatalog catalog = catalogRepository.query(sku.getCatalogId());
        if (null == catalog) {
            return;
        }
        CloudService service = cloudServiceRepository.query(category.getServiceId());
        if (null == service) {
            return;
        }
        if (StringUtils.isNotEmpty(catalog.getSn()) && StringUtils.isNotEmpty(service.getSn()) && StringUtils
                .isNotEmpty(category.getSn()) && StringUtils.isNotEmpty(sku.getSn())) {
            OrderSn orderSn =
                    new OrderSn(catalog.getId(), service.getId(), category.getId(), sku.getId(), catalog.getSn(),
                            service.getSn(), category.getSn(), sku.getSn(), 0L);
            snRepository.save(orderSn);
        }
    }

    /**
     * 生成规格字典
     *
     * @param specs
     * @param skuId
     * @param code
     * @return
     */
    private List<SkuSpec> buildSpecs(List<SkuSpec> specs, Long skuId, String code) {
        return Optional.ofNullable(specs).orElse(new ArrayList<>()).stream().map(spec -> {
            spec.setSkuId(skuId);
            spec.setSpecCategory(code);
            return spec;
        }).collect(Collectors.toList());
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result modify(Long id, SkuParams sku, RequestContext context) {
        String path = ServiceSku.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceSku oldSku = serviceSkuRepository.query(id);
            Assert.notNull(oldSku, "数据不存在！");
            String tariffCode = Optional.ofNullable(sku.getTariffCode()).orElse("");
            if (!tariffCode.equals(oldSku.getTariffCode()) && serviceSkuRepository
                    .exists("tariff_code", sku.getTariffCode())) {
                return new Result(false, "服务产品资费代码已存在");
            }
            if (!oldSku.getCode().equals(sku.getCode()) && serviceSkuRepository.exists("code", sku.getCode())) {
                return new Result(false, "服务产品代码已存在");
            }
            if (!oldSku.getSpec().equals(sku.getSpec()) && serviceSkuRepository
                    .exists("spec", sku.getSpec(), sku.getCategoryId())) {
                return new Result(false, "当前产品类型下的产品规格已存在");
            }
            oldSku.setName(sku.getName());
            oldSku.setSpec(sku.getSpec());
            oldSku.setHourPrice(sku.getHourPrice());
            oldSku.setMonthPrice(sku.getMonthPrice());
            oldSku.setYearPrice(sku.getYearPrice());
            oldSku.setRemark(sku.getRemark());
            oldSku.setCategoryId(sku.getCategoryId());
            oldSku.setMenderId(context.getTarget());
            oldSku.setGmtModify(new Date());
            oldSku.setCode(sku.getCode());
            oldSku.setTariffCode(sku.getTariffCode());
            boolean flag = serviceSkuRepository.update(oldSku);
            if (!flag) {
                return new Result(false, "修改失败");
            }
            if (!oldSku.getSpec().equals(sku.getSpec())) {
                List<SkuSpec> specs = skuSpecRepository.listBySkuId(id);
                List<SkuSpec> newSpecs = JSONArray.parseArray(sku.getSpec(), SkuSpec.class);
                if (ListTool.isEmpty(specs)) {
                    skuSpecRepository.batchSave(newSpecs, newSpecs.size());
                } else {
                    Map<String, SkuSpec> map = new HashMap<>(8);
                    for (SkuSpec spec : specs) {
                        map.put(spec.getSpecName(), spec);
                    }
                    for (SkuSpec newSpec : newSpecs) {
                        SkuSpec spec = map.get(newSpec.getSpecName());
                        if (!newSpec.getSpecValue().equals(spec.getSpecValue())) {
                            spec.setSpecValue(newSpec.getSpecValue());
                            skuSpecRepository.update(spec);
                        }
                    }
                }
            }
            return new Result(true, "修改成功");
        } catch (Exception e) {
            logger.error("Modify sku error:", e);
            return new Result(false, "修改失败");
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result remove(Long id, Long managerId) {
        String path = ServiceSku.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceSku sku = serviceSkuRepository.query(id);
            Assert.notNull(sku, "数据不存在！");
            if (null != sku.getReserved() && sku.getReserved()) {
                return new Result(false, "系统内置数据不允许删除");
            }
            serviceSkuRepository.remove(id, managerId);
            skuSpecRepository.removeBySkuId(id);
            snRepository.removeBySkuId(id);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            logger.error("Remove sku error:", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    public BocloudResult<ServiceSku> getBySpec(String spec, Long categoryId) {
        try {
            ServiceSku sku = this.serviceSkuRepository.getBySpec(spec, categoryId);
            if (null == sku) {
                return new BocloudResult<>(false, "数据不存在");
            }
            return new BocloudResult<>(true, sku, "查询成功");
        } catch (Exception e) {
            logger.error("Get service sku error：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result updateStatus(List<Long> ids, String status, Long userId) {
        for (Long id : ids) {
            String path = ServiceSku.class.getSimpleName() + "_" + id;
            try (BocloudACLock lock = lockFactory.getACLock(path)) {
                Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
                ServiceSku sku = serviceSkuRepository.query(id);
                Assert.notNull(sku, "数据不存在！");
                boolean success = serviceSkuRepository.updateStatus(id, status, userId);
                assert success : "SKU 状态更新失败";
            } catch (Exception e) {
                logger.error("batch modify sku status error:", e);
                return new Result(false, "修改失败：" + e.getMessage());
            }
        }
        return new Result(true, "修改成功");

    }

    @Override
    public BocloudResult<List<ServiceSku>> listByTemplate(Long templateId) {
        try {
            List<ServiceSku> list = serviceSkuRepository.listByTemplate(templateId);
            return new BocloudResult<>(true, list, "查询成功");
        } catch (Exception e) {
            logger.error("List service sku by template fail:", e);
            return new BocloudResult<>(false, "通过模板查询服务产品失败");
        }
    }

    @Override
    public BocloudResult<List<ServiceSku>> list(List<Param> params) {
        try {
            params = Optional.ofNullable(params).orElse(new ArrayList<>());
            params.add(new Param(MapTools.simpleMap("status", ServiceSku.Status.enable.name()), Sign.EQ));
            List<ServiceSku> skus = this.serviceSkuRepository.list(params);
            CloudService.BillPolicy billPolicy = null;
            String basicPrice = null;
            Boolean billable = null;
            if (!skus.isEmpty()) {
                Long serviceId = skus.get(0).getServiceId();
                CloudService cloudService = cloudServiceRepository.query(serviceId);
                if (cloudService != null) {
                    billPolicy = cloudService.getBillPolicy();
                    basicPrice = cloudService.getBasicPrice();
                    billable = cloudService.getBillable();
                }
            }
            for (ServiceSku sku : skus) {
                sku.setBillPolicy(billPolicy);
                sku.setBasicPrice(basicPrice);
                sku.setBillable(billable);
            }
            skus.sort(Comparator.comparing(sku -> {
                List<SkuSpec> specs = JSONArray.parseArray(((ServiceSku) sku).getSpec(), SkuSpec.class);
                return Long.parseLong(specs.get(0).getSpecValue());
            }).thenComparing(sku -> {
                List<SkuSpec> specs = JSONArray.parseArray(((ServiceSku) sku).getSpec(), SkuSpec.class);
                if (specs.size() > 1) {
                    return Long.parseLong(specs.get(1).getSpecValue());
                } else {
                    return 0L;
                }
            }));
            return new BocloudResult<>(true, skus, "查询成功");
        } catch (Exception e) {
            logger.error("Get service skus error：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }
}
