package com.bocloud.cos.service;

import com.bocloud.common.exception.BoCloudDBException;
import com.bocloud.common.model.*;
import com.bocloud.common.model.RequestContext.Catalog;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.ServiceSla;
import com.bocloud.cos.intf.ServiceSlaService;
import com.bocloud.cos.params.ServiceSlaParams;
import com.bocloud.cos.repository.ServiceSlaRepository;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 服务协议业务实现
 *
 * @author weiwei
 * @version 1.0
 * @since 2020年3月11日
 */
@Service
public class ServiceSlaServiceImpl implements ServiceSlaService {

    private static final Logger logger = LoggerFactory.getLogger(ServiceSlaServiceImpl.class);
    private final ServiceSlaRepository serviceSlaRepository;
    private final LockFactory lockFactory;
    private final RedisTemplate<String, String> redisTemplate;

    public ServiceSlaServiceImpl(ServiceSlaRepository serviceSlaRepository, LockFactory lockFactory, RedisTemplate<String, String> redisTemplate) {
        this.serviceSlaRepository = serviceSlaRepository;
        this.lockFactory = lockFactory;
        this.redisTemplate = redisTemplate;
    }

    @Override
    public BocloudResult<GridBean<ServiceSla>> list(Pager pager, RequestContext context) {
        List<ServiceSla> list;
        int total;
        GridBean<ServiceSla> gridBean;
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            sorter.put("gmtCreate", Common.ONE);
            total = serviceSlaRepository.count(params);
            if (pager.getSimple()) {
                List<SimpleBean> beans = serviceSlaRepository.list(params, sorter);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                list = serviceSlaRepository.list(pager.getPage(), pager.getRows(), sorter, params);
                for (ServiceSla sla : list) {
                    sla.setCreatorName(
                            redisTemplate.opsForValue().get(Catalog.User.name() + "_" + sla.getCreatorId()));
                }
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result create(ServiceSlaParams slaParams, RequestContext context) {
        try {
            ServiceSla sla = new ServiceSla();
            BeanUtils.copyProperties(slaParams, sla);
            if (serviceSlaRepository.existByCondition("name", sla.getName())) {
                return new Result(false, "标题已存在");
            }
            sla.setCreatorId(context.getTarget());
            sla.setGmtCreate(new Date());
            sla.setStatus(ServiceSla.Status.enable.name());
            serviceSlaRepository.save(sla);
            return new Result(true, "创建成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, "创建失败");
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result modify(Long id, ServiceSlaParams sla, RequestContext context) {
        String path = ServiceSla.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceSla object = serviceSlaRepository.query(id);
            Assert.notNull(object, "数据不存在！");
            if (!object.getName().equals(sla.getName())
                    && serviceSlaRepository.existByCondition("name", sla.getName())) {
                return new Result(false, "标题已存在");
            }
            if (ServiceSla.Status.disable.name().equalsIgnoreCase(sla.getStatus())) {
                BeanUtils.copyProperties(sla, object);
                object.setMenderId(context.getTarget());
                object.setGmtModify(new Date());
                serviceSlaRepository.update(object);
                return new Result(true, "修改成功");
            }
            return new Result(false, "启用状态下的SLA无法编辑修改");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, "修改失败");
        }
    }

    @Override
    public BocloudResult<ServiceSla> detail(Long id) {
        try {
            ServiceSla sla = serviceSlaRepository.query(id);
            if (null == sla) {
                return new BocloudResult<>(false, "SLA不存在");
            }
            sla.setCreatorName(
                    redisTemplate.opsForValue().get(Catalog.User.name() + "_" + sla.getCreatorId()));
            return new BocloudResult<>(true, sla, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    @Transactional(rollbackFor = BoCloudDBException.class)
    public Result remove(Long id, RequestContext context) {
        String path = ServiceSla.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceSla sla = serviceSlaRepository.query(id);
            Assert.notNull(sla, "数据不存在！");
            // sonar : || 操作符两边返回都一样，删除一个
            if (ServiceSla.Status.disable.name().equalsIgnoreCase(sla.getStatus())
//                    || ServiceSla.Status.disable.name().equalsIgnoreCase(sla.getStatus())
            ) {
                serviceSlaRepository.remove(id, context.getTarget());
                return new Result(true, "删除成功");
            }
            return new Result(false, "启用状态下的SAL无法删除");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage());
            return new Result(false, "删除失败");
        }
    }

    @Override
    public Result updateStatus(Long id, String status, Long userId) {
        String path = ServiceSla.class.getSimpleName() + "_" + id;
        String message = ServiceSla.Status.disable.name().equals(status) ? "禁用" : "启用";
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ServiceSla sla = serviceSlaRepository.query(id);
            Assert.notNull(sla, "数据不存在！");
            serviceSlaRepository.updateStatus(id, status, userId);
            return new Result(true, message + "成功");
        } catch (Exception e) {
            logger.error("modify sla status error,", e);
            return new Result(false, message + "失败");
        }
    }

    @Override
    public Result batchUpdateStatus(List<Long> ids, String status, Long menderId) {
        String message = ServiceSla.Status.disable.name().equals(status) ? "禁用" : "启用";
        for (Long id : ids) {
            String path = ServiceSla.class.getSimpleName() + "_" + id;
            try (BocloudACLock lock = lockFactory.getACLock(path)) {
                Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
                ServiceSla sla = serviceSlaRepository.query(id);
                Assert.notNull(sla, "数据不存在！");
                serviceSlaRepository.updateStatus(id, status, menderId);
            } catch (Exception e) {
                logger.error("batch modify sla status error,", e);
                return new Result(false, message + "失败");
            }
        }
        return new Result(true, message + "成功");
    }
}
