package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudExchange;
import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.http.HttpClient;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.common.utils.JSONTools;
import com.bocloud.common.utils.ListTool;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.utils.StringUtil;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ResourceQuota;
import com.bocloud.cos.entity.ServiceCatalog;
import com.bocloud.cos.entity.ServiceTenant;
import com.bocloud.cos.intf.ResourceQuotaService;
import com.bocloud.cos.intf.ServiceTenantService;
import com.bocloud.cos.model.GrantBean;
import com.bocloud.cos.model.MetaBean;
import com.bocloud.cos.model.NotifyMessage;
import com.bocloud.cos.model.ServiceBean;
import com.bocloud.cos.repository.CloudServiceRepository;
import com.bocloud.cos.repository.ResourceQuotaRepository;
import com.bocloud.cos.repository.ServiceCatalogRepository;
import com.bocloud.cos.repository.ServiceTenantRepository;
import com.bocloud.cos.internal.SmsInternalService;
import com.bocloud.cos.internal.SmsTenantService;
import com.bocloud.cos.internal.SmsUserService;
import com.bocloud.cos.internel.entity.Favorite;
import com.bocloud.cos.internel.entity.Tenant;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 服务授权
 *
 * @author Administrator
 */
@Service
public class ServiceTenantServiceImpl implements ServiceTenantService {
    private static final Logger logger = LoggerFactory.getLogger(ServiceTenantServiceImpl.class);
    private final String CONTAINER_SERVICE_CODE = "vmware.k8s.cluster";
    private final CloudServiceRepository cloudServiceRepository;
    private final SmsTenantService tenantService;
    private final ResourceQuotaRepository quotaRepository;
    private final AmqpTemplate amqpTemplate;
    private final LockFactory lockFactory;
    private final SmsInternalService smsInternalService;
    private final SmsUserService smsUserService;
    private final SmsTenantService smsTenantService;
    private final ResourceQuotaService resourceQuotaService;
    private final ServiceCatalogRepository catalogRepository;
    private final ThreadPoolTaskExecutor taskExecutor;
    private final ServiceTenantRepository serviceTenantRepository;

    public ServiceTenantServiceImpl(CloudServiceRepository cloudServiceRepository, SmsTenantService tenantService,
                                    ResourceQuotaRepository quotaRepository, AmqpTemplate amqpTemplate, LockFactory lockFactory,
                                    SmsInternalService smsInternalService, SmsUserService smsUserService, SmsTenantService smsTenantService,
                                    ResourceQuotaService resourceQuotaService, ServiceCatalogRepository catalogRepository,
                                    ThreadPoolTaskExecutor taskExecutor, ServiceTenantRepository serviceTenantRepository) {
        this.cloudServiceRepository = cloudServiceRepository;
        this.tenantService = tenantService;
        this.quotaRepository = quotaRepository;
        this.amqpTemplate = amqpTemplate;
        this.lockFactory = lockFactory;
        this.smsInternalService = smsInternalService;
        this.smsUserService = smsUserService;
        this.smsTenantService = smsTenantService;
        this.resourceQuotaService = resourceQuotaService;
        this.catalogRepository = catalogRepository;
        this.taskExecutor = taskExecutor;
        this.serviceTenantRepository = serviceTenantRepository;
    }

    /**
     * 查看租户授权服务详情
     *
     * @param serviceId 服务id
     * @return
     */
    @Override
    public BocloudResult<List<ServiceTenant>> detail(Long serviceId) {
        try {
            List<ServiceTenant> tenants = serviceTenantRepository.listTenant(serviceId);
            return new BocloudResult<>(true, tenants, "查询成功");
        } catch (Exception e) {
            logger.error("Query Service Item error:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 服务目录授众
     *
     * @param serviceIds     服务id集合
     * @param serviceIds     租户id集合
     * @param type           操作类型
     * @param requestContext 请求对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class, propagation = Propagation.SUPPORTS)
    public Result grantBatch(List<Long> serviceIds, List<Long> tenantIds, String type, RequestContext requestContext)
            throws InternalServerException {
        if (null == type) {
            return new Result(false, "参数格式错误");
        }
        if (ListTool.isEmpty(serviceIds)) {
            return new Result(false, "请至少选择一个服务进行授权");
        }
        if (ListTool.isEmpty(tenantIds)) {
            return new Result(false, "请至少选择一个租户进行授权");
        }
        String path = ServiceTenant.class.getSimpleName() + "_" + requestContext.getTarget();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Map<Long, Tenant> tenantMap = smsTenantService.listByIds(tenantIds, "NORMAL").stream()
                    .collect(Collectors.toMap(Tenant::getId, Function.identity()));
            switch (type) {
                case "add":
                    return grantServicesToTenants(serviceIds, tenantIds, tenantMap, requestContext);
                case "remove":
                    return removeTenantsServices(serviceIds, tenantIds, requestContext);
                default:
                    return new Result(false, "不支持该操作");
            }
        } catch (Exception e) {
            logger.error("Grant Service Item to tenant error:", e);
            throw new InternalServerException("批量操作失败", e, e.getMessage());
        }
    }

    private Result grantServicesToTenants(List<Long> serviceIds, List<Long> tenantIds, Map<Long, Tenant> tenantMap,
                                          RequestContext requestContext) throws Exception {
        // 用于创建租户与服务关联的集合
        List<ServiceTenant> itemTenants = new ArrayList<>();
        // 用于接收创建收藏的集合
        ArrayList<Favorite> favorites = new ArrayList<>();
        // 用于接收添加配额的集合
        ArrayList<ResourceQuota> addQuota = new ArrayList<>();
        // 用于接收授权容器服务的集合
        ArrayList<Long> containerTenants = new ArrayList<>();
        for (Long serviceId : serviceIds) {
            for (Long tenantId : tenantIds) {
                ServiceTenant itemTenant = serviceTenantRepository.getByTenant(tenantId, serviceId);
                if (null == itemTenant) {
                    // 添加租户服务关联集合
                    itemTenants.add(new ServiceTenant(serviceId, tenantId));
                    // 添加租户收藏
                    favorites.add(new Favorite(null, Favorite.Category.Tenant, tenantId, serviceId, 0));
                    CloudService cloudService = cloudServiceRepository.query(serviceId);
                    // 判断是否存在容器服务 存在则添加租户到容器租户集合（后续处理容器租户创建问题）
                    if (cloudService.getCode().equals(CONTAINER_SERVICE_CODE)) {
                        containerTenants.add(tenantId);
                    }
                    Tenant tenant = tenantMap.get(tenantId);
                    if (tenant.getQuotaAble()) {
                        ResourceQuota quota = quotaRepository.query(tenantId, serviceId);
                        // 如果配额不存在（新增加的服务）则新增配额对象到配额集合
                        if (quota == null && StringUtil.isNotEmpty(cloudService.getUnit())) {
                            addQuota.add(
                                    new ResourceQuota(tenantId, 0L, 0L, cloudService.getUnit(), cloudService.getCode(),
                                            ResourceQuota.Category.Service));
                        }
                    }
                }
            }
        }
        // 处理租户服务授权
        if (!ListTool.isEmpty(itemTenants)) {
            serviceTenantRepository.batchSave(itemTenants, 1000);
            // 如果容器租户集合中有租户则创建对应的容器租户
            for (Long tenant : containerTenants) {
                createContainerTenant(tenant, requestContext);
            }
            // 处理租户服务收藏 收藏历史
            tenantService.addFavorites(favorites);
            if (!addQuota.isEmpty()) {
                Map<Long, List<ResourceQuota>> addQuotaMap =
                        addQuota.stream().collect(Collectors.groupingBy(ResourceQuota::getTarget));
                for (Map.Entry<Long, List<ResourceQuota>> entry : addQuotaMap.entrySet()) {
                    Long tenantId = entry.getKey();
                    List<ResourceQuota> quotas = entry.getValue();
                    resourceQuotaService.assign(tenantId, quotas, requestContext);
                }
            }
        }
        return new Result(true, "授权成功");
    }

    private Result removeTenantsServices(List<Long> serviceIds, List<Long> tenantIds, RequestContext requestContext)
            throws Exception {
        for (Long serviceId : serviceIds) {
            for (Long tenantId : tenantIds) {
                ResourceQuota quota = quotaRepository.query(tenantId, serviceId);
                if (quota != null) {
                    if (quota.getUsed() <= 0) {
                        // 资源配额无使用则可以删除
                        resourceQuotaService
                                .delete(tenantId, quota.getCategory().name(), quota.getCode(), requestContext);
                    } else {
                        // 资源配额有使用，不能解除服务授权
                        return new Result(false, "批量操作失败,租户下存在服务配额使用不为空，无法解除服务授权");
                    }
                }
                // 解除该租户的授权服务
                this.cloudServiceRepository.deleteTenant(tenantId, serviceId);
                // 解除租户的服务收藏
                tenantService.cancelFavorite(tenantId, serviceId);
            }
        }
        return new Result(true, "授权成功");
    }

    /**
     * 授权服务
     *
     * @param id             租户id
     * @param grantBean      授权对象
     * @param requestContext
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class, propagation = Propagation.SUPPORTS)
    public Result grantService(Long id, GrantBean grantBean, RequestContext requestContext)
            throws InternalServerException {
        String path = ServiceTenant.class.getSimpleName() + "_" + requestContext.getTarget();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Tenant tenant = smsTenantService.detail(id);
            Assert.notNull(tenant, "租户不存在");
            List<ServiceBean> services = grantBean.getServices();
            //获取需要授权的服务id集合
            Set<Long> serviceIds = services.stream().map(ServiceBean::getServiceId).collect(Collectors.toSet());
            List<MetaBean> mates = grantBean.getMetas();
            //不开启配额 只处理授权
            if (!grantBean.getQuotaAble()) {
                Result grantResult = grantTenantService(id, serviceIds);
                if (grantResult.isFailed()) {
                    return grantResult;
                }
            } else {
                //配额处理 资源配额加入服务配额中
                for (MetaBean mate : mates) {
                    services.add(new ServiceBean(null, mate.getCode(), mate.getQuota(), mate.getUnit()));
                }
                Result quotaResult = grantTenantQuota(id, services);
                if (quotaResult.isFailed()) {
                    return quotaResult;
                }
                //处理授权信息
                Result grantResult = grantTenantService(id, serviceIds);
                if (grantResult.isFailed()) {
                    return grantResult;
                }
            }
            //配额开启设置变化则需要更新租户信息
            if (tenant.getQuotaAble() ^ grantBean.getQuotaAble()) {
                tenant.setQuotaAble(grantBean.getQuotaAble());
                Result result = smsTenantService.update(tenant);
                if (result.isFailed()) {
                    return new Result(false, "更新租户授权配置失败");
                }
            }
            return new Result(true, "授权成功");
        } catch (Exception e) {
            logger.error("Grant Service Item to tenant error:", e);
            throw new InternalServerException("授权失败", e, e.getMessage());
        }
    }

    private Result grantTenantQuota(Long id, List<ServiceBean> services) {
        try {
            //查询租户已经有的配额
            Map<String, ResourceQuota> tenantQuotaMap = quotaRepository.list(id, null, null).stream().collect(Collectors.toMap(ResourceQuota::getCode, Function.identity()));
            //特殊处理 避免配额报错
            services.removeIf(serviceBean -> !StringUtils.hasText(serviceBean.getUnit()) || null == serviceBean.getQuota());
            //授权配额的编码
            List<String> quotaCodes = services.stream().map(ServiceBean::getCode).collect(Collectors.toList());
            //判断配额是否能否被回收
            Result flag = checkServiceQuota(quotaCodes, tenantQuotaMap);
            if (flag.isFailed()) {
                return flag;
            }
            List<ResourceQuota> addQuotas = new ArrayList<>();
            List<ResourceQuota> updateQuotas = new ArrayList<>();
            List<String> removeQuotaCodes = new ArrayList<>();
            for (Map.Entry<String, ResourceQuota> entry : tenantQuotaMap.entrySet()) {
                String quotaCode = entry.getKey();
                //旧配额不存在新配额code集合 说明配额被回收
                if (!quotaCodes.contains(quotaCode)) {
                    removeQuotaCodes.add(quotaCode);
                }
            }
            for (ServiceBean service : services) {
                String code = service.getCode();
                ResourceQuota quota = tenantQuotaMap.get(code);
                //有单位的才处理配额
                if (StringUtils.hasText(service.getUnit())) {
                    //存在旧配额则加入更新集合
                    if (quota != null) {
                        quota.setQuota(service.getQuota());
                        updateQuotas.add(quota);
                    } else {
                        //不存在旧配额则加入新增配额集合
                        ResourceQuota resourceQuota =
                                new ResourceQuota(id, service.getQuota(), 0L, service.getUnit(), service.getCode(), null);
                        //有服务id则是服务配额
                        if (service.getServiceId() != null) {
                            resourceQuota.setCategory(ResourceQuota.Category.Service);
                        } else {
                            //无服务id则是基础配额
                            resourceQuota.setCategory(ResourceQuota.Category.Meta);
                        }
                        addQuotas.add(resourceQuota);
                    }
                }
            }
            //处理删除回收的配额
            if (!removeQuotaCodes.isEmpty()) {
                quotaRepository.delete(removeQuotaCodes);
            }
            //处理新增的配额
            if (!addQuotas.isEmpty()) {
                quotaRepository.batchSave(addQuotas, addQuotas.size());
            }
            //处理更新的配额
            if (!updateQuotas.isEmpty()) {
                quotaRepository.batchUpdate(updateQuotas);
            }
            return new Result(true, "授权配额成功");
        } catch (Exception e) {
            logger.error("grant tenant quota error,", e);
            return new Result(false, "授权配额失败：", e.getMessage());
        }

    }

    private Result grantTenantService(Long id, Set<Long> serviceIds) {
        try {
            List<ServiceTenant> serviceTenants = new ArrayList<>();
            List<Favorite> favorites = new ArrayList<>();
            for (Long serviceId : serviceIds) {
                ServiceTenant serviceTenant = new ServiceTenant(serviceId, id);
                serviceTenants.add(serviceTenant);
                Favorite favorite = new Favorite(null, Favorite.Category.Tenant, id, serviceId, 0);
                favorites.add(favorite);
            }
            // 先解除租户原有的服务授权
            this.cloudServiceRepository.deleteTenant(id, null);
            // 解除租户原有的收藏夹
            this.tenantService.cancelFavorite(id, null);
            //添加新的服务授权和收藏夹
            if (!serviceTenants.isEmpty()) {
                // 绑定当前授权的服务
                serviceTenantRepository.batchSave(serviceTenants, serviceTenants.size());
            }
            if (!favorites.isEmpty()) {
                // 绑定当前授权的服务
                smsTenantService.addFavorites(favorites);
            }
            return new Result(true, "授权成功");
        } catch (Exception e) {
            logger.error("grant tenant service error,", e);
            return new Result(false, "授权租户服务失败:", e.getMessage());
        }
    }

    /**
     * 校验服务配额能否被回收（有配额使用不能回收）
     *
     * @param quotaCodes 授权配额编码集合
     * @param quotaMap   已经授权配额集合
     * @return
     */
    private Result checkServiceQuota(List<String> quotaCodes, Map<String, ResourceQuota> quotaMap) {
        for (Map.Entry<String, ResourceQuota> entry : quotaMap.entrySet()) {
            String code = entry.getKey();
            //授权配额编码不包含原有配额编码(配额被移除了)
            if (!quotaCodes.contains(code)) {
                ResourceQuota quota = entry.getValue();
               // if (quota.getUsed() > 0) {
                if (quota.getUsed() > 0 && quota.getName() != null) {
                    return new Result(false, quota.getName() + "配额存在使用,无法回收");
                }
            }
        }
        return new Result(true, "校验通过");
    }

    /**
     * 服务授权多个租户
     *
     * @param serviceId      授权服务id
     * @param tenantIds      租户id
     * @param requestContext 请求对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class, propagation = Propagation.SUPPORTS)
    public Result grantTenant(Long serviceId, List<Long> tenantIds, RequestContext requestContext)
            throws InternalServerException {
        if (null == serviceId) {
            return new Result(false, "服务为空，授权失败");
        }
        String path = ServiceTenant.class.getSimpleName() + "_" + requestContext.getTarget();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            if (ListTool.isEmpty(tenantIds)) {
                // 如果租户id集合为空，则取消所有租户该服务及配额
                List<ResourceQuota> resourceQuotas = quotaRepository.listServiceQuota(serviceId);
                // 有配额使用则无法取消服务授权
                if (resourceQuotas.stream().anyMatch(resourceQuota -> resourceQuota.getUsed() > 0)) {
                    return new Result(false, "授权失败,服务存在租户已使用配额，不能被解除");
                }
                // 取消服务授权
                this.cloudServiceRepository.deleteTenant(null, serviceId);
                // 取消租户服务收藏
                tenantService.cancelFavorite(null, serviceId);
                // 删除配额
                CloudService cloudService = cloudServiceRepository.query(serviceId);
                resourceQuotaService
                        .delete(null, ResourceQuota.Category.Service.name(), cloudService.getCode(), requestContext);
            } else {
                // 获取该服务所有已经存在租户服务授权关系
                List<ServiceTenant> tenants = serviceTenantRepository.listTenant(serviceId);
                // 用于接收当前已授权该服务的租户ID集合
                List<Long> currentTenantIds = Lists.newArrayList();
                // 用于接收需要解除该服务的租户id集合
                List<Long> removeTenantIds = Lists.newArrayList();
                // 用于存放容器服务相关的租户id
                List<Long> containerTenants = Lists.newArrayList();
                for (ServiceTenant tenant : tenants) {
                    // 当前服务授权集合添加已经授权的租户id
                    currentTenantIds.add(tenant.getTenantId());
                    // 如果当前租户id集合中没有已授权租户id，则是需要解除授权的租户，把其id添加到解除服务租户id集合中
                    if (!tenantIds.contains(tenant.getTenantId())) {
                        removeTenantIds.add(tenant.getTenantId());
                    }
                }
                // 用于处理需要增加服务授权的租户id集合
                List<Long> addTenantIds = Lists.newArrayList();
                for (Long tenantId : tenantIds) {
                    // 已经授权该服务的租户id集合不包含当前租户id，则是需要新增授权的租户，添加的增加服务授权的租户id集合
                    if (!currentTenantIds.contains(tenantId)) {
                        addTenantIds.add(tenantId);
                    }
                }
                // 用于处理新增授权服务租户和服务关联的集合
                List<ServiceTenant> addTenants = Lists.newArrayList();
                // 用于处理新增收藏的集合
                List<Favorite> addFavorites = Lists.newArrayList();
                // 用于处理增加配额的集合
                List<ResourceQuota> addQuota = Lists.newArrayList();
                // 遍历新增服务的租户id集合，处理相关新增操作需要的数据集合
                if (!addTenantIds.isEmpty()) {
                    List<Long> quotaAbleTenants =
                            tenantService.listByIds(addTenantIds, "NORMAL").stream().filter(Tenant::getQuotaAble).map(Tenant::getId)
                                    .collect(Collectors.toList());
                    for (Long tenantId : addTenantIds) {
                        // 构建新增租户和服务关联对象集合及新增收藏集合
                        addTenants.add(new ServiceTenant(serviceId, tenantId));
                        addFavorites.add(new Favorite(null, Favorite.Category.Tenant, tenantId, serviceId, 0));
                        CloudService service = cloudServiceRepository.query(serviceId);
                        // 当前服务如果是容器服务的话租户id添加到容器服务租户id集合中
                        if (CONTAINER_SERVICE_CODE.equals(service.getCode())) {
                            containerTenants.add(tenantId);
                        }
                        // 处理需要新增的服务配额
                        if (StringUtil.isNotEmpty(service.getUnit()) && quotaAbleTenants.contains(tenantId)) {
                            addQuota.add(new ResourceQuota(tenantId, 0L, 0L, service.getUnit(), service.getCode(),
                                    ResourceQuota.Category.Service));
                        }
                    }
                }
                if (!ListTool.isEmpty(removeTenantIds)) {
                    // 遍历解除服务的租户id集合，处理相关解除服务操作
                    for (Long tenantId : removeTenantIds) {
                        ResourceQuota quota = quotaRepository.query(tenantId, serviceId);
                        // 判断配额是否可以解除
                        if (quota != null && quota.getUsed() > 0) {
                            return new Result(false, "授权失败,租户存在服务已使用配额，无法解除");
                        }
                        // 解除该租户的授权服务
                        this.cloudServiceRepository.deleteTenant(tenantId, serviceId);
                        // 解除该租户的服务收藏
                        tenantService.cancelFavorite(tenantId, serviceId);
                        // 删除配额信息
                        CloudService cloudService = cloudServiceRepository.query(serviceId);
                        resourceQuotaService
                                .delete(tenantId, ResourceQuota.Category.Service.name(), cloudService.getCode(),
                                        requestContext);
                        // 默认租户角色服务权限调整
                        tenantService.changeServiceRole(tenantId, Lists.newArrayList(serviceId), "delete");
                    }
                }
                boolean flag = true;
                if (!ListTool.isEmpty(addTenants)) {
                    // 批量新增租户授权关系
                    flag = serviceTenantRepository.batchSave(addTenants, addTenantIds.size());
                    for (Long tenant : containerTenants) {
                        createContainerTenant(tenant, requestContext);
                    }
                    // 批量处理添加租户收藏夹
                    tenantService.addFavorites(addFavorites);
                    // 处理新增配额
                    if (!addQuota.isEmpty()) {
                        Map<Long, List<ResourceQuota>> addQuotaMap =
                                addQuota.stream().collect(Collectors.groupingBy(ResourceQuota::getTarget));
                        for (Map.Entry<Long, List<ResourceQuota>> entry : addQuotaMap.entrySet()) {
                            Long tenantId = entry.getKey();
                            List<ResourceQuota> quotas = entry.getValue();
                            resourceQuotaService.assign(tenantId, quotas, requestContext);
                        }
                    }
                    // 角色调整
                    for (Long tenantId : addTenantIds) {
                        tenantService.changeServiceRole(tenantId, Lists.newArrayList(serviceId), "update");
                    }
                }
                if (!flag) {
                    return new Result(false, "授权失败");
                }
            }
        } catch (Exception e) {
            logger.error("Grant Service Item to tenant error:", e);
            throw new InternalServerException("授权失败", e, e.getMessage());
        }
        return new Result(true, "授权成功");
    }

    /**
     * 使用一个线程异步调用容器接口创建容器租户
     *
     * @param tenantId       租户id
     * @param requestContext 请求对象
     */
    private void createContainerTenant(Long tenantId, RequestContext requestContext) {
        // 从系统配置中获取容器地址
        String apiAddress = smsInternalService.getSystemConfig("K8sAddress");
        taskExecutor.execute(() -> {
            Tenant tenant = tenantService.detail(tenantId);
            String url = apiAddress + "/upmsapi/env/createUserEnv";
            HttpClient httpClient = new HttpClient();
            httpClient.setTimeout(5000);
            // 构建请求参数
            Map<String, Object> params = MapTools.simpleMap("userId", tenant.getId());
            params.put("userName", tenant.getAccount());
            params.put("userRealName", tenant.getName());
            Result result = httpClient.post(null, params, url);
            // 请求调用失败
            if (result.isFailed()) {
                // 回滚容器服务
                callBackContainer(tenantId);
                // 发送消息，接口存在问题
                String messageContent =
                        "id为:" + tenant.getId() + ",账号为：" + tenant.getAccount() + ",租户名称为：" + tenant.getName()
                                + "的租户,创建容器租户失败,接口异常。";
                NotifyMessage notifyMessage =
                        new NotifyMessage(requestContext, "创建容器租户失败", messageContent, true, false);
                this.amqpTemplate
                        .convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(), "event.notice.queue", notifyMessage);
            }
            String resultString = result.getMessage();
            JSONObject jsonObj = JSONTools.isJSONObj(resultString);
            // 请求调用成功，但是响应编码不是200
            if (jsonObj.getInteger("code") != 200) {
                callBackContainer(tenantId);
                // 发送消息，接口无问题，接口创建租户过程存在问题。
                String messageContent =
                        "id为:" + tenant.getId() + ",账号为：" + tenant.getAccount() + ",租户名称为：" + tenant.getName()
                                + "的租户,创建容器租户失败,创建容器租户出现异常。";
                NotifyMessage notifyMessage =
                        new NotifyMessage(requestContext, "创建容器租户失败", messageContent, true, false);
                this.amqpTemplate
                        .convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(), "event.notice.queue", notifyMessage);
            }
        });
    }

    /**
     * 解除租户授权服务
     *
     * @param id             租户id
     * @param requestContext 请求对象
     * @return
     * @throws InternalServerException
     */
    @Override
    @Transactional(rollbackFor = InternalServerException.class, propagation = Propagation.SUPPORTS)
    public Result deleteByTenant(Long id, RequestContext requestContext) throws InternalServerException {
        String path = ServiceTenant.class.getSimpleName() + "_" + requestContext.getTarget();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            // 判断租户下服务配额是否使用，有使用无法删除
            List<ResourceQuota> resourceQuotas = quotaRepository.listByTenantId(id);
            if (resourceQuotas.stream().anyMatch(resourceQuota -> resourceQuota.getUsed() > 0)) {
                return new Result(false, "删除租户服务关系失败,存在服务配额已使用。");
            }
            List<Long> services = this.cloudServiceRepository.listTenantService(id).stream().map(CloudService::getId)
                    .collect(Collectors.toList());
            // 解绑租户服务授权
            this.cloudServiceRepository.deleteByTenant(id);
            // 取消租户服务收藏夹
            tenantService.cancelFavorite(id, null);
            // 删除租户配额
            resourceQuotaService.delete(id, ResourceQuota.Category.Service.name(), null, requestContext);
            // 角色服务权限调整
            tenantService.changeServiceRole(id, services, "delete");
        } catch (Exception e) {
            logger.error("Grant Service Item to tenant error:", e);
            throw new InternalServerException("删除租户服务关系失败", e, e.getMessage());
        }
        return new Result(true, "删除租户服务关系成功");
    }

    /**
     * 创建容器租户失败回滚
     */
    private void callBackContainer(Long tenantId) {
        try {
            CloudService containerService = cloudServiceRepository.queryByCode(CONTAINER_SERVICE_CODE);
            // 取消服务授权
            this.cloudServiceRepository.deleteTenant(tenantId, containerService.getId());
            // 取消服务收藏
            tenantService.cancelFavorite(tenantId, containerService.getId());
            // 删除服务配额
            quotaRepository.delete(tenantId, ResourceQuota.Category.Service.name(), containerService.getId());
            // 删除角色权限
            tenantService.changeServiceRole(tenantId, Lists.newArrayList(containerService.getId()), "delete");
        } catch (Exception e) {
            logger.error("call back container service error", e);
        }
    }

    @Override
    public BocloudResult<List<CloudService>> listByTenant(String code, String suffix, RequestContext context) {
        try {
            List<CloudService> services = new ArrayList<>();
            if (RequestContext.Catalog.Tenant.equals(context.getCatalog())) {
                ServiceCatalog catalog = catalogRepository.getByCode(code);
                Assert.notNull(catalog, "未找到服务目录");
                List<ServiceTenant> serviceTenants = serviceTenantRepository.listByTenantId(context.getTenant());
                Assert.notEmpty(serviceTenants, "当前租户未授权任何云服务");
                for (ServiceTenant serviceTenant : serviceTenants) {
                    CloudService service = cloudServiceRepository.query(serviceTenant.getServiceId());
                    if (null != service && catalog.getId().equals(service.getCatalogId())) {
                        if (!StringUtil.isEmpty(suffix) && !service.getCode().contains(suffix)) {
                            continue;
                        }
                        if (!"vmware.storage.disk".equalsIgnoreCase(service.getCode())) {
                            services.add(service);
                        }
                    }
                }
            } else {
                ServiceCatalog catalog = catalogRepository.getByCode(code);
                List<CloudService> cloudServices = smsUserService.getUserServices(context.getApiKey());
                services = cloudServices.stream().filter(cloudService -> {
                            if (catalog.getId().equals(cloudService.getCatalogId())) {
                                return StringUtil.isEmpty(suffix) || cloudService.getCode().contains(suffix);
                            }
                            return false;
                        }).filter(cloudService -> !"vmware.storage.disk".equalsIgnoreCase(cloudService.getCode()))
                        .collect(Collectors.toList());
            }
            return new BocloudResult<>(true, services, "查询成功");
        } catch (Exception e) {
            logger.error("Get error mesage：{}", e.getMessage());
            return new BocloudResult<>(false, e.getMessage());
        }
    }
}
