package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.enums.BoCloudExchange;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.entity.CloudService;
import com.bocloud.cos.entity.ServiceSku;
import com.bocloud.cos.entity.ShoppingCart;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.cos.intf.ShoppingCartService;
import com.bocloud.cos.model.*;
import com.bocloud.cos.repository.CloudServiceRepository;
import com.bocloud.cos.repository.ServiceSkuRepository;
import com.bocloud.cos.repository.ShoppingCartRepository;
import com.bocloud.cos.operate.ServiceResourceMapping;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 购物车service实现
 *
 * @author wangyu
 * @version 4.1.0-SNAPSHOT
 * @since 2019/2/18
 */
@Service
public class ShoppingCartServiceImpl implements ShoppingCartService {

    private static final Logger logger = LoggerFactory.getLogger(ShoppingCartServiceImpl.class);

    private final ShoppingCartRepository shoppingCartRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceSkuRepository skuRepository;
    private final LockFactory lockFactory;
    private final ResourceApplicationServiceImpl resourceApplyService;
    private final AmqpTemplate amqpTemplate;

    public ShoppingCartServiceImpl(ShoppingCartRepository shoppingCartRepository, CloudServiceRepository cloudServiceRepository,
                                   ServiceSkuRepository skuRepository, LockFactory lockFactory, ResourceApplicationServiceImpl resourceApplyService,
                                   AmqpTemplate amqpTemplate) {
        this.shoppingCartRepository = shoppingCartRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.skuRepository = skuRepository;
        this.lockFactory = lockFactory;
        this.resourceApplyService = resourceApplyService;
        this.amqpTemplate = amqpTemplate;
    }


    /**
     * 分页查询购物车数据
     *
     * @param pager
     * @param context
     * @return
     */
    @Override
    public BocloudResult<GridBean<ShoppingCart>> list(Pager pager, RequestContext context) {
        List<ShoppingCart> list;
        int total;
        GridBean<ShoppingCart> gridBean;
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            sorter.put("gmtCreate", Common.ONE);
            Long userId = context.getTarget();
            Long ownerId = 0L;
            if (context.getCatalog().equals(RequestContext.Catalog.User)) {
                ownerId = context.getTenant();
            }
            total = this.shoppingCartRepository.count(params, userId, ownerId);
            list = this.shoppingCartRepository.list(pager.getPage(), pager.getRows(), params, sorter, userId, ownerId);
            for (ShoppingCart shoppingCart : list) {
                ApplicationParam applicationParam = JSONObject.parseObject(shoppingCart.getInventory(), ApplicationParam.class);
                List<ElementModel> elements = applicationParam.getElements();
                List<String> elementList = getElementList(elements);
                Integer count = applicationParam.getEmption().getCount();
                shoppingCart.setCount(count);
                shoppingCart.setElementList(elementList);
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list shopping cart failure:", e);
            return new BocloudResult<>(false, "查询失败", null, null);
        }
    }

    private List<String> getElementList(List<ElementModel> elementModels) {

        List<String> resultList = new ArrayList<>();
        // 性能类型
        List<String> performanceList = new ArrayList<>();
        // 规格
        List<String> specList = new ArrayList<>();
        for (ElementModel model : elementModels) {
            Long skuId = model.getSkuId();
            ServiceSku sku = skuRepository.query(skuId);
            if (null != sku) {
                // 具体规格信息
                List<SpecModel> specs = model.getSpecs();
                Integer cpu = null;
                BigDecimal memory = null;
                Integer disk = null;
                if (!ListTool.isEmpty(specs)) {
                    for (SpecModel spec : specs) {
                        if (null != spec.getCpu()) {
                            cpu = spec.getCpu();
                        }
                        if (null != spec.getMemory()) {
                            memory = spec.getMemory();
                        }
                        if (null != spec.getDisk()) {
                            disk = spec.getDisk();
                        }
                    }
                }
                if (ListTool.isEmpty(performanceList) && null != cpu) {
                    Integer count = model.getCount();
                    String version = "单机版";
                    if (null != count && count > 1) {
                        version = "集群版";
                    }
                    performanceList.add("类型：" + version);
                }
                StringBuilder specNames = new StringBuilder();
                if (null != cpu) {
                    specNames.append(cpu).append("C/");
                }
                if (null != memory) {
                    specNames.append(memory).append("GB/");
                }
                if (null != disk) {
                    specNames.append(disk).append("GB");
                }
                if (null != cpu) {
                    specList.add("规格：" + specNames.toString());
                } else if (null != disk) {
                    specList.add("数据盘：" + specNames.toString());
                } else {
                    specList.add("标准版");
                }
            }
        }
        resultList.addAll(performanceList);
        resultList.addAll(specList);
        return resultList;
    }

    @Override
    public Result batchApply(RequestContext context) {
        try {
            Long ownerId = 0L;
            if (context.getCatalog().equals(RequestContext.Catalog.User)) {
                ownerId = context.getTenant();
            }
            List<ShoppingCart> shoppingCarts = this.shoppingCartRepository.list(1, Integer.MAX_VALUE, null, null, context.getTarget(), ownerId);
            for (ShoppingCart shoppingCart : shoppingCarts) {
                String inventory = shoppingCart.getInventory();
                ApplicationParam applicationParam = JSONObject.parseObject(inventory, ApplicationParam.class);
                BocloudResult result = resourceApplyService.process(applicationParam, context);
                Assert.isTrue(result.isSuccess(), result.getMessage());
                shoppingCartRepository.remove(shoppingCart.getId(), context.getTarget());
            }
            shoppingCarUpdate(context);
            return new BocloudResult<>(true, "购物车申请订购成功");
        } catch (Exception e) {
            logger.error("list shopping cart failure:", e);
            shoppingCarUpdate(context);
            return new BocloudResult<>(false, "购物车申请订购失败");
        }
    }

    @Override
    public Result update(Long id, ApplicationParam applicationParam, RequestContext context) {
        try {
            ShoppingCart cart = shoppingCartRepository.query(id);
            Assert.notNull(cart, "购物车信息不存在");
            cart.setInventory(JSONObject.toJSONString(applicationParam));
            shoppingCartRepository.update(cart);
            shoppingCarUpdate(context);
            return new BocloudResult(true, "更新购物车信息成功");
        } catch (Exception e) {
            logger.error("update shopping car info error,", e);
            return BocloudResult.FAILED("更新购物车信息成功失败:" + e.getMessage());
        }
    }

    /**
     * 删除购物车数据
     *
     * @param id
     * @param context
     * @return
     */
    @Override
    public Result remove(Long id, RequestContext context) {
        String path = ShoppingCart.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ShoppingCart cart = shoppingCartRepository.query(id);
            if (null == cart) {
                logger.error("Shopping cart does not exist");
                return new Result(false, "购物车清单不存在");
            }
            // 删除购物车清单
            shoppingCartRepository.remove(id, context.getTarget());
            shoppingCarUpdate(context);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new Result(false, "删除失败");
        }
    }

    /**
     * 查询购物车清单详情
     *
     * @param id
     * @return
     */
    @Override
    public BocloudResult<ShoppingCart> detail(Long id) {
        try {
            ShoppingCart cart = shoppingCartRepository.query(id);
            return new BocloudResult<>(true, cart, "查询成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    /**
     * 清空购物车
     *
     * @param context
     * @return
     */
    @Override
    public Result empty(RequestContext context) {
        try {
            if (context.getCatalog().equals(RequestContext.Catalog.User)) {
                shoppingCartRepository.empty(context.getTarget(), context.getTenant());
            } else {
                shoppingCartRepository.empty(context.getTarget(), 0L);
            }
            shoppingCarUpdate(context);
            return new Result(true, "操作成功");
        } catch (Exception e) {
            logger.error("Get error message：", e);
            return new Result(false, "操作失败");
        }
    }

    /**
     * 添加至购物车
     *
     * @param applicationParam
     * @param context
     * @return
     */
    @Override
    public Result create(ApplicationParam applicationParam, RequestContext context) {
        try {
            String service = applicationParam.getService();
            ShoppingCart shoppingCart = new ShoppingCart();
            shoppingCart.setInventory(JSONObject.toJSONString(applicationParam));
            CloudService cloudService = cloudServiceRepository.queryByCode(service);
            shoppingCart.setServiceId(cloudService.getId());
            EmptionModel emption = applicationParam.getEmption();
            shoppingCart.setCount(emption.getCount());
            DurationModel duration = emption.getDuration();
            if (duration.getMode().equals(DurationModel.Mode.Month)) {
                shoppingCart.setMonths(duration.getAmount());
            }
            // 资源类型
            String category = ServiceResourceMapping.getResourceCategory(service).name();
            shoppingCart.setResourceCategory(category);
            LocationModel location = applicationParam.getLocation();
            shoppingCart.setProjectId(location.getProjectId());
            shoppingCart.setTenantId(context.getTenant());
            shoppingCart.setGmtCreate(new Date());
            shoppingCart.setDeleted(false);
            shoppingCart.setCreatorId(context.getTarget());
            if (context.getCatalog().equals(RequestContext.Catalog.Tenant)) {
                shoppingCart.setOwnerId(0L);
            } else {
                shoppingCart.setOwnerId(context.getTenant());

            }
            shoppingCart.setStatus(ChargeCommon.CarStatus.noOrder.name());
            shoppingCartRepository.save(shoppingCart);
            shoppingCarUpdate(context);
            return new Result(true, "添加成功");
        } catch (Exception e) {
            logger.error("add to shopping cart error：", e);
            return new Result(false, "操作失败");
        }
    }

    private void shoppingCarUpdate(RequestContext context) {
        //给前端发送消息以改变安装状态
        OperateResult operateResult = new OperateResult(true, "购物车状态有更新", "shopping.car", null,
                OperateResult.OperateCategory.ShoppingCar, context);
        this.amqpTemplate.convertAndSend(BoCloudExchange.Default.TOPIC.getExchange(), "bocloud.message.queue.#",
                operateResult);
    }
}
