package com.bocloud.cos.service;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.entity.SkuSpec;
import com.bocloud.cos.enums.ServiceCode;
import com.bocloud.cos.intf.SkuSpecService;
import com.bocloud.cos.repository.SkuSpecRepository;
import com.bocloud.cos.internal.ImsServerService;
import com.bocloud.cos.internel.entity.CloudServer;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * SKU规格业务操作实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月10日
 */
@Service
public class SkuSpecServiceImpl implements SkuSpecService {

    private static final Logger logger = LoggerFactory.getLogger(SkuSpecServiceImpl.class);
    private final SkuSpecRepository skuSpecRepository;
    private final ImsServerService imsServerService;

    public SkuSpecServiceImpl(SkuSpecRepository skuSpecRepository, ImsServerService imsServerService) {
        this.skuSpecRepository = skuSpecRepository;
        this.imsServerService = imsServerService;
    }

    @Override
    public BocloudResult<GridBean<SkuSpec>> list(Pager pager) {
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            int total = this.skuSpecRepository.count(params);
            List<SkuSpec> list = this.skuSpecRepository.list(pager.getPage(), pager.getRows(), params, sorter);
            GridBean<SkuSpec> gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query service spec list error:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult<SkuSpec> detail(Long id) {
        try {
            SkuSpec spec = this.skuSpecRepository.query(id);
            assert Objects.nonNull(spec) : "SKU SPEC 数据不存在";
            return new BocloudResult<>(true, spec, "查询成功");
        } catch (Exception e) {
            logger.error("Get service spec error：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult listSpecs(String code, Long categoryId) {
        try {
            ServiceCode serviceCode = ServiceCode.valueOf(code);
            BocloudResult result = new BocloudResult<>(true, "查询成功");
            switch (serviceCode) {
                case ECS:
                    List<SkuSpec> ecsSpecs = skuSpecRepository.listByCategory(code, categoryId);
                    Map<String, Set<SkuSpec>> resultMap = new HashMap<>(8);
                    ecsSpecs.stream().filter(ecsSpec -> "cpu".equalsIgnoreCase(ecsSpec.getSpecName())).forEach(spec -> {
                        SkuSpec memory = ecsSpecs.stream().filter(
                                        mem -> "mem".equalsIgnoreCase(mem.getSpecName()) && mem.getSkuId().equals(spec.getSkuId()))
                                .findFirst().get();
                        Optional.of(memory).ifPresent(mem -> {
                            Set<SkuSpec> memorySet = resultMap.get(spec.getSpecValue());
                            memorySet = Optional.ofNullable(memorySet).orElse(new HashSet<>());
                            memorySet.add(memory);
                            resultMap.put(spec.getSpecValue(), memorySet);
                        });
                    });
                    result.setData(resultMap);
                    break;
                case EVS:
                    List<SkuSpec> evsSpecs = skuSpecRepository.listByCategory(code, categoryId);
                    if (!ListTool.isEmpty(evsSpecs)) {
                        evsSpecs.sort((first, second) -> {
                            if (null == first || second == null) {
                                return 0;
                            }
                            return Integer.valueOf(first.getSpecValue()).compareTo(Integer.valueOf(second.getSpecValue()));
                        });
                    }
                    result.setData(evsSpecs);
                    break;
                case OSS:
                    List<SkuSpec> ossSpecs = skuSpecRepository.listByCategory(code, categoryId);
                    result.setData(ossSpecs);
                    break;
                case BMS:
                case CSBS:
                case VBS:
                case ADSL:
                case VP:
                case ELB:
                case VPN:
                case VPC:
                case WEB:
                case INT:
                case FORTMACHINE:
                    List<SkuSpec> specs = skuSpecRepository.listByCategory(code, categoryId);
                    result.setData(specs);
                    break;
                case MYSQL:
                case REDIS:
                    List<SkuSpec> dbSpecs = skuSpecRepository.listByCondition(categoryId);
                    Map<Long, List<SkuSpec>> specMap = new HashMap<>();
                    for (SkuSpec spec : dbSpecs) {
                        if (specMap.containsKey(spec.getSkuId())) {
                            specMap.get(spec.getSkuId()).add(spec);
                        } else {
                            List<SkuSpec> list = new ArrayList<>();
                            list.add(spec);
                            specMap.put(spec.getSkuId(), list);
                        }
                    }
                    Map<String, List<SkuSpec>> cpuMap = new HashMap<>();
                    Map<String, SkuSpec> memMap = new HashMap<>();
                    for (Long key : specMap.keySet()) {
                        List<SkuSpec> list = specMap.get(key);
                        Map<String, SkuSpec> map = new HashMap<>();
                        for (SkuSpec spec : list) {
                            map.put(spec.getSpecName(), spec);
                        }
                        SkuSpec cpuSpec = map.get("cpu");
                        if (cpuMap.containsKey(cpuSpec.getSpecValue())) {
                            cpuMap.get(cpuSpec.getSpecValue()).add(map.get("mem"));
                        } else {
                            List<SkuSpec> mems = new ArrayList<>();
                            mems.add(map.get("mem"));
                            cpuMap.put(cpuSpec.getSpecValue(), mems);
                        }
                        SkuSpec memSpec = map.get("mem");
                        memMap.put(memSpec.getSkuId() + memSpec.getSpecName() + memSpec.getSpecValue(), map.get("disk"));
                    }
                    Map<String, Map<String, List<SkuSpec>>> specsMap = new HashMap<>();
                    for (String key : cpuMap.keySet()) {
                        Map<String, List<SkuSpec>> valueMap = new HashMap<>();
                        List<SkuSpec> list = cpuMap.get(key);
                        for (SkuSpec spec : list) {
                            SkuSpec diskSpec = memMap.get(spec.getSkuId() + spec.getSpecName() + spec.getSpecValue());
                            if (valueMap.containsKey(spec.getSpecValue())) {
                                valueMap.get(spec.getSpecValue()).add(diskSpec);
                            } else {
                                List<SkuSpec> disks = new ArrayList<>();
                                disks.add(diskSpec);
                                valueMap.put(spec.getSpecValue(), disks);
                            }
                        }
                        specsMap.put(key, valueMap);
                    }
                    result.setData(specsMap);
                    break;
                default:
                    break;
            }
            return result;
        } catch (Exception e) {
            logger.error("Get service spec error：", e);
            return new BocloudResult(false, "查询失败");
        }
    }

    @Override
    public BocloudResult<List<SkuSpec>> listSpecsByCode(Long serverId, String code, String type, RequestContext context) {
        try {
            BocloudResult<CloudServer> boCloudResult = imsServerService.serverDetail(context.getApiKey(), serverId);
            CloudServer server = boCloudResult.getData();
            assert Objects.nonNull(server) : "云主机不存在！";
            long currentCpu = server.getCpu() == null ? 0L : server.getCpu().longValue();
            long currentMem = server.getMemory() == null ? 0L : server.getMemory().longValue();
            List<SkuSpec> serviceSpecs = this.skuSpecRepository.listUpgradeSpec(code);
            assert Objects.nonNull(serviceSpecs) : "服务规格数据不存在！";
            List<SkuSpec> upSpecs = new ArrayList<>();
            List<SkuSpec> lowSpecs = new ArrayList<>();
            for (SkuSpec serviceSpec : serviceSpecs) {
                long cpu = Long.parseLong(serviceSpec.getCpu());
                long mem = Long.parseLong(serviceSpec.getMem());
                if ((cpu >= currentCpu && mem > currentMem) || (cpu > currentCpu && mem >= currentMem)) {
                    upSpecs.add(serviceSpec);
                } else if ((cpu <= currentCpu && mem < currentMem) || (cpu < currentCpu && mem <= currentMem)) {
                    lowSpecs.add(serviceSpec);
                }
            }
            List<SkuSpec> results;
            if ("upgrade".equals(type)) {
                results = upSpecs;
            } else if ("demotion".equals(type)) {
                results = lowSpecs;
            } else {
                results = serviceSpecs;
            }
            return new BocloudResult<>(true, results, "查询成功");
        } catch (Exception e) {
            logger.error("Get service spec error：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

}
