package com.bocloud.cos.service;

import com.bocloud.common.exception.InternalServerException;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.cos.entity.AccountRecord;
import com.bocloud.cos.entity.TenantAccount;
import com.bocloud.cos.entity.TenantAccount.Action;
import com.bocloud.cos.intf.TenantAccountService;
import com.bocloud.cos.repository.AccountRecordRepository;
import com.bocloud.cos.repository.TenantAccountRepository;
import com.bocloud.cos.internal.SmsTenantService;
import com.bocloud.cos.internel.entity.Tenant;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import static com.bocloud.cos.entity.TenantAccount.Action.Increase;
import static com.bocloud.cos.entity.TenantAccount.Action.Reduce;

/**
 * @author tyl
 * @time 2020/7/15 周三 14:14
 */
@Service
public class TenantAccountServiceImpl implements TenantAccountService {
    private final TenantAccountRepository tenantAccountRepository;
    private final LockFactory lockFactory;
    private final StringRedisTemplate redisTemplate;
    private final SmsTenantService tenantService;
    private final Logger logger = LoggerFactory.getLogger(TenantAccountService.class);
    private final AccountRecordRepository accountRecordRepository;

    public TenantAccountServiceImpl(TenantAccountRepository tenantAccountRepository, LockFactory lockFactory,
                                    StringRedisTemplate redisTemplate, SmsTenantService tenantService,AccountRecordRepository accountRecordRepository) {
        this.tenantAccountRepository = tenantAccountRepository;
        this.lockFactory = lockFactory;
        this.redisTemplate = redisTemplate;
        this.tenantService = tenantService;
        this.accountRecordRepository = accountRecordRepository;
    }

    @Override
    @Transactional(rollbackFor = InternalServerException.class, propagation = Propagation.SUPPORTS)
    public Result assignAccount(TenantAccount tenantAccount, RequestContext requestContext)
            throws InternalServerException {
        String path = TenantAccount.class.getSimpleName() + "_" + requestContext.getTarget();
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(20, TimeUnit.SECONDS), "请求超时");
            //管理端只能控制 授信额度的总量（使用量 分配量都在自服务端处理）
            //一级租户授信
            TenantAccount oldAccount = tenantAccountRepository.getAccount(tenantAccount.getTenantId());
            //存在旧的授信账户 则修改账户额度
            if (oldAccount != null) {
                //账户使用总额 = 分配出去的额度+自身使用的额度
                BigDecimal usedTotal = oldAccount.getAccountAllocated().add(oldAccount.getAccountUsed());
                Assert.isTrue(tenantAccount.getAccountTotal().compareTo(usedTotal) >= 0, "授信额度必须大于分配额度与已用额度之和");
                //计算变更的额度
                BigDecimal money = tenantAccount.getAccountTotal().subtract(oldAccount.getAccountTotal());
                Action action = money.compareTo(BigDecimal.ZERO) > 0 ? Increase : Reduce;
                //更新原有账户
                tenantAccountRepository.update(tenantAccount);
                //目标账户记录
                if (money.compareTo(BigDecimal.ZERO) != 0) {
                    AccountRecord record =
                            new AccountRecord(money.abs(), tenantAccount.getTenantId(), action, new Date(),
                                    requestContext.getTarget());
                    accountRecordRepository.saveAccountRecord(record);
                }
                //可超支账号改为不可超支 分出去的部分在账号余额里面扣除
                if (oldAccount.getOverdraft() && !tenantAccount.getOverdraft()) {
                    BigDecimal allocated = oldAccount.getAccountAllocated();
                    //分出去的量如果小于总授信量
                    if (oldAccount.getAccountTotal().subtract(oldAccount.getAccountAllocated())
                            .compareTo(BigDecimal.ZERO) < 0) {
                        tenantAccount.setBalance(tenantAccount.getBalance().subtract(allocated));
                    }
                }
            } else {
                //不存在旧的授信账户 则直接存授信账号
                tenantAccount.setGmtCreate(new Date());
                tenantAccount.setMenderId(requestContext.getTarget());
                tenantAccountRepository.save(tenantAccount);
                if (tenantAccount.getAccountTotal().compareTo(BigDecimal.ZERO) > 0) {
                    AccountRecord record =
                            new AccountRecord(tenantAccount.getAccountTotal(), tenantAccount.getTenantId(), Increase,
                                    new Date(), requestContext.getTarget());
                    accountRecordRepository.saveAccountRecord(record);
                }
            }

            return new Result(true, "授信租户成功");
        } catch (Exception e) {
            logger.error("assign tenant account error,", e);
            throw new InternalServerException("租户授信失败：" + e.getMessage(), e, e.getMessage());
        }
    }

    @Override
    public Result deleteAccount(Long tenantId, RequestContext requestContext) {
        String path = TenantAccount.class.getSimpleName() + "_" + tenantId;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            boolean delete = tenantAccountRepository.delete(tenantId);
            if (delete) {
                Tenant tenant = tenantService.detail(tenantId);
                Assert.notNull(tenant, "租户信息不存在");
                //删除缓存数据
                redisTemplate.delete(tenant.getAccount() + "_overdraft");
                redisTemplate.delete(tenant.getAccount() + "_balance");
                return new Result(true, "删除成功");
            }
            return BocloudResult.FAILED("删除授信账号失败");
        } catch (Exception e) {
            logger.error("delete tenant account error,", e);
            return BocloudResult.FAILED("删除授信账号失败");
        }
    }

    @Override
    @Transactional(rollbackFor = InternalServerException.class, propagation = Propagation.SUPPORTS)
    public Result accountAction(Long tenantId, Long billId, Action action, BigDecimal money,
                                RequestContext requestContext) throws InternalServerException {
        String path = TenantAccount.class.getSimpleName() + "_" + tenantId;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(20, TimeUnit.SECONDS), "请求超时");
            if (money.compareTo(BigDecimal.ZERO) != 0) {
                TenantAccount account = tenantAccountRepository.getAccount(tenantId);
                //如果没有租户授信账号 则给他默认授信账号
                if (account == null) {
                    account =
                            new TenantAccount(tenantId, BigDecimal.ZERO, BigDecimal.ZERO, BigDecimal.ZERO, BigDecimal.ZERO,
                                    true);
                    tenantAccountRepository.save(account);
                }
                AccountRecord record =
                        new AccountRecord(money, tenantId, action, new Date(), requestContext.getTarget());
                switch (action) {
                    //充值 账户金额增加
                    case Recharge:
                        //充值金额大于使用额度 使用清0 剩下的加入余额
                        if (money.subtract(account.getAccountUsed()).compareTo(BigDecimal.ZERO) > 0) {
                            account.setBalance(account.getBalance().add(money.subtract(account.getAccountUsed())));
                            account.setAccountUsed(BigDecimal.ZERO);
                        } else {
                            //充值金额小于使用额度  使用额度减少
                            account.setAccountUsed(account.getAccountUsed().subtract(money));
                        }
                        break;
                    case Callback:
                        //回滚消费
                        if (money.subtract(account.getAccountUsed()).compareTo(BigDecimal.ZERO) > 0) {
                            account.setBalance(account.getBalance().add(money.subtract(account.getAccountUsed())));
                            account.setAccountUsed(BigDecimal.ZERO);
                        } else {
                            //回滚金额小于使用额度  使用额度减少
                            account.setAccountUsed(account.getAccountUsed().subtract(money));
                        }
                        break;
                    case Consume:
                        //不可超支账户  判断余额是否够
                        if (!account.getOverdraft()) {
                            //可消费金额 = 总额度-分配额度-使用额度+账户余额
                            BigDecimal availableBalance = account.getAccountTotal().subtract(account.getAccountAllocated()).subtract(account.getAccountUsed()).add(account.getBalance());
                            //可消费金额小于消费金额
                            if (availableBalance.subtract(money).compareTo(BigDecimal.ZERO) < 0) {
                                return new Result(false, "账户可消费金额不足");
                            }
                        }
                        //消费金额大于余额 余额清零 剩下的扣除授信使用
                        if (money.subtract(account.getBalance()).compareTo(BigDecimal.ZERO) > 0) {
                            account.setAccountUsed(account.getAccountUsed().add(money.subtract(account.getBalance())));
                            account.setBalance(BigDecimal.ZERO);
                        } else {
                            //消费金额小于余额  直接扣除余额
                            account.setAccountUsed(account.getAccountUsed().add(money));
                            account.setBalance(account.getBalance().subtract(money));
                        }
                        //消费记录对应相应账单明细id
                        record.setBillId(billId);
                        break;
                    default:
                        //其他操作（分配、设置） 都在assignAccount里面
                        break;
                }
                tenantAccountRepository.update(account);
                accountRecordRepository.saveAccountRecord(record);
                Tenant tenant = tenantService.detail(account.getTenantId());
                Assert.notNull(tenant, "租户信息不存在");
                if (account.getOverdraft()) {
                    //可透支的
                    redisTemplate.opsForValue().set(tenant.getAccount() + "_overdraft", "1");
                } else {
                    //授信总额度-分配额度-使用额度+账户余额 = 实际可用金额
                    BigDecimal availableBalance = account.getAccountTotal().subtract(account.getAccountAllocated()).subtract(account.getAccountUsed()).add(account.getBalance());
                    redisTemplate.opsForValue().set(tenant.getAccount() + "_balance", availableBalance.toString());
                }
            }
            return new Result(true, "租户账户操作成功");
        } catch (Exception e) {
            logger.error("get tenant account action error", e);
            throw new InternalServerException("租户授信账号操作失败：" + e.getMessage(), e, e.getMessage());
        }
    }

    @Override
    public BocloudResult<TenantAccount> accountDetail(Long tenantId, RequestContext requestContext) {
        try {
            TenantAccount account = tenantAccountRepository.getAccount(tenantId);
            //有账户则直接获取计算剩余额度 无账户则构建初始化账户
            if (account != null) {
                //剩余额度
                BigDecimal freeBalance = account.getAccountTotal().subtract(account.getAccountAllocated())
                        .subtract(account.getAccountUsed());
                account.setBalanceFree(freeBalance.compareTo(BigDecimal.ZERO) > 0 ? freeBalance : BigDecimal.ZERO);
            } else {
                account =
                        new TenantAccount(tenantId, BigDecimal.ZERO, BigDecimal.ZERO, BigDecimal.ZERO, BigDecimal.ZERO,
                                true);
                account.setBalanceFree(BigDecimal.ZERO);
            }
            //管理端一级租户 最大没限制  最小受分配量限制
            account.setMaxBalance(BigDecimal.valueOf(9999999L));
            BigDecimal assigned = Optional.ofNullable(account.getAccountAllocated()).orElse(BigDecimal.ZERO);
            BigDecimal used = Optional.ofNullable(account.getAccountUsed()).orElse(BigDecimal.ZERO);
            account.setMinBalance(assigned.add(used));

            return new BocloudResult<>(true, account, "查询成功");
        } catch (Exception e) {
            logger.error("get tenant account detail error", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public BocloudResult<List<AccountRecord>> accountRecord(Long tenantId, String action, String start, String end,
                                                            RequestContext requestContext) {
        try {
            List<AccountRecord> records =
                    accountRecordRepository.getRecords(tenantId, TenantAccount.Action.valueOf(action), start, end);
            return new BocloudResult<>(true, records, "查询成功");
        } catch (Exception e) {
            logger.error("get tenant account record error", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }
}
