package com.bocloud.cos.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.internel.entity.Tenant;
import com.bocloud.cos.utils.MathUtils;
import com.bocloud.cos.entity.*;
import com.bocloud.cos.enums.ChargeCommon;
import com.bocloud.cos.enums.QuotaCategory;
import com.bocloud.cos.intf.TenantAnalysisService;
import com.bocloud.cos.model.ChargeResultTrend;
import com.bocloud.cos.model.ChargeTrendData;
import com.bocloud.cos.repository.*;
import com.bocloud.cos.internal.ImsInternalService;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.cos.internel.entity.CloudVendor;
import com.google.common.collect.Maps;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.streaming.SXSSFRow;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.groupingBy;

/**
 * @author tyl
 * @time 2020/6/29 周一 17:01
 */
@Service
public class TenantAnalysisServiceImpl implements TenantAnalysisService {
    private final ResourceQuotaRepository resourceQuotaRepository;
    private final ServiceOrderRepository serviceOrderRepository;
    private final CloudServiceRepository cloudServiceRepository;
    private final ServiceBillRepository serviceBillRepository;
    private final ImsInternalService imsInternalService;
    private final ThreadPoolTaskExecutor taskExecutor;
    private final Logger logger = LoggerFactory.getLogger(TenantAnalysisService.class);
    private final QuotaHistoryRepository quotaHistoryRepository;

    private final TenantAccountRepository tenantAccountRepository;

    public TenantAnalysisServiceImpl(ResourceQuotaRepository resourceQuotaRepository,
                                     ServiceOrderRepository serviceOrderRepository, CloudServiceRepository cloudServiceRepository,
                                     ServiceBillRepository serviceBillRepository, ImsInternalService imsInternalService,
                                     ThreadPoolTaskExecutor taskExecutor, QuotaHistoryRepository quotaHistoryRepository,TenantAccountRepository tenantAccountRepository) {
        this.resourceQuotaRepository = resourceQuotaRepository;
        this.serviceOrderRepository = serviceOrderRepository;
        this.cloudServiceRepository = cloudServiceRepository;
        this.serviceBillRepository = serviceBillRepository;
        this.imsInternalService = imsInternalService;
        this.taskExecutor = taskExecutor;
        this.quotaHistoryRepository = quotaHistoryRepository;
        this.tenantAccountRepository = tenantAccountRepository;
    }

    @Override
    public BocloudResult tenantResourceUsed(Long tenantId) {
        JSONArray result = new JSONArray();
        try {
            List<ResourceQuota> metaQuotas =
                    resourceQuotaRepository.list(tenantId, ResourceQuota.Category.Meta.name(), null);
            Map<String, List<ResourceQuota>> resourceQuota =
                    metaQuotas.stream().collect(groupingBy(ResourceQuota::getCode));
            HashMap<String, JSONObject> resultMap = new HashMap<>(4);
            for (Map.Entry<String, List<ResourceQuota>> entry : resourceQuota.entrySet()) {
                List<ResourceQuota> metaQuota = entry.getValue();
                Long total = metaQuota.stream().map(ResourceQuota::getQuota).reduce(Long::sum).orElse(0L);
                Long used = metaQuota.stream().map(ResourceQuota::getUsed).reduce(Long::sum).orElse(0L);
                Long free = total - used;
                double rate = total == 0L ? 0.00 : MathUtils.round(1.00d * used / total, 2);
                String key = entry.getKey();
                if ("meta.snapshot".equals(key)) {
                    continue;
                }
                JSONObject jsonObject = new JSONObject();
                switch (key) {
                    case "meta.cpu":
                        jsonObject.put("name", "CPU配额使用率");
                        jsonObject.put("unit", "C");
                        break;
                    case "meta.mem":
                        jsonObject.put("name", "内存配额使用率");
                        jsonObject.put("unit", "GB");
                        break;
                    case "meta.disk":
                        jsonObject.put("name", "磁盘配额使用率");
                        jsonObject.put("unit", "GB");
                        break;
                    default:
                        break;
                }
                jsonObject.put("total", total);
                jsonObject.put("used", used);
                jsonObject.put("free", free);
                jsonObject.put("value", rate);
                resultMap.put(entry.getKey(), jsonObject);
            }
            if (resultMap.isEmpty()) {
                for (QuotaCategory category : QuotaCategory.values()) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("name", category.getName() + "配额使用率");
                    jsonObject.put("unit", category.getUnit());
                    jsonObject.put("total", 0);
                    jsonObject.put("used", 0);
                    jsonObject.put("free", 0);
                    jsonObject.put("value", 0);
                    resultMap.put(category.getCode(), jsonObject);
                }
            }
            result.add(resultMap.get(QuotaCategory.CPU.getCode()));
            result.add(resultMap.get(QuotaCategory.MEM.getCode()));
            result.add(resultMap.get(QuotaCategory.DISK.getCode()));
            return new BocloudResult(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("get tenant resource used error", e);
            return BocloudResult.FAILED("查询失败");
        }
    }

    @Override
    public BocloudResult tenantAnalysis(String condition, String time, String startTime, String endTime, Long tenantId,
                                        String resourceType) {
        if ("getTop5".equals(condition)) {
            return tenantServiceTop5(startTime, endTime, tenantId);
        }
        Map<String, Object> timeMap = MapTools.simpleMap("time", time);
        timeMap.put("startTime", startTime);
        timeMap.put("endTime", endTime);
        List<String> timeList = DateUtil.timeToList(new JSONObject(timeMap));
        if (timeList.size() <= 1) {
            return BocloudResult.FAILED("时间区间选择太短，请重新选择");
        }
        if ("getResourceTend".equals(condition)) {
            return tenantResourceUsedTrend(timeList, tenantId, resourceType);
        }
        if ("getIncomeTrend".equals(condition)) {
            return tenantIncomeTrend(timeList, tenantId);
        }
        if ("getOrderTrend".equals(condition)) {
            return tenantOrdersTrend(timeList, tenantId);
        }
        if ("getServiceApply".equals(condition)) {
            return tenantServiceApplyTrend(timeList, tenantId);
        }
        return BocloudResult.FAILED("不支持的查询类型");
    }

    @Override
    public BocloudResult<Map<String, Object>> tenantResourceAnalysisDownLoad() {

        SXSSFWorkbook workbook = new SXSSFWorkbook();
        try {
            SXSSFSheet sheet = workbook.createSheet("sheet");
            sheet.setRandomAccessWindowSize(-1);
            // 样式一，标题数据蓝色填充
            CellStyle style = workbook.createCellStyle();
            style.setFillForegroundColor(IndexedColors.ROYAL_BLUE.getIndex());
            style.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            style.setBorderBottom(BorderStyle.THIN); // 下边框
            style.setBorderLeft(BorderStyle.THIN);// 左边框
            style.setBorderTop(BorderStyle.THIN);// 上边框
            style.setBorderRight(BorderStyle.THIN);// 右边框
            style.setAlignment(HorizontalAlignment.CENTER);
            // 样式二，内容样式
            CellStyle cellStyle = workbook.createCellStyle();
            cellStyle.setBorderTop(BorderStyle.THIN);// 上边框
            cellStyle.setBorderBottom(BorderStyle.THIN); // 下边框
            cellStyle.setBorderLeft(BorderStyle.THIN);// 左边框
            cellStyle.setBorderRight(BorderStyle.THIN);// 右边框
            cellStyle.setAlignment(HorizontalAlignment.CENTER);// 水平对齐方式
            cellStyle.setVerticalAlignment(VerticalAlignment.TOP);// 垂直对齐方式
            // 标题写入
            createVmReport(sheet, style);

            List<Tenant> tenantList  = tenantAccountRepository.getAllTenant();
            if(tenantList.isEmpty()){
                return new BocloudResult<>(false,"暂无租户");
            }
            Integer count = 2;
            for (Tenant tenant : tenantList) {
                SXSSFRow sheetRow = sheet.createRow(count);
                sheetRow.createCell(0).setCellValue(tenant.getName());
                sheet.getRow(count).getCell(0).setCellStyle(cellStyle);
                excelExporter(tenant,sheet,cellStyle,sheetRow,count);
                count++;
            }

            String fileName = "租户配额分析";
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            workbook.write(byteArrayOutputStream);
            byte[] ba = byteArrayOutputStream.toByteArray();
            Map<String, Object> resultMap = Maps.newHashMap();
            resultMap.put("fileName", fileName + System.currentTimeMillis() + ".xlsx");
            resultMap.put("data", Base64.getEncoder().encodeToString(ba));
            return new BocloudResult<>(true, resultMap, "下载成功");
        } catch (Exception e) {
            logger.error("Get error message：{}", e.getMessage(), e);
            return new BocloudResult<>(false, e.getMessage());
        } finally {
            if (null != workbook) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    logger.error("error message：", e);
                }
            }
        }
    }

    private void excelExporter(Tenant tenant, SXSSFSheet sheet, CellStyle cellStyle, SXSSFRow sheetRow, Integer count) {
        List<ResourceQuota> metaQuotas =
                resourceQuotaRepository.list(tenant.getId(), ResourceQuota.Category.Meta.name(), null);
        Map<String, List<ResourceQuota>> resourceQuota =
                metaQuotas.stream().collect(groupingBy(ResourceQuota::getCode));
        HashMap<String, JSONObject> resultMap = new HashMap<>(4);
        for (Map.Entry<String, List<ResourceQuota>> entry : resourceQuota.entrySet()) {
            List<ResourceQuota> metaQuota = entry.getValue();
            Long total = metaQuota.stream().map(ResourceQuota::getQuota).reduce(Long::sum).orElse(0L);
            Long used = metaQuota.stream().map(ResourceQuota::getUsed).reduce(Long::sum).orElse(0L);
            Long free = total - used;
            double rate = total == 0L ? 0.00 : MathUtils.round(1.00d * used / total, 2);
            String key = entry.getKey();
            if ("meta.snapshot".equals(key)) {
                continue;
            }
            JSONObject jsonObject = new JSONObject();
            switch (key) {
                case "meta.cpu":
                    jsonObject.put("name", "CPU配额使用率");
                    jsonObject.put("unit", "C");
                    break;
                case "meta.mem":
                    jsonObject.put("name", "内存配额使用率");
                    jsonObject.put("unit", "GB");
                    break;
                case "meta.disk":
                    jsonObject.put("name", "磁盘配额使用率");
                    jsonObject.put("unit", "GB");
                    break;
                default:
                    break;
            }
            jsonObject.put("total", total);
            jsonObject.put("used", used);
            jsonObject.put("free", free);
            jsonObject.put("value", rate);
            resultMap.put(entry.getKey(), jsonObject);
        }
        if (resultMap.isEmpty()) {
            for (QuotaCategory category : QuotaCategory.values()) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("name", category.getName() + "配额使用率");
                jsonObject.put("unit", category.getUnit());
                jsonObject.put("total", 0);
                jsonObject.put("used", 0);
                jsonObject.put("free", 0);
                jsonObject.put("value", 0);
                resultMap.put(category.getCode(), jsonObject);
            }
        }

        JSONObject cpuObj = resultMap.get(QuotaCategory.CPU.getCode());
        JSONObject memObj = resultMap.get(QuotaCategory.MEM.getCode());
        JSONObject diskObj = resultMap.get(QuotaCategory.DISK.getCode());

        sheetRow.createCell(1).setCellValue(cpuObj.getString("total"));
        sheet.getRow(count).getCell(1).setCellStyle(cellStyle);
        sheetRow.createCell(2).setCellValue(cpuObj.getString("used"));
        sheet.getRow(count).getCell(2).setCellStyle(cellStyle);
        sheetRow.createCell(3).setCellValue(cpuObj.getString("free"));
        sheet.getRow(count).getCell(3).setCellStyle(cellStyle);
        sheetRow.createCell(4).setCellValue(memObj.getString("total"));
        sheet.getRow(count).getCell(4).setCellStyle(cellStyle);
        sheetRow.createCell(5).setCellValue(memObj.getString("used"));
        sheet.getRow(count).getCell(5).setCellStyle(cellStyle);
        sheetRow.createCell(6).setCellValue(memObj.getString("free"));
        sheet.getRow(count).getCell(6).setCellStyle(cellStyle);
        sheetRow.createCell(7).setCellValue(diskObj.getString("total"));
        sheet.getRow(count).getCell(7).setCellStyle(cellStyle);
        sheetRow.createCell(8).setCellValue(diskObj.getString("used"));
        sheet.getRow(count).getCell(8).setCellStyle(cellStyle);
        sheetRow.createCell(9).setCellValue(diskObj.getString("free"));
        sheet.getRow(count).getCell(9).setCellStyle(cellStyle);
    }

    /**
     * 创建标题
     *
     * @param sheet
     * @param style
     * @param action
     */
    public void createVmReport(SXSSFSheet sheet, CellStyle style) {
        // 模板标题处理
        String[] titles = new String[]{"租户名称", "CPU配额总量(C)", "CPU配额已使用(C)", "CPU配额未使用(C)", "内存配额总量(GB)", "内存配额已使用(GB)", "内存配额未使用(GB)", "磁盘配额总量(GB)", "磁盘配额已使用(GB)", "磁盘配额未使用(GB)"};
        int number = -1;
        SXSSFRow row = sheet.createRow(0);
        for (String title : titles) {
            number++;
            row.createCell(number).setCellValue(title);
            sheet.getRow(0).getCell(number).setCellStyle(style);
            sheet.setColumnWidth(number, 30 * 256);
            sheet.addMergedRegion(new CellRangeAddress(0, 1, number, number));
        }
    }

    private BocloudResult tenantResourceUsedTrend(List<String> timeList, Long tenantId, String resourceType) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            ArrayList<Object> data = new ArrayList<>();
            List<QuotaHistory> quotaHistories = quotaHistoryRepository
                    .getQuotaUsedHistory(timeList.get(0).substring(0, 10) + " 00:00:00",
                            timeList.get(timeList.size() - 1).substring(0, 10) + " 23:59:59", tenantId, resourceType);
            keys.addAll(timeList);
            for (String time : timeList) {
                Map<ResourceQuota.Action, List<QuotaHistory>> historyMap = quotaHistories.stream()
                        .filter(quotaHistory -> quotaHistory.getGmtCreate().before(DateUtil.timeDateProcess(time)))
                        .collect(groupingBy(QuotaHistory::getAction));
                Long orderCount = 0L;
                if (historyMap.get(ResourceQuota.Action.Order) != null) {
                    orderCount = historyMap.get(ResourceQuota.Action.Order).stream().map(QuotaHistory::getValue)
                            .reduce(Long::sum).orElse(0L);
                }
                Long callBackCount = 0L;
                if (historyMap.get(ResourceQuota.Action.Callback) != null) {
                    callBackCount = historyMap.get(ResourceQuota.Action.Callback).stream().map(QuotaHistory::getValue)
                            .reduce(Long::sum).orElse(0L);
                }
                data.add(orderCount - callBackCount);
            }
            String dataUnit;
            if (QuotaCategory.CPU.getCode().equals(resourceType)) {
                dataUnit = "C";
            } else {
                dataUnit = "GB";
            }
            values.add(new ChargeTrendData("使用量（" + dataUnit + ")", data));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get tenant resource used trend error", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    private BocloudResult tenantServiceTop5(String start, String end, Long tenantId) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            HashMap<String, Long> resultMap = new HashMap<>();
            //查询租户下有的服务code和name映射关系
            Map<String, String> serviceMap;
            if (null != tenantId && 0L != tenantId) {
                serviceMap = cloudServiceRepository.listTenantService(tenantId).stream()
                        .collect(Collectors.toMap(CloudService::getCode, CloudService::getName));
            } else {
                serviceMap = cloudServiceRepository.listByStatus("PUBLISHED").stream()
                        .collect(Collectors.toMap(CloudService::getCode, CloudService::getName));
            }
            //获取时间段内所有配额历史（是否会有内存堆栈溢出问题？）
            Map<String, List<QuotaHistory>> quotaMap =
                    quotaHistoryRepository.getServiceQuotaHistory(start, end, tenantId).stream()
                            .collect(groupingBy(QuotaHistory::getCode));
            for (Map.Entry<String, List<QuotaHistory>> quotaEntry : quotaMap.entrySet()) {
                String serviceCode = quotaEntry.getKey();
                List<QuotaHistory> quotaHistories = quotaEntry.getValue();
                Map<ResourceQuota.Action, List<QuotaHistory>> historyMap =
                        quotaHistories.stream().collect(groupingBy(QuotaHistory::getAction));
                Long orderCount = 0L;
                if (historyMap.get(ResourceQuota.Action.Order) != null) {
                    orderCount = historyMap.get(ResourceQuota.Action.Order).stream().map(QuotaHistory::getValue)
                            .reduce(Long::sum).orElse(0L);
                }
                Long callBackCount = 0L;
                if (historyMap.get(ResourceQuota.Action.Callback) != null) {
                    callBackCount = historyMap.get(ResourceQuota.Action.Callback).stream().map(QuotaHistory::getValue)
                            .reduce(Long::sum).orElse(0L);
                }
                long serviceNumber = orderCount - callBackCount;
                String serviceName = serviceMap.get(serviceCode);
                //订购数量-回滚数量=真实配额使用量
                if (resultMap.containsKey(serviceName)) {
                    resultMap.put(serviceName, resultMap.get(serviceName) + serviceNumber);
                } else {
                    resultMap.put(serviceName, serviceNumber);
                }

            }
            List<Map.Entry<String, Long>> resultEntry =
                    resultMap.entrySet().stream().sorted(((o1, o2) -> o2.getValue().compareTo(o1.getValue()))).limit(5)
                            .collect(Collectors.toList());
            ArrayList<Object> data = new ArrayList<>();
            for (Map.Entry<String, Long> entry : resultEntry) {
                keys.add(entry.getKey());
                data.add(entry.getValue());
            }
            values.add(new ChargeTrendData("资源数(个)", data));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get tenant service top error", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }

    }

    private BocloudResult tenantIncomeTrend(List<String> timeList, Long tenantId) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            List<String> vendorNames =
                    imsInternalService.getVendorList().stream().map(CloudVendor::getName).collect(Collectors.toList());
            List<Object> total = new ArrayList<>();
            ArrayList<Future<List<ServiceBill>>> futures = new ArrayList<>();
            keys.addAll(timeList);
            for (String time : timeList) {
                Future<List<ServiceBill>> future =
                        taskExecutor.submit(() -> serviceBillRepository.listByTenant(null, DateUtil.timeProcess(time), tenantId, null));
                futures.add(future);
            }
            HashMap<String, List<Object>> countMap = new HashMap<>();
            for (Future<List<ServiceBill>> future : futures) {
                BigDecimal totalIncome = BigDecimal.ZERO;
                List<ServiceBill> serviceBills = future.get();
                Map<String, List<ServiceBill>> vendorOrders = serviceBills.stream().peek(serviceBill -> {
                    if (serviceBill.getVendorName() == null) {
                        serviceBill.setVendorName("其他");
                    }
                }).collect(groupingBy(ServiceBill::getVendorName));
                for (String vendorName : vendorNames) {
                    List<ServiceBill> vendorBills =
                            Optional.ofNullable(vendorOrders.get(vendorName)).orElse(new ArrayList<>());
                    BigDecimal income = BigDecimal.ZERO;
                    if (!vendorBills.isEmpty()) {
                        income = vendorBills.stream()
                                .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                                .reduce(BigDecimal.ZERO, BigDecimal::add);
                    }
                    totalIncome = totalIncome.add(income);
                    //统计过了则更新 数据列表 没有统计的则新增数据列表
                    if (countMap.containsKey(vendorName)) {
                        List<Object> countNum = countMap.get(vendorName);
                        countNum.add(income);
                        countMap.put(vendorName, countNum);
                    } else {
                        ArrayList<Object> countNum = new ArrayList<>();
                        countNum.add(income);
                        countMap.put(vendorName, countNum);
                    }
                }
                List<ServiceBill> vendorBills = Optional.ofNullable(vendorOrders.get("其他")).orElse(new ArrayList<>());
                BigDecimal income = BigDecimal.ZERO;
                if (!vendorBills.isEmpty()) {
                    income = vendorBills.stream()
                            .map(serviceBill -> Optional.ofNullable(serviceBill.getMoney()).orElse(BigDecimal.ZERO))
                            .reduce(BigDecimal.ZERO, BigDecimal::add);
                }
                totalIncome = totalIncome.add(income);
                //统计过了则更新 数据列表 没有统计的则新增数据列表
                if (countMap.containsKey("其他")) {
                    List<Object> countNum = countMap.get("其他");
                    countNum.add(income);
                    countMap.put("其他", countNum);
                } else {
                    ArrayList<Object> countNum = new ArrayList<>();
                    countNum.add(income);
                    countMap.put("其他", countNum);
                }
                total.add(totalIncome);
            }
            for (Map.Entry<String, List<Object>> countEntry : countMap.entrySet()) {
                ChargeTrendData chargeTrendData = new ChargeTrendData(countEntry.getKey(), countEntry.getValue());
                values.add(chargeTrendData);
            }
            values.add(new ChargeTrendData("合计(￥)", total));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get tenant income trend error", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    private BocloudResult tenantOrdersTrend(List<String> timeList, Long tenantId) {
        List<String> keys = new ArrayList<>();
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            HashMap<ChargeCommon.OrderStatus, List<Object>> countMap = new HashMap<>();
            ArrayList<Object> total = new ArrayList<>();
            keys.addAll(timeList);
            List<Future<List<ServiceOrder>>> futures = new ArrayList<>();
            for (String time : timeList) {
                Future<List<ServiceOrder>> future =
                        taskExecutor.submit(() -> serviceOrderRepository.listByTenantId(null, DateUtil.timeProcess(time), tenantId));
                futures.add(future);
            }
            for (Future<List<ServiceOrder>> future : futures) {
                List<ServiceOrder> orderList = future.get();
                total.add(orderList.size());
                Map<ChargeCommon.OrderStatus, List<ServiceOrder>> serviceOrders =
                        orderList.stream().collect(groupingBy(ServiceOrder::getOrderStatus));
                for (ChargeCommon.OrderStatus orderStatus : ChargeCommon.OrderStatus.values()) {
                    int orderNum = Optional.ofNullable(serviceOrders.get(orderStatus)).orElse(new ArrayList<>()).size();
                    //统计过了则更新 数据列表 没有统计的则新增数据列表
                    if (countMap.containsKey(orderStatus)) {
                        List<Object> countNum = countMap.get(orderStatus);
                        countNum.add(orderNum);
                        countMap.put(orderStatus, countNum);
                    } else {
                        ArrayList<Object> countNum = new ArrayList<>();
                        countNum.add(orderNum);
                        countMap.put(orderStatus, countNum);
                    }
                }
            }
            for (Map.Entry<ChargeCommon.OrderStatus, List<Object>> entry : countMap.entrySet()) {
                ChargeTrendData chargeTrendData = null;
                ChargeCommon.OrderStatus status = entry.getKey();
                switch (status) {
                    case unpay:
                        chargeTrendData = new ChargeTrendData("未支付", entry.getValue());
                        break;
                    case canceld:
                        chargeTrendData = new ChargeTrendData("已取消", entry.getValue());
                        break;
                    case paid:
                        chargeTrendData = new ChargeTrendData("已支付", entry.getValue());
                        break;
                    case audit:
                        chargeTrendData = new ChargeTrendData("审批中", entry.getValue());
                    default:
                        break;
                }
                values.add(chargeTrendData);
            }
            values.add(new ChargeTrendData("合计(个)", total));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get tenant orders trend error", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

    private BocloudResult tenantServiceApplyTrend(List<String> timeList, Long tenantId) {
        List<String> keys = new ArrayList<>(timeList.size());
        List<ChargeTrendData> values = new ArrayList<>();
        try {
            List<ServiceOrder> orderList = serviceOrderRepository
                    .listByTenantId(timeList.get(0).substring(0, 10) + " 00:00:00",
                            timeList.get(timeList.size() - 1).substring(0, 10) + " 23:59:59", tenantId);
            orderList.removeIf(order -> !StringUtils.hasText(order.getServiceName()) || !order.getType()
                    .equals(ChargeCommon.Type.apply));
            Set<String> serviceNames = orderList.stream().map(ServiceOrder::getServiceName).collect(Collectors.toSet());
            HashMap<String, List<Object>> countMap = new HashMap<>();
            ArrayList<Object> total = new ArrayList<>();
            keys.addAll(timeList);
            for (String time : timeList) {
                int totalOrder = 0;
                List<ServiceOrder> serviceOrderList = orderList.stream()
                        .filter(serviceOrder -> serviceOrder.getGmtCreate().before(DateUtil.timeDateProcess(time)))
                        .collect(Collectors.toList());
                Map<String, List<ServiceOrder>> serviceOrders =
                        serviceOrderList.stream().filter(order -> null != order.getServiceName())
                                .collect(groupingBy(ServiceOrder::getServiceName));
                for (String serviceName : serviceNames) {
                    int orderNum = Optional.ofNullable(serviceOrders.get(serviceName)).orElse(new ArrayList<>()).size();
                    totalOrder += orderNum;
                    //统计过了则更新 数据列表 没有统计的则新增数据列表
                    if (countMap.containsKey(serviceName)) {
                        List<Object> countNum = countMap.get(serviceName);
                        countNum.add(orderNum);
                        countMap.put(serviceName, countNum);
                    } else {
                        ArrayList<Object> countNum = new ArrayList<>(timeList.size());
                        countNum.add(orderNum);
                        countMap.put(serviceName, countNum);
                    }
                }
                total.add(totalOrder);
            }
            for (Map.Entry<String, List<Object>> entry : countMap.entrySet()) {
                ChargeTrendData chargeTrendData = new ChargeTrendData(entry.getKey(), entry.getValue());
                values.add(chargeTrendData);
            }
            values.add(new ChargeTrendData("合计(个)", total));
            return new BocloudResult(true, new ChargeResultTrend(keys, values), "查询成功");
        } catch (Exception e) {
            logger.error("get tenant orders trend error", e);
            return new BocloudResult(false, new ChargeResultTrend(keys, values), "查询失败");
        }
    }

}
