package com.bocloud.cos.service;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.cos.entity.VendorSyncRecord;
import com.bocloud.cos.intf.VendorSyncRecordService;
import com.bocloud.cos.repository.VendorSyncRecordRepository;
import com.bocloud.cos.utils.DateUtil;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * @author admin
 */
@Service
public class VendorSyncRecordServiceImpl implements VendorSyncRecordService {

    private static final Logger logger = LoggerFactory.getLogger(VendorSyncRecordServiceImpl.class);
    private final VendorSyncRecordRepository vendorSyncRecordRepository;
    private final LockFactory lockFactory;

    public VendorSyncRecordServiceImpl(VendorSyncRecordRepository vendorSyncRecordRepository, LockFactory lockFactory) {
        this.vendorSyncRecordRepository = vendorSyncRecordRepository;
        this.lockFactory = lockFactory;
    }

    @Override
    public BocloudResult<GridBean<VendorSyncRecord>> list(Pager pager) {
        GridBean<VendorSyncRecord> gridBean;
        try {
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            sorter.put("endTime", Common.ONE);
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            int total = this.vendorSyncRecordRepository.count(params);
            List<VendorSyncRecord> list = this.vendorSyncRecordRepository.list(pager.getPage(), pager.getRows(), params, sorter);
            for (VendorSyncRecord record : list) {
                double cost = (double) record.getCost();
                record.setCostStr(DateUtil.dateConvert(cost));
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("Query vendor sync record list error:", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result save(VendorSyncRecord record) {
        try {
            vendorSyncRecordRepository.save(record);
        } catch (Exception e) {
            logger.error("Save vendor sync record error:", e);
        }
        return new Result(true, "保存成功");
    }

    @Override
    public Result remove(Long id) {
        String path = VendorSyncRecord.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            VendorSyncRecord object = this.vendorSyncRecordRepository.query(id);
            if (null == object) {
                logger.warn("Vendor sync record does not exist!");
                return new Result(false, "数据不存在");
            }
            this.vendorSyncRecordRepository.delete(id);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            logger.error("Remove vendor sync record error:", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    public BocloudResult<VendorSyncRecord> detail(Long id) {
        try {
            VendorSyncRecord record = this.vendorSyncRecordRepository.query(id);
            if (null == record) {
                return new BocloudResult<>(false, "数据不存在");
            }
            return new BocloudResult<>(true, record, "查询成功");
        } catch (Exception e) {
            logger.error("Get vendor sync record error：", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result removeByVendor(Long vendorId) {
        String path = VendorSyncRecord.class.getSimpleName() + "_" + vendorId;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            VendorSyncRecord object = this.vendorSyncRecordRepository.queryByVendor(vendorId);
            if (null == object) {
                logger.warn("Vendor sync record does not exist!");
                return new Result(false, "数据不存在");
            }
            this.vendorSyncRecordRepository.deleteByVendor(vendorId);
            return new Result(true, "删除成功");
        } catch (Exception e) {
            logger.error("Remove vendor sync record error:", e);
            return new Result(false, "删除失败");
        }
    }

    @Override
    public Result removeByTime(String startTime, String endTime, Long vendorId) {
        try {
            vendorSyncRecordRepository.deleteByTime(startTime, endTime, vendorId);
        } catch (Exception e) {
            logger.error("Remove vendor sync record error:", e);
            return new Result(false, "删除失败");
        }
        return new Result(true, "删除成功");
    }

}
