package com.bocloud.cos.service;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.MapTools;
import com.bocloud.cos.entity.InteractiveRecord;
import com.bocloud.cos.entity.WorkOrder;
import com.bocloud.cos.enums.WorkOrderStatus;
import com.bocloud.cos.intf.WorkOrderService;
import com.bocloud.cos.model.StatsResult;
import com.bocloud.cos.params.WorkOrderParams;
import com.bocloud.cos.repository.InteractiveRecordRepository;
import com.bocloud.cos.repository.WorkOrderRepository;
import com.bocloud.cos.internal.SmsTenantService;
import com.bocloud.cos.internal.SmsUserService;
import com.bocloud.cos.internel.entity.Tenant;
import com.bocloud.cos.internel.entity.User;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * 问题工单业务实现
 *
 * @author wangyu
 * @version 1.0
 * @since 2019年5月8日
 */
@Service
public class WorkOrderServiceImpl implements WorkOrderService {

    private final Logger logger = LoggerFactory.getLogger(WorkOrderServiceImpl.class);
    private final WorkOrderRepository workOrderRepository;
    private final InteractiveRecordRepository recordRepository;
    private final SmsUserService smsUserService;
    private final SmsTenantService smsTenantService;
    private final LockFactory lockFactory;

    public WorkOrderServiceImpl(WorkOrderRepository workOrderRepository, InteractiveRecordRepository recordRepository, SmsUserService smsUserService, SmsTenantService smsTenantService, LockFactory lockFactory) {
        this.workOrderRepository = workOrderRepository;
        this.recordRepository = recordRepository;
        this.smsUserService = smsUserService;
        this.smsTenantService = smsTenantService;
        this.lockFactory = lockFactory;
    }

    @Override
    public Result create(WorkOrderParams orderParams, RequestContext context) {
        try {
            String instanceUuid = orderParams.getInstanceUuid();
            String name = orderParams.getName();
            WorkOrder workOrder = workOrderRepository.detailByUuid(instanceUuid);
            if (null != workOrder) {
                return new Result(false, "添加工单失败: 问题单号重复");
            }

            workOrder = workOrderRepository.detailByName(name);
            if (null != workOrder) {
                return new Result(false, "添加工单失败: 问题标题重复");
            }
            WorkOrder order = new WorkOrder();
            BeanUtils.copyProperties(orderParams, order);
            Long tenantId = context.getTenant();
            Tenant tenant = smsTenantService.detail(tenantId);
            Assert.notNull(tenant, "当前用户所属租户不存在");
            order.setCreatorId(context.getTarget());
            order.setTenantId(context.getTenant());
            order.setGmtCreate(new Date());
            order.setStatus(WorkOrderStatus.submitted.name());
            workOrderRepository.save(order);
            InteractiveRecord record = new InteractiveRecord(order.getContent(), order.getId(), context.getTarget(),
                    context.getTenant(), new Date(), context.getCatalog().name());
            recordRepository.save(record);
            return new Result(true, "添加工单成功");
        } catch (Exception e) {
            logger.error("Create order fail：", e);
            return new Result(false, "添加工单失败:" + e.getMessage());
        }
    }

    @Override
    public Result remove(Long id, RequestContext context) {
        String path = WorkOrder.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            WorkOrder order = workOrderRepository.query(id);
            if (null == order) {
                logger.warn("order data not exist");
                return new Result(false, "数据不存在");
            }
            boolean flag = workOrderRepository.remove(id, context.getTarget(), context.getTenant());
            return flag ? new Result(true, "删除工单成功") : new Result(false, "删除工单失败");
        } catch (Exception e) {
            logger.error("remove order fail:", e);
            return new Result(false, "删除工单失败");
        }
    }

    @Override
    public BocloudResult<WorkOrder> detail(Long id) {
        try {
            WorkOrder order = workOrderRepository.query(id);
            if (null == order) {
                return new BocloudResult<>(false, "工单不存在");
            }
            return new BocloudResult<>(true, order, "查询工单详情成功");
        } catch (Exception e) {
            logger.error("Query order detail fail:", e);
            return new BocloudResult<>(false, "查询工单详情失败");
        }
    }

    @Override
    public BocloudResult<GridBean<WorkOrder>> list(Pager pager, RequestContext context) {
        try {
            List<WorkOrder> list;
            int total;
            GridBean<WorkOrder> gridBean;
            Map<String, String> sorter = Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap());
            sorter.put("gmtCreate", Common.ONE);
            List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
            //自服务端登录 筛选自己的
            if (null != context.getTenant() && 0L != context.getTenant()) {
                if (RequestContext.Catalog.Tenant.equals(context.getCatalog())) {
                    params.add(new Param(MapTools.simpleMap("tenantId", context.getTenant()), Sign.EQ));
                } else {
                    Map<String, Object> paramsMap = MapTools.simpleMap("tenantId", context.getTenant());
                    paramsMap.put("creatorId", context.getTarget());
                    params.add(new Param(paramsMap, Sign.EQ));
                }
            }
            total = workOrderRepository.count(params);
            if (pager.getSimple()) {
                List<SimpleBean> beans = workOrderRepository.list(params, sorter);
                gridBean = new GridBean(1, 1, total, beans);
            } else {
                list = workOrderRepository.list(pager.getPage(), pager.getRows(), params, sorter);
                gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            }
            return new BocloudResult<>(true, gridBean, "查询工单成功");
        } catch (Exception e) {
            logger.error("list order fail:", e);
            return new BocloudResult<>(false, "查询工单失败");
        }
    }

    /**
     * 工单概况
     *
     * @param tenantId
     * @return
     * @throws Exception
     */
    @Override
    public BocloudResult<List<StatsResult>> orderPie(Long tenantId) {
        try {

            List<WorkOrder> orders = workOrderRepository.listByTenant(tenantId);
            // 待处理
            Integer submitted = 0;
            // 处理中
            Integer feedback = 0;
            // 已处理
            Integer closed = 0;
            for (WorkOrder order : orders) {
                if (WorkOrderStatus.submitted.name().equals(order.getStatus())) {
                    submitted++;
                } else if (WorkOrderStatus.feedback.name().equals(order.getStatus())) {
                    feedback++;
                } else if (WorkOrderStatus.closed.name().equals(order.getStatus())) {
                    closed++;
                }
            }
            List<StatsResult> result = Lists.newArrayList();
            result.add(new StatsResult("已处理", closed));
            result.add(new StatsResult("处理中", feedback));
            result.add(new StatsResult("待处理", submitted));
            return new BocloudResult<>(true, result, "查询成功");
        } catch (Exception e) {
            logger.error("get work order pie error,", e);
            return new BocloudResult<>(false, "查询失败:" + e.getMessage());
        }
    }

    @Override
    public Result createRecord(Long id, InteractiveRecord record, RequestContext context) {
        try {
            WorkOrder order = workOrderRepository.query(id);
            if (null == order) {
                return new Result(false, "工单不存在");
            }
            record.setUserId(context.getTarget());
            record.setTenantId(context.getTenant());
            record.setCatalog(context.getCatalog().name());
            record.setGmtCreate(null == record.getGmtCreate() ? new Date() : record.getGmtCreate());
            String portrait;
            if (context.getCatalog() == RequestContext.Catalog.Tenant) {
                Tenant tenant = smsTenantService.detail(context.getTarget());
                Assert.notNull(tenant, "租户信息不存在");
                portrait = tenant.getPortrait();
            } else {
                BocloudResult<User> userResult = smsUserService.detail(context.getTarget());
                Assert.isTrue(userResult.isSuccess(), "当前用户不存在");
                User user = userResult.getData();
                Assert.notNull(user, "用户信息不存在");
                portrait = user.getPortrait();
            }
            record.setPortrait(portrait);
            recordRepository.save(record);
            order.setStatus(WorkOrderStatus.feedback.name());
            order.setMenderId(context.getTarget());
            order.setGmtModify(new Date());
            workOrderRepository.update(order);
            return new Result(true, "添加沟通记录成功");
        } catch (Exception e) {
            logger.error("Create order record fail：", e);
            return new Result(false, "添加沟通记录失败");
        }
    }

    @Override
    public BocloudResult<List<InteractiveRecord>> records(Long orderId) {
        try {
            List<InteractiveRecord> records = recordRepository.listByOrderId(orderId);
            for (InteractiveRecord record : records) {
                String catalog = record.getCatalog();
                if ("Tenant".equals(catalog)) {
                    Tenant tenant = smsTenantService.detail(record.getTenantId());
                    if (null != tenant) {
                        record.setUserName(tenant.getName());
                        record.setPortrait(tenant.getPortrait());
                    }
                } else {
                    BocloudResult<User> userResult = smsUserService.detail(record.getUserId());
                    if (userResult.isSuccess()) {
                        User user = userResult.getData();
                        if (null != user) {
                            record.setUserName(user.getName());
                            record.setPortrait(user.getPortrait());
                        }
                    }
                }
            }
            return new BocloudResult<>(true, records, "查询沟通记录成功");
        } catch (Exception e) {
            logger.error("Query order record fail:", e);
            return new BocloudResult<>(false, "查询沟通记录失败");
        }
    }

    @Override
    public Result modify(Long id, WorkOrderParams order, RequestContext context) {
        String path = WorkOrder.class.getSimpleName() + "_" + id;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            WorkOrder oldOrder = workOrderRepository.query(id);
            if (null == oldOrder) {
                logger.warn("order data not exist");
                return new Result(false, "数据不存在");
            }
            if (null != order.getStatus()) {
                oldOrder.setStatus(order.getStatus());
                oldOrder.setRemark(order.getRemark());
                oldOrder.setStar(order.getStar());
                boolean flag = workOrderRepository.update(oldOrder);
                if (!flag) {
                    return new Result(false, "关闭失败");
                }
                return new Result(true, "关闭成功");
            } else {
                oldOrder.setCatalog(order.getCatalog());
                oldOrder.setResourceCatalog(order.getResourceCatalog());
                oldOrder.setResourceCategory(order.getResourceCategory());
                oldOrder.setOrderItemSn(order.getOrderItemSn());
                oldOrder.setName(order.getName());
                oldOrder.setContent(order.getContent());
                oldOrder.setMenderId(context.getTarget());
                oldOrder.setGmtModify(new Date());
                oldOrder.setInstanceUuid(order.getInstanceUuid());
                boolean flag = workOrderRepository.update(oldOrder);
                if (!flag) {
                    return new Result(false, "修改工单失败");
                }
                return new Result(true, "修改工单成功");
            }
        } catch (Exception e) {
            logger.error("Modify order fail:", e);
            return new Result(false, "修改工单失败");
        }
    }

    @Override
    public int count(List<Param> params) {
        return workOrderRepository.count(params);
    }
}
