package com.bocloud.cos.service.task;


import com.bocloud.common.enums.BoCloudExchange;
import com.bocloud.cos.model.NotifyMessage;
import com.bocloud.cos.internal.SmsSystemConfigService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;

/**
 * 抽象到期任务
 *
 * @author weiwei
 */
public abstract class AbstractExpiredTask {

    private SmsSystemConfigService smsSystemConfigService;
    private AmqpTemplate amqpTemplate;
    public static final Long ONE_DAY = 24 * 60 * 60 * 1000L;// 一天
    /**
     * 资源到期提前提醒时间
     */
    protected int resourceExpireRemindMillis = 3 * ONE_DAY.intValue();
    /**
     * 资源到期最长保留时长
     */
    protected int resourceExpireReservedMillis = 7 * ONE_DAY.intValue();
    /**
     * 消息通知队列
     */
    private static final String NOTICE_QUEUE = "event.notice.queue";

    /**
     *
     */
    public AbstractExpiredTask() {
    }

    /**
     * @param smsSystemConfigService
     * @param amqpTemplate
     */
    public AbstractExpiredTask(SmsSystemConfigService smsSystemConfigService, AmqpTemplate amqpTemplate) {
        super();
        this.smsSystemConfigService = smsSystemConfigService;
        this.amqpTemplate = amqpTemplate;
        this.initConfig();
    }

    /**
     * 初始化相关配置
     */
    private void initConfig() {
        String remindConfig = smsSystemConfigService.getByCode("ResourceExpireRemindDays");
        if (StringUtils.isNotEmpty(remindConfig)) {
            resourceExpireRemindMillis = Integer.valueOf(remindConfig) * ONE_DAY.intValue();
        }
        String reservedConfig = smsSystemConfigService.getByCode("ResourceExpireReservedDays");
        if (StringUtils.isNotEmpty(reservedConfig)) {
            resourceExpireReservedMillis = Integer.valueOf(reservedConfig) * ONE_DAY.intValue();
        }
    }

    /**
     * 按步执行资源到期或快到期相关处理
     */
    public void step() {
        boolean flag = this.preCheck();
        if (!flag) {
            return;
        }
        NotifyMessage message = this.process();
        if (null == message) {
            return;
        }
        // 发送消息通知用户
        this.sendMessage(message);
    }

    /**
     * 预检查是否已到期
     */
    public abstract boolean preCheck();

    /**
     * 处理已到期或即将到期任务
     *
     * @return
     */
    public abstract NotifyMessage process();

    /**
     * 发送消息通知用户
     *
     * @param message
     */
    private void sendMessage(NotifyMessage message) {
        amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(), NOTICE_QUEUE, message);
    }

}
