package com.bocloud.cos.utils;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.common.utils.ListTool;
import com.bocloud.cos.entity.OrderItem;
import com.bocloud.cos.model.ElementModel;
import com.bocloud.cos.model.QuotaBean;
import com.bocloud.cos.model.SpecModel;
import org.apache.commons.lang3.StringUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author tyl
 * @since 2021/6/10 18:30
 */
public class OrderItemUtils {

    /**
     * 计算订单项配额
     *
     * @param items
     * @return
     */
    public static Map<String, Integer> getItemQuota(List<OrderItem> items) {
        Map<String, Integer> quotaMap = new HashMap<>(8);
        int cpuTotal = 0;
        int memTotal = 0;
        int diskTotal = 0;
        for (OrderItem item : items) {
            List<ElementModel> elementModels = JSONArray.parseArray(item.getElements(), ElementModel.class);
            if (ListTool.isEmpty(elementModels)) {
                continue;
            }
            ElementModel elementModel = elementModels.get(0);
            Integer count = Optional.ofNullable(elementModel.getCount()).orElse(1);
            String serviceCode = elementModel.getServiceCode();
            if (quotaMap.containsKey(serviceCode)) {
                Integer total = quotaMap.get(serviceCode);
                quotaMap.put(serviceCode, total + 1);
            } else {
                quotaMap.put(serviceCode, 1);
            }
            List<SpecModel> specs = elementModel.getSpecs();
            if (null != specs && !specs.isEmpty()) {
                Integer cpu =
                        specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0)).reduce(Integer::sum)
                                .orElse(0);
                cpuTotal += cpu * count;
                BigDecimal mem =
                        specs.stream().map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                memTotal += mem.intValue() * count;
                Integer disk =
                        specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0)).reduce(Integer::sum)
                                .orElse(0);
                diskTotal += disk * count;
            }
        }
        if (cpuTotal != 0L) {
            quotaMap.put("meta.cpu", cpuTotal);
        }
        if (memTotal != 0L) {
            quotaMap.put("meta.mem", memTotal);
        }
        if (diskTotal != 0L) {
            quotaMap.put("meta.disk", diskTotal);
        }
        return quotaMap;
    }

    /**
     * 根据订购项获取规格Bean集合
     *
     * @param items
     * @return
     */
    public static List<QuotaBean> getItemQuotaBeans(List<OrderItem> items) {
        Map<String, Integer> quotaMap = getItemQuota(items);
        List<QuotaBean> beans = new ArrayList<>();
        for (String key : quotaMap.keySet()) {
            beans.add(new QuotaBean(key, quotaMap.get(key)));
        }
        return beans;
    }

    /**
     * 规格名称
     *
     * @param item
     * @return
     */
    public static void elementNames(OrderItem item) {
        String elements = item.getElements();
        List<String> resultList = new ArrayList<>();
        // 性能类型
        List<String> performanceList = new ArrayList<>();
        // 规格
        List<String> specList = new ArrayList<>();
        // 折扣列表
        List<String> discountList = new ArrayList<>();
        if (StringUtils.isNotEmpty(elements)) {
            List<ElementModel> elementModels = JSONArray.parseArray(elements, ElementModel.class);
            for (ElementModel model : elementModels) {
                // 具体规格信息
                List<SpecModel> specs = Optional.ofNullable(model.getSpecs()).orElse(new ArrayList<>());
                Integer cpu =
                        specs.stream().map(spec -> Optional.ofNullable(spec.getCpu()).orElse(0)).reduce(Integer::sum)
                                .orElse(0);
                BigDecimal memory =
                        specs.stream().map(spec -> Optional.ofNullable(spec.getMemory()).orElse(BigDecimal.ZERO))
                                .reduce(BigDecimal::add).orElse(BigDecimal.ZERO);
                Integer disk =
                        specs.stream().map(spec -> Optional.ofNullable(spec.getDisk()).orElse(0)).reduce(Integer::sum)
                                .orElse(0);
                if (ListTool.isEmpty(performanceList) && 0 != cpu) {
                    Integer count = model.getCount();
                    String version = "单机版";
                    if (null != count && count > 1) {
                        version = "集群版";
                        performanceList.add("类型：" + version);
                        performanceList.add("节点数：" + count);
                    } else {
                        performanceList.add("类型：" + version);
                    }

                }
                BigDecimal ratio = model.getRatio();
                String ratioStr;
                if (null != ratio) {
                    int i = ratio.multiply(BigDecimal.valueOf(10)).intValue();
                    if (i == 10) {
                        ratioStr = "无折扣";
                    } else if (i == 0) {
                        ratioStr = "免费";
                    } else {
                        ratioStr = i + "折";
                    }
                } else {
                    ratioStr = "--";
                }
                StringBuilder specNames = new StringBuilder();
                if (cpu != 0) {
                    //云主机有cpu、内存
                    specNames.append(cpu).append("C/").append(memory).append("GB");
                    specList.add("规格：" + specNames.toString());
                    discountList.add("实例折扣：" + ratioStr);
                } else if (disk != 0) {
                    //磁盘项只有磁盘大小
                    specNames.append(disk).append("GB");
                    specList.add("磁盘规格：" + specNames.toString());
                    discountList.add("磁盘折扣：" + ratioStr);
                } else {
                    specList.add("标准版");
                    discountList.add("折扣：" + ratioStr);
                }
            }
        }
        resultList.addAll(performanceList);
        resultList.addAll(specList);
        item.setElementList(resultList);
        item.setDiscountList(discountList);
    }
}
